import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["accessKey", "activeStateEnabled", "children", "className", "component", "disabled", "focusStateEnabled", "height", "hint", "hoverStateEnabled", "icon", "iconComponent", "iconPosition", "iconRender", "iconTemplate", "onClick", "onKeyDown", "onSubmit", "pressed", "render", "rtlEnabled", "stylingMode", "tabIndex", "template", "templateData", "text", "type", "useInkRipple", "useSubmitBehavior", "visible", "width"];
import { createDefaultOptionRules, convertRulesToOptions } from 'devextreme/core/options/utils';
import devices from 'devextreme/core/devices';
import { isMaterial, current } from 'devextreme/ui/themes';
import { click } from 'devextreme/events/short';
import { combineClasses } from "../utils/combine_classes";
import { getImageSourceType } from 'devextreme/core/utils/icon';
import { Icon } from "./common/icon";
import { InkRipple } from "./common/ink_ripple";
import { Widget } from "./common/widget";
import { BaseWidgetProps } from "./common/base_props";
var stylingModes = ["outlined", "text", "contained"];

var getCssClasses = model => {
  var {
    icon,
    iconPosition,
    stylingMode,
    text,
    type
  } = model;
  var isValidStylingMode = stylingMode && stylingModes.includes(stylingMode);
  var classesMap = {
    "dx-button": true,
    ["dx-button-mode-".concat(isValidStylingMode ? stylingMode : "contained")]: true,
    ["dx-button-".concat(type !== null && type !== void 0 ? type : "normal")]: true,
    "dx-button-has-text": !!text,
    "dx-button-has-icon": !!icon,
    "dx-button-icon-right": iconPosition !== "left"
  };
  return combineClasses(classesMap);
};

export var viewFunction = viewModel => {
  var {
    children,
    iconPosition,
    iconTemplate: IconTemplate,
    template: ButtonTemplate,
    text
  } = viewModel.props;
  var renderText = !viewModel.props.template && !children && text !== "";
  var isIconLeft = iconPosition === "left";
  var iconComponent = !viewModel.props.template && !children && (viewModel.iconSource || viewModel.props.iconTemplate) && React.createElement(Icon, {
    source: viewModel.iconSource,
    position: iconPosition,
    iconTemplate: IconTemplate
  });
  return React.createElement(Widget, _extends({
    ref: viewModel.widgetRef,
    accessKey: viewModel.props.accessKey,
    activeStateEnabled: viewModel.props.activeStateEnabled,
    aria: viewModel.aria,
    className: viewModel.props.className,
    classes: viewModel.cssClasses,
    disabled: viewModel.props.disabled,
    focusStateEnabled: viewModel.props.focusStateEnabled,
    height: viewModel.props.height,
    hint: viewModel.props.hint,
    hoverStateEnabled: viewModel.props.hoverStateEnabled,
    onActive: viewModel.onActive,
    onClick: viewModel.onWidgetClick,
    onInactive: viewModel.onInactive,
    onKeyDown: viewModel.keyDown,
    rtlEnabled: viewModel.props.rtlEnabled,
    tabIndex: viewModel.props.tabIndex,
    visible: viewModel.props.visible,
    width: viewModel.props.width
  }, viewModel.restAttributes), React.createElement("div", {
    className: "dx-button-content",
    ref: viewModel.contentRef
  }, viewModel.props.template && ButtonTemplate({
    data: viewModel.buttonTemplateData
  }), !viewModel.props.template && children, isIconLeft && iconComponent, renderText && React.createElement("span", {
    className: "dx-button-text"
  }, text), !isIconLeft && iconComponent, viewModel.props.useSubmitBehavior && React.createElement("input", {
    ref: viewModel.submitInputRef,
    type: "submit",
    tabIndex: -1,
    className: "dx-button-submit-input"
  }), viewModel.props.useInkRipple && React.createElement(InkRipple, {
    config: viewModel.inkRippleConfig,
    ref: viewModel.inkRippleRef
  })));
};
export var ButtonProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(BaseWidgetProps), Object.getOwnPropertyDescriptors({
  activeStateEnabled: true,
  hoverStateEnabled: true,
  icon: "",
  iconPosition: "left",
  stylingMode: "contained",
  text: "",
  type: "normal",
  useInkRipple: false,
  useSubmitBehavior: false,

  get templateData() {
    return {};
  }

})));
export var defaultOptionRules = createDefaultOptionRules([{
  device: () => devices.real().deviceType === "desktop" && !devices.isSimulator(),
  options: {
    focusStateEnabled: true
  }
}, {
  device: () => isMaterial(current()),
  options: {
    useInkRipple: true
  }
}]);
import * as React from "react";
import { useCallback, useMemo, useEffect, useRef, useImperativeHandle, forwardRef } from "react";
import { getTemplate } from "@devextreme/runtime/react";
var Button = forwardRef(function button(props, ref) {
  var __contentRef = useRef(null);

  var __submitInputRef = useRef(null);

  var __inkRippleRef = useRef(null);

  var __widgetRef = useRef(null);

  var __onActive = useCallback(function __onActive(event) {
    var {
      useInkRipple
    } = props;
    useInkRipple && __inkRippleRef.current.showWave({
      element: __contentRef.current,
      event
    });
  }, [props.useInkRipple]);

  var __onInactive = useCallback(function __onInactive(event) {
    var {
      useInkRipple
    } = props;
    useInkRipple && __inkRippleRef.current.hideWave({
      element: __contentRef.current,
      event
    });
  }, [props.useInkRipple]);

  var __onWidgetClick = useCallback(function __onWidgetClick(event) {
    var {
      onClick,
      useSubmitBehavior
    } = props;
    onClick === null || onClick === void 0 ? void 0 : onClick({
      event
    });
    useSubmitBehavior && __submitInputRef.current.click();
  }, [props.onClick, props.useSubmitBehavior]);

  var __aria = useCallback(function __aria() {
    var {
      icon,
      text
    } = props;
    var label = (text !== null && text !== void 0 ? text : "") || icon;

    if (!text && icon && getImageSourceType(icon) === "image") {
      label = !icon.includes("base64") ? icon.replace(/.+\/([^.]+)\..+$/, "$1") : "Base64";
    }

    return _extends({
      role: "button"
    }, label ? {
      label
    } : {});
  }, [props.icon, props.text]);

  var __cssClasses = useCallback(function __cssClasses() {
    return getCssClasses(props);
  }, [props]);

  var __iconSource = useCallback(function __iconSource() {
    var {
      icon,
      type
    } = props;

    if (icon || type === "back") {
      return (icon !== null && icon !== void 0 ? icon : "") || "back";
    }

    return "";
  }, [props.icon, props.type]);

  var __inkRippleConfig = useMemo(function __inkRippleConfig() {
    var {
      icon,
      text,
      type
    } = props;
    return !text && icon || type === "back" ? {
      isCentered: true,
      useHoldAnimation: false,
      waveSizeCoefficient: 1
    } : {};
  }, [props.icon, props.text, props.type]);

  var __buttonTemplateData = useCallback(function __buttonTemplateData() {
    var {
      icon,
      templateData,
      text
    } = props;
    return _extends({
      icon,
      text
    }, templateData);
  }, [props.icon, props.templateData, props.text]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __focus = useCallback(function __focus() {
    __widgetRef.current.focus();
  }, []);

  var __activate = useCallback(function __activate() {
    __widgetRef.current.activate();
  }, []);

  var __deactivate = useCallback(function __deactivate() {
    __widgetRef.current.deactivate();
  }, []);

  var __keyDown = useCallback(function __keyDown(e) {
    var {
      onKeyDown
    } = props;
    var {
      keyName,
      originalEvent,
      which
    } = e;
    var result = onKeyDown === null || onKeyDown === void 0 ? void 0 : onKeyDown(e);

    if (result !== null && result !== void 0 && result.cancel) {
      return result;
    }

    if (keyName === "space" || which === "space" || keyName === "enter" || which === "enter") {
      originalEvent.preventDefault();

      __onWidgetClick(originalEvent);
    }

    return undefined;
  }, [props.onKeyDown, __onWidgetClick]);

  useEffect(() => {
    var namespace = "UIFeedback";
    var {
      onSubmit,
      useSubmitBehavior
    } = props;

    if (useSubmitBehavior && onSubmit) {
      click.on(__submitInputRef.current, event => onSubmit({
        event,
        submitInput: __submitInputRef.current
      }), {
        namespace
      });
      return () => click.off(__submitInputRef.current, {
        namespace
      });
    }

    return undefined;
  }, [props.onSubmit, props.useSubmitBehavior]);
  useImperativeHandle(ref, () => ({
    focus: __focus,
    activate: __activate,
    deactivate: __deactivate
  }), [__focus, __activate, __deactivate]);
  return viewFunction({
    props: _extends({}, props, {
      template: getTemplate(props.template, props.render, props.component),
      iconTemplate: getTemplate(props.iconTemplate, props.iconRender, props.iconComponent)
    }),
    contentRef: __contentRef,
    submitInputRef: __submitInputRef,
    inkRippleRef: __inkRippleRef,
    widgetRef: __widgetRef,
    onActive: __onActive,
    onInactive: __onInactive,
    onWidgetClick: __onWidgetClick,
    keyDown: __keyDown,
    aria: __aria(),
    cssClasses: __cssClasses(),
    iconSource: __iconSource(),
    inkRippleConfig: __inkRippleConfig,
    buttonTemplateData: __buttonTemplateData(),
    restAttributes: __restAttributes()
  });
});
export { Button };
export default Button;
Button.defaultProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(ButtonProps), Object.getOwnPropertyDescriptors(_extends({}, convertRulesToOptions(defaultOptionRules)))));
var __defaultOptionRules = [];
export function defaultOptions(rule) {
  __defaultOptionRules.push(rule);

  Button.defaultProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(Button.defaultProps), Object.getOwnPropertyDescriptors(convertRulesToOptions(defaultOptionRules)), Object.getOwnPropertyDescriptors(convertRulesToOptions(__defaultOptionRules))));
}