import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["valueChange"],
    _excluded2 = ["componentProps", "componentType", "rootElementRef", "templateNames"];
import { renderTemplate, hasTemplate } from "@devextreme/runtime/react";
import { ConfigContext } from "../../common/config_context";
import { getUpdatedOptions } from "./utils/get_updated_options";

var normalizeProps = props => Object.keys(props).reduce((accumulator, key) => {
  if (props[key] !== undefined) {
    accumulator[key] = props[key];
  }

  return accumulator;
}, {});

export var viewFunction = _ref => {
  var {
    props: {
      componentProps: {
        className
      }
    },
    restAttributes,
    widgetRef
  } = _ref;
  return React.createElement("div", _extends({
    ref: widgetRef,
    className: className
  }, restAttributes));
};
export var DomComponentWrapperProps = {};
import * as React from "react";
import { useContext, useCallback, useEffect, useRef, useImperativeHandle, forwardRef } from "react";
var DomComponentWrapper = forwardRef(function domComponentWrapper(props, ref) {
  var __widgetRef = useRef(null);

  var instance = useRef();
  var prevProps = useRef();
  var config = useContext(ConfigContext);

  var __properties = useCallback(function __properties() {
    var normalizedProps = normalizeProps(props.componentProps);

    var {
      valueChange
    } = normalizedProps,
        restProps = _objectWithoutPropertiesLoose(normalizedProps, _excluded);

    var properties = _extends({
      rtlEnabled: !!(config !== null && config !== void 0 && config.rtlEnabled),
      isRenovated: true
    }, restProps);

    if (valueChange) {
      properties.onValueChanged = _ref2 => {
        var {
          value
        } = _ref2;
        return valueChange(value);
      };
    }

    var templates = props.templateNames;
    templates.forEach(name => {
      if (hasTemplate(name, properties, this)) {
        properties[name] = (item, index, container) => {
          renderTemplate(props.componentProps[name], {
            item,
            index,
            container
          }, this);
        };
      }
    });
    return properties;
  }, [props.componentProps, config, props.templateNames]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded2);

    return restProps;
  }, [props]);

  var __getInstance = useCallback(function __getInstance() {
    return instance.current;
  }, [instance]);

  useEffect(() => {
    var componentInstance = new props.componentType(__widgetRef.current, __properties());
    instance.current = componentInstance;
    return () => {
      componentInstance.dispose();
      instance.current = null;
    };
  }, []);
  useEffect(() => {
    var {
      rootElementRef
    } = props;

    if (rootElementRef) {
      rootElementRef.current = __widgetRef.current;
    }
  }, []);
  useEffect(() => {
    var instance = __getInstance();

    if (!instance) {
      return;
    }

    var updatedOptions = getUpdatedOptions(prevProps.current || {}, __properties());

    if (updatedOptions.length) {
      instance.beginUpdate();
      updatedOptions.forEach(_ref3 => {
        var {
          path,
          value
        } = _ref3;
        instance.option(path, value);
      });
      instance.endUpdate();
    }

    prevProps.current = __properties();
  }, [__getInstance, prevProps, __properties]);
  useImperativeHandle(ref, () => ({
    getInstance: __getInstance
  }), [__getInstance]);
  return viewFunction({
    props: _extends({}, props),
    widgetRef: __widgetRef,
    config,
    properties: __properties(),
    restAttributes: __restAttributes()
  });
});
export { DomComponentWrapper };
export default DomComponentWrapper;
DomComponentWrapper.defaultProps = DomComponentWrapperProps;