import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["_feedbackHideTimeout", "_feedbackShowTimeout", "accessKey", "activeStateEnabled", "activeStateUnit", "addWidgetClass", "aria", "children", "className", "classes", "cssText", "disabled", "focusStateEnabled", "height", "hint", "hoverStateEnabled", "name", "onActive", "onClick", "onDimensionChanged", "onFocusIn", "onFocusOut", "onHoverEnd", "onHoverStart", "onInactive", "onKeyDown", "onRootElementRendered", "onVisibilityChange", "rootElementRef", "rtlEnabled", "tabIndex", "visible", "width"];
import 'devextreme/events/click';
import 'devextreme/events/hover';
import { isFunction } from 'devextreme/core/utils/type';
import { active, dxClick, focus, hover, keyboard, resize, visibility } from 'devextreme/events/short';
import { combineClasses } from "../../utils/combine_classes";
import { extend } from 'devextreme/core/utils/extend';
import { focusable } from 'devextreme/ui/widget/selectors';
import { normalizeStyleProp } from 'devextreme/core/utils/style';
import { BaseWidgetProps } from "./base_props";
import { ConfigContext } from "../../common/config_context";
import { ConfigProvider } from "../../common/config_provider";
import { resolveRtlEnabled, resolveRtlEnabledDefinition } from "../../utils/resolve_rtl";
import resizeCallbacks from 'devextreme/core/utils/resize_callbacks';
import errors from 'devextreme/core/errors';
import domAdapter from 'devextreme/core/dom_adapter';
var DEFAULT_FEEDBACK_HIDE_TIMEOUT = 400;
var DEFAULT_FEEDBACK_SHOW_TIMEOUT = 30;

var getAria = args => Object.keys(args).reduce((r, key) => {
  if (args[key]) {
    return _extends({}, r, {
      [key === "role" || key === "id" ? key : "aria-".concat(key)]: String(args[key])
    });
  }

  return r;
}, {});

export var viewFunction = viewModel => {
  var widget = React.createElement("div", _extends({
    ref: viewModel.widgetElementRef
  }, viewModel.attributes, {
    tabIndex: viewModel.tabIndex,
    title: viewModel.props.hint,
    hidden: !viewModel.props.visible,
    className: viewModel.cssClasses,
    style: normalizeStyles(viewModel.styles)
  }), viewModel.props.children);
  return viewModel.shouldRenderConfigProvider ? React.createElement(ConfigProvider, {
    rtlEnabled: viewModel.rtlEnabled
  }, widget) : widget;
};
export var WidgetProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(BaseWidgetProps), Object.getOwnPropertyDescriptors({
  _feedbackHideTimeout: DEFAULT_FEEDBACK_HIDE_TIMEOUT,
  _feedbackShowTimeout: DEFAULT_FEEDBACK_SHOW_TIMEOUT,
  cssText: "",

  get aria() {
    return {};
  },

  classes: "",
  name: "",
  addWidgetClass: true
})));
import * as React from "react";
import { useState, useContext, useCallback, useEffect, useRef, useImperativeHandle, forwardRef } from "react";
var NUMBER_STYLES = new Set(["animationIterationCount", "borderImageOutset", "borderImageSlice", "border-imageWidth", "boxFlex", "boxFlexGroup", "boxOrdinalGroup", "columnCount", "fillOpacity", "flex", "flexGrow", "flexNegative", "flexOrder", "flexPositive", "flexShrink", "floodOpacity", "fontWeight", "gridColumn", "gridRow", "lineClamp", "lineHeight", "opacity", "order", "orphans", "stopOpacity", "strokeDasharray", "strokeDashoffset", "strokeMiterlimit", "strokeOpacity", "strokeWidth", "tabSize", "widows", "zIndex", "zoom"]);

var isNumeric = value => {
  if (typeof value === "number") return true;
  return !isNaN(Number(value));
};

var getNumberStyleValue = (style, value) => {
  return NUMBER_STYLES.has(style) ? value : "".concat(value, "px");
};

var normalizeStyles = styles => {
  if (!(styles instanceof Object)) return undefined;
  return Object.entries(styles).reduce((result, _ref) => {
    var [key, value] = _ref;
    result[key] = isNumeric(value) ? getNumberStyleValue(key, value) : value;
    return result;
  }, {});
};

var Widget = forwardRef(function widget(props, ref) {
  var __widgetElementRef = useRef(null);

  var [__state_active, __state_setActive] = useState(false);
  var [__state_focused, __state_setFocused] = useState(false);
  var [__state_hovered, __state_setHovered] = useState(false);
  var config = useContext(ConfigContext);

  var __shouldRenderConfigProvider = useCallback(function __shouldRenderConfigProvider() {
    var {
      rtlEnabled
    } = props;
    return resolveRtlEnabledDefinition(rtlEnabled, config);
  }, [props.rtlEnabled, config]);

  var __rtlEnabled = useCallback(function __rtlEnabled() {
    var {
      rtlEnabled
    } = props;
    return resolveRtlEnabled(rtlEnabled, config);
  }, [props.rtlEnabled, config]);

  var __tabIndex = useCallback(function __tabIndex() {
    var {
      disabled,
      focusStateEnabled,
      tabIndex
    } = props;
    var isFocusable = focusStateEnabled && !disabled;
    return isFocusable ? tabIndex : undefined;
  }, [props.disabled, props.focusStateEnabled, props.tabIndex]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __focus = useCallback(function __focus() {
    focus.trigger(__widgetElementRef.current);
  }, []);

  var __blur = useCallback(function __blur() {
    var activeElement = domAdapter.getActiveElement();

    if (__widgetElementRef.current === activeElement) {
      activeElement.blur();
    }
  }, []);

  var __activate = useCallback(function __activate() {
    __state_setActive(__state_active => true);
  }, []);

  var __deactivate = useCallback(function __deactivate() {
    __state_setActive(__state_active => false);
  }, []);

  var __attributes = useCallback(function __attributes() {
    var {
      aria,
      disabled,
      focusStateEnabled,
      visible
    } = props;
    var accessKey = focusStateEnabled && !disabled && props.accessKey;
    return _extends({}, extend({}, __restAttributes(), accessKey && {
      accessKey
    }), getAria(_extends({}, aria, {
      disabled,
      hidden: !visible
    })));
  }, [props.aria, props.disabled, props.focusStateEnabled, props.visible, props.accessKey, __restAttributes]);

  var __styles = useCallback(function __styles() {
    var {
      height,
      width
    } = props;
    var style = __restAttributes().style || {};
    var computedWidth = normalizeStyleProp("width", isFunction(width) ? width() : width);
    var computedHeight = normalizeStyleProp("height", isFunction(height) ? height() : height);
    return _extends({}, style, {
      height: computedHeight !== null && computedHeight !== void 0 ? computedHeight : style.height,
      width: computedWidth !== null && computedWidth !== void 0 ? computedWidth : style.width
    });
  }, [props.height, props.width, __restAttributes]);

  var __cssClasses = useCallback(function __cssClasses() {
    var {
      activeStateEnabled,
      addWidgetClass,
      className,
      classes,
      disabled,
      focusStateEnabled,
      hoverStateEnabled,
      onVisibilityChange,
      visible
    } = props;
    var isFocusable = !!focusStateEnabled && !disabled;
    var isHoverable = !!hoverStateEnabled && !disabled;
    var canBeActive = !!activeStateEnabled && !disabled;
    var classesMap = {
      "dx-widget": !!addWidgetClass,
      [String(classes)]: !!classes,
      [String(className)]: !!className,
      "dx-state-disabled": !!disabled,
      "dx-state-invisible": !visible,
      "dx-state-focused": !!__state_focused && isFocusable,
      "dx-state-active": !!__state_active && canBeActive,
      "dx-state-hover": !!__state_hovered && isHoverable && !__state_active,
      "dx-rtl": !!__rtlEnabled(),
      "dx-visibility-change-handler": !!onVisibilityChange
    };
    return combineClasses(classesMap);
  }, [props.activeStateEnabled, props.addWidgetClass, props.className, props.classes, props.disabled, props.focusStateEnabled, props.hoverStateEnabled, props.onVisibilityChange, props.visible, __state_focused, __state_active, __state_hovered, __rtlEnabled]);

  useEffect(() => {
    var {
      onRootElementRendered,
      rootElementRef
    } = props;

    if (rootElementRef) {
      rootElementRef.current = __widgetElementRef.current;
    }

    onRootElementRendered === null || onRootElementRendered === void 0 ? void 0 : onRootElementRendered(__widgetElementRef.current);
  }, []);
  useEffect(() => {
    var {
      _feedbackHideTimeout,
      _feedbackShowTimeout,
      activeStateEnabled,
      activeStateUnit,
      disabled,
      onActive,
      onInactive
    } = props;
    var namespace = "UIFeedback";
    var selector = activeStateUnit;

    if (activeStateEnabled) {
      if (disabled) {
        __state_setActive(__state_active => false);
      } else {
        active.on(__widgetElementRef.current, _ref2 => {
          var {
            event
          } = _ref2;

          __state_setActive(__state_active => true);

          onActive === null || onActive === void 0 ? void 0 : onActive(event);
        }, _ref3 => {
          var {
            event
          } = _ref3;

          __state_setActive(__state_active => false);

          onInactive === null || onInactive === void 0 ? void 0 : onInactive(event);
        }, {
          hideTimeout: _feedbackHideTimeout,
          namespace,
          selector,
          showTimeout: _feedbackShowTimeout
        });
        return () => active.off(__widgetElementRef.current, {
          selector,
          namespace
        });
      }
    }

    return undefined;
  }, [props._feedbackHideTimeout, props._feedbackShowTimeout, props.activeStateEnabled, props.activeStateUnit, props.disabled, props.onActive, props.onInactive]);
  useEffect(() => {
    var {
      disabled,
      name,
      onClick
    } = props;
    var namespace = name;

    if (onClick && !disabled) {
      dxClick.on(__widgetElementRef.current, onClick, {
        namespace
      });
      return () => dxClick.off(__widgetElementRef.current, {
        namespace
      });
    }

    return undefined;
  }, [props.disabled, props.name, props.onClick]);
  useEffect(() => {
    var {
      disabled,
      focusStateEnabled,
      name,
      onFocusIn,
      onFocusOut
    } = props;
    var namespace = "".concat(name, "Focus");

    if (focusStateEnabled) {
      if (disabled) {
        __state_setFocused(__state_focused => false);
      } else {
        focus.on(__widgetElementRef.current, e => {
          if (!e.isDefaultPrevented()) {
            __state_setFocused(__state_focused => true);

            onFocusIn === null || onFocusIn === void 0 ? void 0 : onFocusIn(e);
          }
        }, e => {
          if (!e.isDefaultPrevented()) {
            __state_setFocused(__state_focused => false);

            onFocusOut === null || onFocusOut === void 0 ? void 0 : onFocusOut(e);
          }
        }, {
          isFocusable: focusable,
          namespace
        });
        return () => focus.off(__widgetElementRef.current, {
          namespace
        });
      }
    }

    return undefined;
  }, [props.disabled, props.focusStateEnabled, props.name, props.onFocusIn, props.onFocusOut]);
  useEffect(() => {
    var {
      activeStateUnit,
      disabled,
      hoverStateEnabled,
      onHoverEnd,
      onHoverStart
    } = props;
    var namespace = "UIFeedback";
    var selector = activeStateUnit;

    if (hoverStateEnabled) {
      if (disabled) {
        __state_setHovered(__state_hovered => false);
      } else {
        hover.on(__widgetElementRef.current, _ref4 => {
          var {
            event
          } = _ref4;
          !__state_active && __state_setHovered(__state_hovered => true);
          onHoverStart === null || onHoverStart === void 0 ? void 0 : onHoverStart(event);
        }, event => {
          __state_setHovered(__state_hovered => false);

          onHoverEnd === null || onHoverEnd === void 0 ? void 0 : onHoverEnd(event);
        }, {
          selector,
          namespace
        });
        return () => hover.off(__widgetElementRef.current, {
          selector,
          namespace
        });
      }
    }

    return undefined;
  }, [props.activeStateUnit, props.disabled, props.hoverStateEnabled, props.onHoverEnd, props.onHoverStart, __state_active]);
  useEffect(() => {
    var {
      focusStateEnabled,
      onKeyDown
    } = props;

    if (focusStateEnabled && onKeyDown) {
      var id = keyboard.on(__widgetElementRef.current, __widgetElementRef.current, e => onKeyDown(e));
      return () => keyboard.off(id);
    }

    return undefined;
  }, [props.focusStateEnabled, props.onKeyDown]);
  useEffect(() => {
    var namespace = "".concat(props.name, "VisibilityChange");
    var {
      onDimensionChanged
    } = props;

    if (onDimensionChanged) {
      resize.on(__widgetElementRef.current, onDimensionChanged, {
        namespace
      });
      return () => resize.off(__widgetElementRef.current, {
        namespace
      });
    }

    return undefined;
  }, [props.name, props.onDimensionChanged]);
  useEffect(() => {
    var {
      onDimensionChanged
    } = props;

    if (onDimensionChanged) {
      resizeCallbacks.add(onDimensionChanged);
      return () => {
        resizeCallbacks.remove(onDimensionChanged);
      };
    }

    return undefined;
  }, [props.onDimensionChanged]);
  useEffect(() => {
    var {
      name,
      onVisibilityChange
    } = props;
    var namespace = "".concat(name, "VisibilityChange");

    if (onVisibilityChange) {
      visibility.on(__widgetElementRef.current, () => onVisibilityChange(true), () => onVisibilityChange(false), {
        namespace
      });
      return () => visibility.off(__widgetElementRef.current, {
        namespace
      });
    }

    return undefined;
  }, [props.name, props.onVisibilityChange]);
  useEffect(() => {
    var {
      height,
      width
    } = props;

    if (isFunction(width)) {
      errors.log("W0017", "width");
    }

    if (isFunction(height)) {
      errors.log("W0017", "height");
    }
  }, [props.height, props.width]);
  useEffect(() => {
    var {
      cssText
    } = props;

    if (cssText !== "") {
      __widgetElementRef.current.style.cssText = cssText;
    }
  }, [props.cssText]);
  useImperativeHandle(ref, () => ({
    focus: __focus,
    blur: __blur,
    activate: __activate,
    deactivate: __deactivate
  }), [__focus, __blur, __activate, __deactivate]);
  return viewFunction({
    props: _extends({}, props),
    active: __state_active,
    focused: __state_focused,
    hovered: __state_hovered,
    widgetElementRef: __widgetElementRef,
    config,
    shouldRenderConfigProvider: __shouldRenderConfigProvider(),
    rtlEnabled: __rtlEnabled(),
    attributes: __attributes(),
    styles: __styles(),
    cssClasses: __cssClasses(),
    tabIndex: __tabIndex(),
    restAttributes: __restAttributes()
  });
});
export { Widget };
export default Widget;
Widget.defaultProps = WidgetProps;