import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["accessKey", "activeStateEnabled", "aria", "className", "defaultValue", "disabled", "focusStateEnabled", "height", "hint", "hoverStateEnabled", "iconSize", "isValid", "name", "onClick", "onFocusIn", "onKeyDown", "readOnly", "rtlEnabled", "saveValueChangeEvent", "tabIndex", "text", "validationError", "validationErrors", "validationMessageMode", "validationStatus", "value", "valueChange", "visible", "width"];
import devices from 'devextreme/core/devices';
import { Editor, EditorProps } from "../internal/editor";
import { combineClasses } from "../../../utils/combine_classes";
import { CheckBoxIcon } from "./check_box_icon";
import { WidgetProps } from "../../common/widget";

var getCssClasses = model => {
  var {
    text,
    value
  } = model;
  var checked = value;
  var indeterminate = checked === null;
  var classesMap = {
    "dx-checkbox": true,
    "dx-checkbox-checked": checked === true,
    "dx-checkbox-has-text": !!text,
    "dx-checkbox-indeterminate": indeterminate
  };
  return combineClasses(classesMap);
};

export var viewFunction = viewModel => {
  var {
    aria,
    cssClasses: classes,
    editorRef,
    keyDown: onKeyDown,
    onWidgetClick: onClick,
    props: {
      accessKey,
      activeStateEnabled,
      className,
      disabled,
      focusStateEnabled,
      height,
      hint,
      hoverStateEnabled,
      iconSize,
      isValid,
      name,
      onFocusIn,
      readOnly,
      rtlEnabled,
      tabIndex,
      text,
      validationError,
      validationErrors,
      validationMessageMode,
      validationStatus,
      value,
      visible,
      width
    },
    restAttributes
  } = viewModel;
  return React.createElement(Editor, _extends({
    ref: editorRef,
    aria: aria,
    classes: classes,
    onClick: onClick,
    onKeyDown: onKeyDown,
    accessKey: accessKey,
    activeStateEnabled: activeStateEnabled,
    focusStateEnabled: focusStateEnabled,
    hoverStateEnabled: hoverStateEnabled,
    className: className,
    disabled: disabled,
    readOnly: readOnly,
    hint: hint,
    height: height,
    width: width,
    rtlEnabled: rtlEnabled,
    tabIndex: tabIndex,
    visible: visible,
    validationError: validationError,
    validationErrors: validationErrors,
    validationMessageMode: validationMessageMode,
    validationStatus: validationStatus,
    isValid: isValid,
    onFocusIn: onFocusIn
  }, restAttributes), React.createElement(React.Fragment, null, React.createElement("input", _extends({
    type: "hidden",
    value: "".concat(value)
  }, name && {
    name
  })), React.createElement("div", {
    className: "dx-checkbox-container"
  }, React.createElement(CheckBoxIcon, {
    size: iconSize,
    isChecked: value === true
  }), text && React.createElement("span", {
    className: "dx-checkbox-text"
  }, text))));
};
export var CheckBoxProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(EditorProps), Object.getOwnPropertyDescriptors({
  text: "",
  activeStateEnabled: true,
  hoverStateEnabled: true,

  get focusStateEnabled() {
    return devices.real().deviceType === "desktop" && !devices.isSimulator();
  },

  defaultValue: false,
  valueChange: () => {}
})));
export var CheckBoxPropsType = {
  text: CheckBoxProps.text,
  activeStateEnabled: CheckBoxProps.activeStateEnabled,
  hoverStateEnabled: CheckBoxProps.hoverStateEnabled,
  focusStateEnabled: CheckBoxProps.focusStateEnabled,
  defaultValue: CheckBoxProps.defaultValue,
  valueChange: CheckBoxProps.valueChange,
  readOnly: CheckBoxProps.readOnly,
  name: CheckBoxProps.name,
  validationError: CheckBoxProps.validationError,
  validationErrors: CheckBoxProps.validationErrors,
  validationMessageMode: CheckBoxProps.validationMessageMode,
  validationStatus: CheckBoxProps.validationStatus,
  isValid: CheckBoxProps.isValid,
  className: CheckBoxProps.className,
  disabled: CheckBoxProps.disabled,
  tabIndex: CheckBoxProps.tabIndex,
  visible: CheckBoxProps.visible,
  aria: WidgetProps.aria
};
import { convertRulesToOptions } from 'devextreme/core/options/utils';
import * as React from "react";
import { useState, useCallback, useRef, useImperativeHandle, forwardRef } from "react";
var CheckBox = forwardRef(function checkBox(props, ref) {
  var __editorRef = useRef(null);

  var [__state_value, __state_setValue] = useState(() => props.value !== undefined ? props.value : props.defaultValue);

  var __onWidgetClick = useCallback(function __onWidgetClick(event) {
    var _ref;

    var {
      readOnly,
      saveValueChangeEvent
    } = props;
    var value = (_ref = props.value !== undefined ? props.value : __state_value) !== null && _ref !== void 0 ? _ref : false;

    if (!readOnly) {
      saveValueChangeEvent === null || saveValueChangeEvent === void 0 ? void 0 : saveValueChangeEvent(event);
      __state_setValue(__state_value => !value), props.valueChange(!value);
    }
  }, [props.readOnly, props.saveValueChangeEvent, props.value, __state_value, props.valueChange]);

  var __cssClasses = useCallback(function __cssClasses() {
    return getCssClasses(_extends({}, props, {
      value: props.value !== undefined ? props.value : __state_value
    }));
  }, [props, __state_value]);

  var __aria = useCallback(function __aria() {
    var checked = (props.value !== undefined ? props.value : __state_value) === true;
    var indeterminate = (props.value !== undefined ? props.value : __state_value) === null;
    var result = {
      role: "checkbox",
      checked: indeterminate ? "mixed" : "".concat(checked)
    };
    return _extends({}, result, props.aria);
  }, [props.value, __state_value, props.aria]);

  var __restAttributes = useCallback(function __restAttributes() {
    var _props$value = _extends({}, props, {
      value: props.value !== undefined ? props.value : __state_value
    }),
        restProps = _objectWithoutPropertiesLoose(_props$value, _excluded);

    return restProps;
  }, [props, __state_value]);

  var __focus = useCallback(function __focus() {
    __editorRef.current.focus();
  }, []);

  var __blur = useCallback(function __blur() {
    __editorRef.current.blur();
  }, []);

  var __keyDown = useCallback(function __keyDown(e) {
    var {
      onKeyDown
    } = props;
    var {
      keyName,
      originalEvent,
      which
    } = e;
    var result = onKeyDown === null || onKeyDown === void 0 ? void 0 : onKeyDown(e);

    if (result !== null && result !== void 0 && result.cancel) {
      return result;
    }

    if (keyName === "space" || which === "space") {
      originalEvent.preventDefault();

      __onWidgetClick(originalEvent);
    }

    return undefined;
  }, [props.onKeyDown, __onWidgetClick]);

  useImperativeHandle(ref, () => ({
    focus: __focus,
    blur: __blur
  }), [__focus, __blur]);
  return viewFunction({
    props: _extends({}, props, {
      value: props.value !== undefined ? props.value : __state_value
    }),
    editorRef: __editorRef,
    onWidgetClick: __onWidgetClick,
    keyDown: __keyDown,
    cssClasses: __cssClasses(),
    aria: __aria(),
    restAttributes: __restAttributes()
  });
});
export { CheckBox };
export default CheckBox;

function __processTwoWayProps(defaultProps) {
  var twoWayProps = ["value"];
  return Object.keys(defaultProps).reduce((props, propName) => {
    var propValue = defaultProps[propName];
    var defaultPropName = twoWayProps.some(p => p === propName) ? "default" + propName.charAt(0).toUpperCase() + propName.slice(1) : propName;
    props[defaultPropName] = propValue;
    return props;
  }, {});
}

CheckBox.defaultProps = CheckBoxPropsType;
var __defaultOptionRules = [];
export function defaultOptions(rule) {
  __defaultOptionRules.push(rule);

  CheckBox.defaultProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(CheckBox.defaultProps), Object.getOwnPropertyDescriptors(__processTwoWayProps(convertRulesToOptions(__defaultOptionRules)))));
}