import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["isChecked", "size"];
import getElementComputedStyle from "../../../utils/get_computed_style";
import { hasWindow } from 'devextreme/core/utils/window';
import { normalizeStyleProp } from 'devextreme/core/utils/style';
import { isNumeric as isNumber } from 'devextreme/core/utils/type';
import { getFontSizeByIconSize } from "./utils";
export var viewFunction = viewModel => {
  var {
    cssStyles,
    elementRef
  } = viewModel;
  return React.createElement("span", {
    className: "dx-checkbox-icon",
    ref: elementRef,
    style: normalizeStyles(cssStyles)
  });
};
export var CheckBoxIconProps = {
  isChecked: false
};
import * as React from "react";
import { useCallback, useMemo, useEffect, useRef } from "react";
var NUMBER_STYLES = new Set(["animationIterationCount", "borderImageOutset", "borderImageSlice", "border-imageWidth", "boxFlex", "boxFlexGroup", "boxOrdinalGroup", "columnCount", "fillOpacity", "flex", "flexGrow", "flexNegative", "flexOrder", "flexPositive", "flexShrink", "floodOpacity", "fontWeight", "gridColumn", "gridRow", "lineClamp", "lineHeight", "opacity", "order", "orphans", "stopOpacity", "strokeDasharray", "strokeDashoffset", "strokeMiterlimit", "strokeOpacity", "strokeWidth", "tabSize", "widows", "zIndex", "zoom"]);

var isNumeric = value => {
  if (typeof value === "number") return true;
  return !isNaN(Number(value));
};

var getNumberStyleValue = (style, value) => {
  return NUMBER_STYLES.has(style) ? value : "".concat(value, "px");
};

var normalizeStyles = styles => {
  if (!(styles instanceof Object)) return undefined;
  return Object.entries(styles).reduce((result, _ref) => {
    var [key, value] = _ref;
    result[key] = isNumeric(value) ? getNumberStyleValue(key, value) : value;
    return result;
  }, {});
};

export function CheckBoxIcon(props) {
  var __elementRef = useRef(null);

  var __setIconFontSize = useCallback(function __setIconFontSize(fontSize) {
    var element = __elementRef.current;
    element.style.fontSize = "".concat(fontSize, "px");
  }, []);

  var __getComputedIconSize = useCallback(function __getComputedIconSize() {
    var element = __elementRef.current;
    var iconComputedStyle = getElementComputedStyle(element);
    var computedIconSize = parseInt(iconComputedStyle === null || iconComputedStyle === void 0 ? void 0 : iconComputedStyle.width, 10);
    return computedIconSize;
  }, []);

  var __cssStyles = useMemo(function __cssStyles() {
    var {
      size
    } = props;
    var width = normalizeStyleProp("width", size);
    var height = normalizeStyleProp("height", size);
    return {
      height,
      width
    };
  }, [props.size]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __getIconSize = useCallback(function __getIconSize(size) {
    if (isNumber(size)) {
      return size;
    }

    if (size.endsWith("px")) {
      return parseInt(size, 10);
    }

    return __getComputedIconSize();
  }, [__getComputedIconSize]);

  useEffect(() => {
    var {
      isChecked,
      size
    } = props;

    if (hasWindow() && size) {
      var newIconSize = __getIconSize(size);

      var newFontSize = getFontSizeByIconSize(newIconSize, isChecked);

      __setIconFontSize(newFontSize);
    }
  }, [props.isChecked, props.size, __getIconSize, __setIconFontSize]);
  return viewFunction({
    props: _extends({}, props),
    elementRef: __elementRef,
    setIconFontSize: __setIconFontSize,
    getIconSize: __getIconSize,
    getComputedIconSize: __getComputedIconSize,
    cssStyles: __cssStyles,
    restAttributes: __restAttributes()
  });
}
CheckBoxIcon.defaultProps = CheckBoxIconProps;