import { BaseWidgetProps } from "../../common/base_props";
export declare const viewFunction: (viewModel: Editor) => any;
export declare type EditorPropsType = typeof BaseWidgetProps & {
    readOnly: boolean;
    name: string;
    value?: any;
    validationError: Record<string, unknown> | null;
    validationErrors: Record<string, unknown>[] | null;
    validationMessageMode: "auto" | "always";
    validationStatus: "valid" | "invalid" | "pending";
    isValid: boolean;
    onFocusIn?: (e: Event) => void;
    defaultValue: any;
    valueChange?: (value?: any) => void;
};
export declare const EditorProps: EditorPropsType;
export declare type EditorPropsTypeType = {
    readOnly: boolean;
    name: string;
    value?: any;
    validationError: Record<string, unknown> | null;
    validationErrors: Record<string, unknown>[] | null;
    validationMessageMode: "auto" | "always";
    validationStatus: "valid" | "invalid" | "pending";
    isValid: boolean;
    onFocusIn?: (e: Event) => void;
    defaultValue: any;
    valueChange?: (value?: any) => void;
    className?: string;
    accessKey?: string;
    activeStateEnabled?: boolean;
    disabled?: boolean;
    focusStateEnabled?: boolean;
    height?: string | number | (() => string | number);
    hint?: string;
    hoverStateEnabled?: boolean;
    onClick?: (e: any) => void;
    onKeyDown?: (e: any) => any;
    rtlEnabled?: boolean;
    tabIndex?: number;
    visible?: boolean;
    width?: string | number | (() => string | number);
    aria?: Record<string, string>;
    children?: React.ReactNode;
    classes?: string | undefined;
};
export declare const EditorPropsType: EditorPropsTypeType;
import { DefaultOptionsRule } from 'devextreme/core/options/utils';
import * as React from "react";
export declare type EditorRef = {
    focus: () => void;
    blur: () => void;
};
declare type RestProps = {
    className?: string;
    style?: {
        [name: string]: any;
    };
    key?: any;
    ref?: any;
};
interface Editor {
    props: typeof EditorPropsType & RestProps;
    widgetRef: any;
    validationMessageGuid: any;
    rootElementRef: any;
    isValidationMessageVisible: boolean;
    onFocusIn: (event: Event) => void;
    cssClasses: string;
    shouldShowValidationMessage: boolean;
    aria: Record<string, string>;
    validationErrors: Record<string, unknown>[] | null | undefined;
    validationMessageTarget: HTMLDivElement | null | undefined;
    restAttributes: RestProps;
}
declare const Editor: React.FC<EditorPropsTypeType & RestProps & {
    ref?: React.Ref<EditorRef>;
}> & {
    defaultProps: typeof EditorPropsType;
};
export { Editor };
export default Editor;
declare type EditorOptionRule = DefaultOptionsRule<typeof EditorPropsType>;
export declare function defaultOptions(rule: EditorOptionRule): void;
