import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["accessKey", "activeStateEnabled", "aria", "children", "className", "classes", "defaultValue", "disabled", "focusStateEnabled", "height", "hint", "hoverStateEnabled", "isValid", "name", "onClick", "onFocusIn", "onKeyDown", "readOnly", "rtlEnabled", "tabIndex", "validationError", "validationErrors", "validationMessageMode", "validationStatus", "value", "valueChange", "visible", "width"];
import Guid from 'devextreme/core/guid';
import { Widget, WidgetProps } from "../../common/widget";
import { BaseWidgetProps } from "../../common/base_props";
import { combineClasses } from "../../../utils/combine_classes";
import { ValidationMessage } from "../../overlays/validation_message";

var getCssClasses = model => {
  var {
    classes,
    isValid,
    readOnly
  } = model;
  var classesMap = {
    "dx-state-readonly": !!readOnly,
    "dx-invalid": !isValid,
    ["".concat(classes)]: !!classes
  };
  return combineClasses(classesMap);
};

export var viewFunction = viewModel => {
  var {
    aria,
    cssClasses: classes,
    isValidationMessageVisible,
    onFocusIn,
    props: {
      accessKey,
      activeStateEnabled,
      children,
      className,
      disabled,
      focusStateEnabled,
      height,
      hint,
      hoverStateEnabled,
      onClick,
      onKeyDown,
      rtlEnabled,
      tabIndex,
      validationMessageMode,
      visible,
      width
    },
    restAttributes,
    rootElementRef,
    validationErrors,
    validationMessageGuid,
    validationMessageTarget,
    widgetRef
  } = viewModel;
  return React.createElement(Widget, _extends({
    ref: widgetRef,
    rootElementRef: rootElementRef,
    aria: aria,
    classes: classes,
    activeStateEnabled: activeStateEnabled,
    focusStateEnabled: focusStateEnabled,
    hoverStateEnabled: hoverStateEnabled,
    accessKey: accessKey,
    className: className,
    rtlEnabled: rtlEnabled,
    hint: hint,
    disabled: disabled,
    height: height,
    width: width,
    onFocusIn: onFocusIn,
    onClick: onClick,
    onKeyDown: onKeyDown,
    tabIndex: tabIndex,
    visible: visible
  }, restAttributes), React.createElement(React.Fragment, null, children, isValidationMessageVisible && React.createElement(ValidationMessage, {
    validationErrors: validationErrors,
    mode: validationMessageMode,
    positionRequest: "below",
    rtlEnabled: rtlEnabled,
    target: validationMessageTarget,
    boundary: validationMessageTarget,
    container: validationMessageTarget,
    contentId: validationMessageGuid
  })));
};
export var EditorProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(BaseWidgetProps), Object.getOwnPropertyDescriptors({
  readOnly: false,
  name: "",
  validationError: null,
  validationErrors: null,
  validationMessageMode: "auto",
  validationStatus: "valid",
  isValid: true,
  defaultValue: null,
  valueChange: () => {}
})));
export var EditorPropsType = {
  readOnly: EditorProps.readOnly,
  name: EditorProps.name,
  validationError: EditorProps.validationError,
  validationErrors: EditorProps.validationErrors,
  validationMessageMode: EditorProps.validationMessageMode,
  validationStatus: EditorProps.validationStatus,
  isValid: EditorProps.isValid,
  defaultValue: EditorProps.defaultValue,
  valueChange: EditorProps.valueChange,
  className: EditorProps.className,
  activeStateEnabled: EditorProps.activeStateEnabled,
  disabled: EditorProps.disabled,
  focusStateEnabled: EditorProps.focusStateEnabled,
  hoverStateEnabled: EditorProps.hoverStateEnabled,
  tabIndex: EditorProps.tabIndex,
  visible: EditorProps.visible,
  aria: WidgetProps.aria,
  classes: WidgetProps.classes
};
import { convertRulesToOptions } from 'devextreme/core/options/utils';
import * as React from "react";
import { useState, useCallback, useMemo, useEffect, useRef, useImperativeHandle, forwardRef } from "react";
var Editor = forwardRef(function editor(props, ref) {
  var __rootElementRef = useRef(null);

  var __widgetRef = useRef(null);

  var [__state_value, __state_setValue] = useState(() => props.value !== undefined ? props.value : props.defaultValue);
  var [__state_validationMessageGuid, __state_setValidationMessageGuid] = useState("dx-".concat(new Guid()));
  var [__state_isValidationMessageVisible, __state_setIsValidationMessageVisible] = useState(false);

  var __onFocusIn = useCallback(function __onFocusIn(event) {
    var {
      onFocusIn
    } = props;
    onFocusIn === null || onFocusIn === void 0 ? void 0 : onFocusIn(event);
  }, [props.onFocusIn]);

  var __cssClasses = useCallback(function __cssClasses() {
    return "".concat(getCssClasses(_extends({}, props, {
      value: props.value !== undefined ? props.value : __state_value
    })));
  }, [props, __state_value]);

  var __validationErrors = useMemo(function __validationErrors() {
    var {
      validationError,
      validationErrors
    } = props;
    var allValidationErrors = validationErrors && [...validationErrors];

    if (!allValidationErrors && validationError) {
      allValidationErrors = [_extends({}, validationError)];
    }

    return allValidationErrors;
  }, [props.validationError, props.validationErrors]);

  var __validationMessageTarget = useCallback(function __validationMessageTarget() {
    return __rootElementRef === null || __rootElementRef === void 0 ? void 0 : __rootElementRef.current;
  }, []);

  var __restAttributes = useCallback(function __restAttributes() {
    var _props$value = _extends({}, props, {
      value: props.value !== undefined ? props.value : __state_value
    }),
        restProps = _objectWithoutPropertiesLoose(_props$value, _excluded);

    return restProps;
  }, [props, __state_value]);

  var __focus = useCallback(function __focus() {
    __widgetRef.current.focus();
  }, []);

  var __blur = useCallback(function __blur() {
    __widgetRef.current.blur();
  }, []);

  var __shouldShowValidationMessage = useCallback(function __shouldShowValidationMessage() {
    var {
      isValid,
      validationStatus
    } = props;
    var validationErrors = __validationErrors !== null && __validationErrors !== void 0 ? __validationErrors : [];
    var isEditorValid = isValid && validationStatus !== "invalid";
    return !isEditorValid && validationErrors.length > 0;
  }, [props.isValid, props.validationStatus, __validationErrors]);

  var __aria = useCallback(function __aria() {
    var {
      isValid,
      readOnly
    } = props;
    var result = {
      readonly: readOnly ? "true" : "false",
      invalid: !isValid ? "true" : "false"
    };

    if (__shouldShowValidationMessage()) {
      result.describedBy = __state_validationMessageGuid;
    }

    return _extends({}, result, props.aria);
  }, [props.isValid, props.readOnly, __shouldShowValidationMessage, __state_validationMessageGuid, props.aria]);

  useEffect(() => {
    __state_setIsValidationMessageVisible(__state_isValidationMessageVisible => __shouldShowValidationMessage());
  }, [__shouldShowValidationMessage]);
  useImperativeHandle(ref, () => ({
    focus: __focus,
    blur: __blur
  }), [__focus, __blur]);
  return viewFunction({
    props: _extends({}, props, {
      value: props.value !== undefined ? props.value : __state_value
    }),
    validationMessageGuid: __state_validationMessageGuid,
    isValidationMessageVisible: __state_isValidationMessageVisible,
    rootElementRef: __rootElementRef,
    widgetRef: __widgetRef,
    onFocusIn: __onFocusIn,
    cssClasses: __cssClasses(),
    shouldShowValidationMessage: __shouldShowValidationMessage(),
    aria: __aria(),
    validationErrors: __validationErrors,
    validationMessageTarget: __validationMessageTarget(),
    restAttributes: __restAttributes()
  });
});
export { Editor };
export default Editor;

function __processTwoWayProps(defaultProps) {
  var twoWayProps = ["value"];
  return Object.keys(defaultProps).reduce((props, propName) => {
    var propValue = defaultProps[propName];
    var defaultPropName = twoWayProps.some(p => p === propName) ? "default" + propName.charAt(0).toUpperCase() + propName.slice(1) : propName;
    props[defaultPropName] = propValue;
    return props;
  }, {});
}

Editor.defaultProps = EditorPropsType;
var __defaultOptionRules = [];
export function defaultOptions(rule) {
  __defaultOptionRules.push(rule);

  Editor.defaultProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(Editor.defaultProps), Object.getOwnPropertyDescriptors(__processTwoWayProps(convertRulesToOptions(__defaultOptionRules)))));
}