import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["className", "displayMode", "gridCompatibility", "hasKnownLastPage", "infoText", "infoTextRef", "infoTextVisible", "isLargeDisplayMode", "lightModeEnabled", "maxPagesCount", "onKeyDown", "pageCount", "pageIndex", "pageIndexChange", "pageSize", "pageSizeChange", "pageSizes", "pageSizesRef", "pagesCountText", "pagesNavigatorVisible", "pagesRef", "rootElementRef", "rtlEnabled", "showInfo", "showNavigationButtons", "showPageSizes", "totalCount", "visible"];
import { InfoText } from "./info";
import { PageIndexSelector } from "./pages/page_index_selector";
import { PageSizeSelector } from "./page_size/selector";
import { PAGER_PAGES_CLASS, PAGER_PAGE_INDEXES_CLASS, LIGHT_MODE_CLASS, PAGER_CLASS } from "./common/consts";
import { InternalPagerProps } from "./common/pager_props";
import { combineClasses } from "../../utils/combine_classes";
import { Widget } from "../common/widget";
import { registerKeyboardAction as _registerKeyboardAction } from 'devextreme/ui/shared/accessibility';
import { KeyboardActionContext } from "./common/keyboard_action_context";
export var viewFunction = _ref => {
  var {
    classes,
    infoVisible,
    isLargeDisplayMode,
    pageIndexSelectorVisible,
    pagesContainerVisibility,
    pagesContainerVisible,
    props: {
      hasKnownLastPage,
      infoText,
      infoTextRef,
      maxPagesCount,
      pageCount,
      pageIndex,
      pageIndexChange,
      pageSize,
      pageSizeChange,
      pageSizes,
      pageSizesRef,
      pagesCountText,
      pagesRef,
      rtlEnabled,
      showNavigationButtons,
      showPageSizes,
      totalCount,
      visible
    },
    restAttributes,
    widgetRootElementRef
  } = _ref;
  return React.createElement(Widget, _extends({
    rootElementRef: widgetRootElementRef,
    rtlEnabled: rtlEnabled,
    classes: classes,
    visible: visible
  }, restAttributes), showPageSizes && React.createElement(PageSizeSelector, {
    rootElementRef: pageSizesRef,
    isLargeDisplayMode: isLargeDisplayMode,
    pageSize: pageSize,
    pageSizeChange: pageSizeChange,
    pageSizes: pageSizes
  }), pagesContainerVisible && React.createElement("div", {
    className: PAGER_PAGES_CLASS,
    style: normalizeStyles({
      visibility: pagesContainerVisibility
    })
  }, infoVisible && React.createElement(InfoText, {
    rootElementRef: infoTextRef,
    infoText: infoText,
    pageCount: pageCount,
    pageIndex: pageIndex,
    totalCount: totalCount
  }), pageIndexSelectorVisible && React.createElement("div", {
    className: PAGER_PAGE_INDEXES_CLASS,
    ref: pagesRef
  }, React.createElement(PageIndexSelector, {
    hasKnownLastPage: hasKnownLastPage,
    isLargeDisplayMode: isLargeDisplayMode,
    maxPagesCount: maxPagesCount,
    pageCount: pageCount,
    pageIndex: pageIndex,
    pageIndexChange: pageIndexChange,
    pagesCountText: pagesCountText,
    showNavigationButtons: showNavigationButtons,
    totalCount: totalCount
  }))));
};
export var PagerContentProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(InternalPagerProps), Object.getOwnPropertyDescriptors({
  infoTextVisible: true,
  isLargeDisplayMode: true
})));
import * as React from "react";
import { useCallback, useEffect, useRef } from "react";
var NUMBER_STYLES = new Set(["animationIterationCount", "borderImageOutset", "borderImageSlice", "border-imageWidth", "boxFlex", "boxFlexGroup", "boxOrdinalGroup", "columnCount", "fillOpacity", "flex", "flexGrow", "flexNegative", "flexOrder", "flexPositive", "flexShrink", "floodOpacity", "fontWeight", "gridColumn", "gridRow", "lineClamp", "lineHeight", "opacity", "order", "orphans", "stopOpacity", "strokeDasharray", "strokeDashoffset", "strokeMiterlimit", "strokeOpacity", "strokeWidth", "tabSize", "widows", "zIndex", "zoom"]);

var isNumeric = value => {
  if (typeof value === "number") return true;
  return !isNaN(Number(value));
};

var getNumberStyleValue = (style, value) => {
  return NUMBER_STYLES.has(style) ? value : "".concat(value, "px");
};

var normalizeStyles = styles => {
  if (!(styles instanceof Object)) return undefined;
  return Object.entries(styles).reduce((result, _ref2) => {
    var [key, value] = _ref2;
    result[key] = isNumeric(value) ? getNumberStyleValue(key, value) : value;
    return result;
  }, {});
};

export function PagerContent(props) {
  var __widgetRootElementRef = useRef(null);

  var __createFakeInstance = useCallback(function __createFakeInstance() {
    return {
      option: () => false,
      element: () => __widgetRootElementRef.current,
      _createActionByOption: () => e => {
        var _props$onKeyDown;

        (_props$onKeyDown = props.onKeyDown) === null || _props$onKeyDown === void 0 ? void 0 : _props$onKeyDown.call(props, e);
      }
    };
  }, [props.onKeyDown]);

  var __pageIndexSelectorVisible = useCallback(function __pageIndexSelectorVisible() {
    return props.pageSize !== 0;
  }, [props.pageSize]);

  var __normalizedDisplayMode = useCallback(function __normalizedDisplayMode() {
    var {
      displayMode,
      lightModeEnabled
    } = props;

    if (displayMode === "adaptive" && lightModeEnabled !== undefined) {
      return lightModeEnabled ? "compact" : "full";
    }

    return displayMode;
  }, [props.displayMode, props.lightModeEnabled]);

  var __pagesContainerVisible = useCallback(function __pagesContainerVisible() {
    return !!props.pagesNavigatorVisible && props.pageCount > 0;
  }, [props.pagesNavigatorVisible, props.pageCount]);

  var __pagesContainerVisibility = useCallback(function __pagesContainerVisibility() {
    if (props.pagesNavigatorVisible === "auto" && props.pageCount === 1 && props.hasKnownLastPage) {
      return "hidden";
    }

    return undefined;
  }, [props.pagesNavigatorVisible, props.pageCount, props.hasKnownLastPage]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __keyboardAction = useCallback(function __keyboardAction() {
    return {
      registerKeyboardAction: (element, action) => {
        var fakePagerInstance = __createFakeInstance();

        return _registerKeyboardAction("pager", fakePagerInstance, element, undefined, action);
      }
    };
  }, [__createFakeInstance]);

  var __isLargeDisplayMode = useCallback(function __isLargeDisplayMode() {
    var displayMode = __normalizedDisplayMode();

    var result = false;

    if (displayMode === "adaptive") {
      result = props.isLargeDisplayMode;
    } else {
      result = displayMode === "full";
    }

    return result;
  }, [__normalizedDisplayMode, props.isLargeDisplayMode]);

  var __infoVisible = useCallback(function __infoVisible() {
    var {
      infoTextVisible,
      showInfo
    } = props;
    return showInfo && infoTextVisible && __isLargeDisplayMode();
  }, [props.infoTextVisible, props.showInfo, __isLargeDisplayMode]);

  var __classes = useCallback(function __classes() {
    var classesMap = {
      ["".concat(props.className)]: !!props.className,
      [PAGER_CLASS]: true,
      [LIGHT_MODE_CLASS]: !__isLargeDisplayMode()
    };
    return combineClasses(classesMap);
  }, [props.className, __isLargeDisplayMode]);

  useEffect(() => {
    var {
      rootElementRef
    } = props;

    if (rootElementRef) {
      rootElementRef.current = __widgetRootElementRef.current;
    }
  }, []);
  return React.createElement(KeyboardActionContext.Provider, {
    value: __keyboardAction()
  }, viewFunction({
    props: _extends({}, props),
    widgetRootElementRef: __widgetRootElementRef,
    keyboardAction: __keyboardAction(),
    infoVisible: __infoVisible(),
    pageIndexSelectorVisible: __pageIndexSelectorVisible(),
    pagesContainerVisible: __pagesContainerVisible(),
    pagesContainerVisibility: __pagesContainerVisibility(),
    isLargeDisplayMode: __isLargeDisplayMode(),
    classes: __classes(),
    restAttributes: __restAttributes()
  }));
}
PagerContent.defaultProps = PagerContentProps;