import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["maxPagesCount", "pageCount", "pageIndex", "pageIndexChange"],
    _excluded2 = ["pageIndexes"];
import { Page } from "./page";
import { InternalPagerProps } from "../common/pager_props";
import { ConfigContext } from "../../../common/config_context";
var PAGER_PAGE_SEPARATOR_CLASS = "dx-separator";
export var viewFunction = _ref => {
  var {
    pages
  } = _ref;
  var PagesMarkup = pages.map(_ref2 => {
    var {
      key,
      pageProps
    } = _ref2;
    return pageProps ? React.createElement(Page, {
      key: key,
      index: pageProps.index,
      selected: pageProps.selected,
      onClick: pageProps.onClick
    }) : React.createElement("div", {
      key: key,
      className: PAGER_PAGE_SEPARATOR_CLASS
    }, ". . .");
  });
  return React.createElement(React.Fragment, null, PagesMarkup);
};
var PAGES_LIMITER = 4;

function getDelimiterType(startIndex, slidingWindowSize, pageCount) {
  if (startIndex === 1) {
    return "high";
  }

  if (startIndex + slidingWindowSize === pageCount - 1) {
    return "low";
  }

  return "both";
}

function createPageIndexesBySlidingWindowIndexes(slidingWindowIndexes, pageCount, delimiter) {
  var pageIndexes = [];
  var indexesForReuse = [];

  switch (delimiter) {
    case "none":
      pageIndexes = [...slidingWindowIndexes];
      break;

    case "both":
      pageIndexes = [0, "low", ...slidingWindowIndexes, "high", pageCount - 1];
      indexesForReuse = slidingWindowIndexes.slice(1, -1);
      break;

    case "high":
      pageIndexes = [0, ...slidingWindowIndexes, "high", pageCount - 1];
      indexesForReuse = slidingWindowIndexes.slice(0, -1);
      break;

    case "low":
      pageIndexes = [0, "low", ...slidingWindowIndexes, pageCount - 1];
      indexesForReuse = slidingWindowIndexes.slice(1);
      break;
  }

  return {
    slidingWindowIndexes,
    indexesForReuse,
    pageIndexes
  };
}

function createPageIndexes(startIndex, slidingWindowSize, pageCount, delimiter) {
  var slidingWindowIndexes = [];

  for (var i = 0; i < slidingWindowSize; i += 1) {
    slidingWindowIndexes.push(i + startIndex);
  }

  return createPageIndexesBySlidingWindowIndexes(slidingWindowIndexes, pageCount, delimiter);
}

var PagesLargePropsType = {
  pageIndex: InternalPagerProps.pageIndex,
  maxPagesCount: InternalPagerProps.maxPagesCount,
  pageCount: InternalPagerProps.pageCount
};
import * as React from "react";
import { useContext, useCallback, useMemo, useRef } from "react";
export function PagesLarge(props) {
  var slidingWindowStateHolder = useRef();
  var config = useContext(ConfigContext);

  var __slidingWindowState = useCallback(function __slidingWindowState() {
    var slidingWindowState = slidingWindowStateHolder.current;

    if (!slidingWindowState) {
      return {
        indexesForReuse: [],
        slidingWindowIndexes: []
      };
    }

    return slidingWindowState;
  }, [slidingWindowStateHolder]);

  var __isSlidingWindowMode = useCallback(function __isSlidingWindowMode() {
    var {
      maxPagesCount,
      pageCount
    } = props;
    return pageCount <= PAGES_LIMITER || pageCount <= maxPagesCount;
  }, [props.maxPagesCount, props.pageCount]);

  var __onPageClick = useCallback(function __onPageClick(pageIndex) {
    props.pageIndexChange(pageIndex);
  }, [props.pageIndexChange]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __canReuseSlidingWindow = useCallback(function __canReuseSlidingWindow(currentPageCount, pageIndex) {
    var {
      indexesForReuse
    } = __slidingWindowState();

    var currentPageNotExistInIndexes = !indexesForReuse.includes(currentPageCount);
    var pageIndexExistInIndexes = indexesForReuse.includes(pageIndex);
    return currentPageNotExistInIndexes && pageIndexExistInIndexes;
  }, [__slidingWindowState]);

  var __generatePageIndexes = useCallback(function __generatePageIndexes() {
    var {
      pageCount,
      pageIndex
    } = props;
    var startIndex = 0;

    var {
      slidingWindowIndexes
    } = __slidingWindowState();

    if (pageIndex === slidingWindowIndexes[0]) {
      startIndex = pageIndex - 1;
    } else if (pageIndex === slidingWindowIndexes[slidingWindowIndexes.length - 1]) {
      startIndex = pageIndex + 2 - PAGES_LIMITER;
    } else if (pageIndex < PAGES_LIMITER) {
      startIndex = 1;
    } else if (pageIndex >= pageCount - PAGES_LIMITER) {
      startIndex = pageCount - PAGES_LIMITER - 1;
    } else {
      startIndex = pageIndex - 1;
    }

    var slidingWindowSize = PAGES_LIMITER;
    var delimiter = getDelimiterType(startIndex, slidingWindowSize, pageCount);

    var _createPageIndexes = createPageIndexes(startIndex, slidingWindowSize, pageCount, delimiter),
        {
      pageIndexes
    } = _createPageIndexes,
        slidingWindowState = _objectWithoutPropertiesLoose(_createPageIndexes, _excluded2);

    slidingWindowStateHolder.current = slidingWindowState;
    return pageIndexes;
  }, [props.pageCount, props.pageIndex, __slidingWindowState]);

  var __pageIndexes = useMemo(function __pageIndexes() {
    var {
      pageCount
    } = props;

    if (__isSlidingWindowMode()) {
      return createPageIndexes(0, pageCount, pageCount, "none").pageIndexes;
    }

    if (__canReuseSlidingWindow(pageCount, props.pageIndex)) {
      var {
        slidingWindowIndexes
      } = __slidingWindowState();

      var delimiter = getDelimiterType(slidingWindowIndexes[0], PAGES_LIMITER, pageCount);
      return createPageIndexesBySlidingWindowIndexes(slidingWindowIndexes, pageCount, delimiter).pageIndexes;
    }

    return __generatePageIndexes();
  }, [__isSlidingWindowMode, __canReuseSlidingWindow, props.pageIndex, __slidingWindowState, __generatePageIndexes]);

  var __pages = useMemo(function __pages() {
    var {
      pageIndex
    } = props;

    var createPage = index => {
      var pagerProps = index === "low" || index === "high" ? null : {
        index,
        onClick: () => __onPageClick(index),
        selected: pageIndex === index
      };
      return {
        key: index.toString(),
        pageProps: pagerProps
      };
    };

    var rtlPageIndexes = config !== null && config !== void 0 && config.rtlEnabled ? [...__pageIndexes].reverse() : __pageIndexes;
    return rtlPageIndexes.map(index => createPage(index));
  }, [props.pageIndex, __onPageClick, config, __pageIndexes]);

  return viewFunction({
    props: _extends({}, props),
    config,
    pageIndexes: __pageIndexes,
    pages: __pages,
    restAttributes: __restAttributes()
  });
}
PagesLarge.defaultProps = PagesLargePropsType;