import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["hasKnownLastPage", "isLargeDisplayMode", "maxPagesCount", "pageCount", "pageIndex", "pageIndexChange", "pagesCountText", "showNavigationButtons", "totalCount"];
import { LightButton } from "../common/light_button";
import { PagesLarge } from "./large";
import { PagesSmall } from "./small";
import { InternalPagerProps } from "../common/pager_props";
import { ConfigContext } from "../../../common/config_context";
var PAGER_NAVIGATE_BUTTON = "dx-navigate-button";
var PAGER_PREV_BUTTON_CLASS = "dx-prev-button";
var PAGER_NEXT_BUTTON_CLASS = "dx-next-button";
export var PAGER_BUTTON_DISABLE_CLASS = "dx-button-disable";
var nextButtonClassName = "".concat(PAGER_NAVIGATE_BUTTON, " ").concat(PAGER_NEXT_BUTTON_CLASS);
var prevButtonClassName = "".concat(PAGER_NAVIGATE_BUTTON, " ").concat(PAGER_PREV_BUTTON_CLASS);
var nextButtonDisabledClassName = "".concat(PAGER_BUTTON_DISABLE_CLASS, " ").concat(PAGER_NAVIGATE_BUTTON, " ").concat(PAGER_NEXT_BUTTON_CLASS);
var prevButtonDisabledClassName = "".concat(PAGER_BUTTON_DISABLE_CLASS, " ").concat(PAGER_NAVIGATE_BUTTON, " ").concat(PAGER_PREV_BUTTON_CLASS);
export var viewFunction = _ref => {
  var {
    navigateToNextPage,
    navigateToPrevPage,
    nextClassName,
    pageIndexChange,
    prevClassName,
    props: {
      isLargeDisplayMode,
      maxPagesCount,
      pageCount,
      pageIndex,
      pagesCountText
    },
    renderNextButton,
    renderPrevButton
  } = _ref;
  return React.createElement(React.Fragment, null, renderPrevButton && React.createElement(LightButton, {
    className: prevClassName,
    label: "Previous page",
    onClick: navigateToPrevPage
  }), isLargeDisplayMode && React.createElement(PagesLarge, {
    maxPagesCount: maxPagesCount,
    pageCount: pageCount,
    pageIndex: pageIndex,
    pageIndexChange: pageIndexChange
  }), !isLargeDisplayMode && React.createElement(PagesSmall, {
    pageCount: pageCount,
    pageIndex: pageIndex,
    pageIndexChange: pageIndexChange,
    pagesCountText: pagesCountText
  }), renderNextButton && React.createElement(LightButton, {
    className: nextClassName,
    label: "Next page",
    onClick: navigateToNextPage
  }));
};

function getIncrement(direction) {
  return direction === "next" ? +1 : -1;
}

export var PageIndexSelectorProps = {
  isLargeDisplayMode: true
};
var PageIndexSelectorPropsType = {
  pageIndex: InternalPagerProps.pageIndex,
  maxPagesCount: InternalPagerProps.maxPagesCount,
  pageCount: InternalPagerProps.pageCount,
  hasKnownLastPage: InternalPagerProps.hasKnownLastPage,
  showNavigationButtons: InternalPagerProps.showNavigationButtons,
  totalCount: InternalPagerProps.totalCount,
  isLargeDisplayMode: PageIndexSelectorProps.isLargeDisplayMode
};
import * as React from "react";
import { useContext, useCallback } from "react";
export function PageIndexSelector(props) {
  var config = useContext(ConfigContext);

  var __getNextDirection = useCallback(function __getNextDirection() {
    return !(config !== null && config !== void 0 && config.rtlEnabled) ? "next" : "prev";
  }, [config]);

  var __getPrevDirection = useCallback(function __getPrevDirection() {
    return !(config !== null && config !== void 0 && config.rtlEnabled) ? "prev" : "next";
  }, [config]);

  var __canNavigateToPage = useCallback(function __canNavigateToPage(pageIndex) {
    if (!props.hasKnownLastPage) {
      return pageIndex >= 0;
    }

    return pageIndex >= 0 && pageIndex <= props.pageCount - 1;
  }, [props.hasKnownLastPage, props.pageCount]);

  var __getNextPageIndex = useCallback(function __getNextPageIndex(direction) {
    return props.pageIndex + getIncrement(direction);
  }, [props.pageIndex]);

  var __renderPrevButton = useCallback(function __renderPrevButton() {
    var {
      isLargeDisplayMode,
      showNavigationButtons
    } = props;
    return !isLargeDisplayMode || showNavigationButtons;
  }, [props.isLargeDisplayMode, props.showNavigationButtons]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __pageIndexChange = useCallback(function __pageIndexChange(pageIndex) {
    if (__canNavigateToPage(pageIndex)) {
      props.pageIndexChange(pageIndex);
    }
  }, [__canNavigateToPage, props.pageIndexChange]);

  var __canNavigateTo = useCallback(function __canNavigateTo(direction) {
    return __canNavigateToPage(__getNextPageIndex(direction));
  }, [__canNavigateToPage, __getNextPageIndex]);

  var __renderNextButton = useCallback(function __renderNextButton() {
    return __renderPrevButton() || !props.hasKnownLastPage;
  }, [__renderPrevButton, props.hasKnownLastPage]);

  var __navigateToPage = useCallback(function __navigateToPage(direction) {
    __pageIndexChange(__getNextPageIndex(direction));
  }, [__pageIndexChange, __getNextPageIndex]);

  var __nextClassName = useCallback(function __nextClassName() {
    var direction = __getNextDirection();

    var canNavigate = __canNavigateTo(direction);

    return canNavigate ? nextButtonClassName : nextButtonDisabledClassName;
  }, [__getNextDirection, __canNavigateTo]);

  var __prevClassName = useCallback(function __prevClassName() {
    var direction = __getPrevDirection();

    var canNavigate = __canNavigateTo(direction);

    return canNavigate ? prevButtonClassName : prevButtonDisabledClassName;
  }, [__getPrevDirection, __canNavigateTo]);

  var __navigateToNextPage = useCallback(function __navigateToNextPage() {
    __navigateToPage(__getNextDirection());
  }, [__navigateToPage, __getNextDirection]);

  var __navigateToPrevPage = useCallback(function __navigateToPrevPage() {
    __navigateToPage(__getPrevDirection());
  }, [__navigateToPage, __getPrevDirection]);

  return viewFunction({
    props: _extends({}, props),
    config,
    pageIndexChange: __pageIndexChange,
    navigateToNextPage: __navigateToNextPage,
    navigateToPrevPage: __navigateToPrevPage,
    renderPrevButton: __renderPrevButton(),
    renderNextButton: __renderNextButton(),
    nextClassName: __nextClassName(),
    prevClassName: __prevClassName(),
    restAttributes: __restAttributes()
  });
}
PageIndexSelector.defaultProps = PageIndexSelectorPropsType;