import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["contentComponent", "contentRender", "contentTemplate", "pagerProps"];
import resizeCallbacks from 'devextreme/core/utils/resize_callbacks';
import { getElementWidth, getElementStyle } from "./utils/get_element_width";
import { isDefined } from 'devextreme/core/utils/type';
export var viewFunction = _ref => {
  var {
    contentAttributes,
    infoTextRef,
    infoTextVisible,
    isLargeDisplayMode,
    pageSizesRef,
    pagesRef,
    parentRef,
    props: {
      contentTemplate: Content
    }
  } = _ref;
  return Content(_extends({
    rootElementRef: parentRef,
    pageSizesRef: pageSizesRef,
    infoTextRef: infoTextRef,
    pagesRef: pagesRef,
    infoTextVisible: infoTextVisible,
    isLargeDisplayMode: isLargeDisplayMode
  }, contentAttributes));
};
export function calculateAdaptivityProps(_ref2) {
  var {
    info: infoWidth,
    pageSizes: pageSizesWidth,
    pages: pagesWidth,
    parent: parentWidth
  } = _ref2;
  var minimalWidth = pageSizesWidth + pagesWidth + infoWidth;
  var infoTextVisible = parentWidth - minimalWidth > 0;
  var isLargeDisplayMode = parentWidth - (pageSizesWidth + pagesWidth) > 0;
  return {
    infoTextVisible,
    isLargeDisplayMode
  };
}

function getElementsWidth(_ref3) {
  var {
    info,
    pageSizes,
    pages,
    parent
  } = _ref3;
  var parentWidth = getElementWidth(parent);
  var pageSizesWidth = getElementWidth(pageSizes);
  var infoWidth = getElementWidth(info);
  var pagesHtmlWidth = getElementWidth(pages);
  return {
    parent: parentWidth,
    pageSizes: pageSizesWidth,
    info: infoWidth + getElementStyle("marginLeft", info) + getElementStyle("marginRight", info),
    pages: pagesHtmlWidth
  };
}

export var ResizableContainerProps = {};
import { useState, useCallback, useEffect, useRef } from "react";
import { getTemplate } from "@devextreme/runtime/react";
export function ResizableContainer(props) {
  var __parentRef = useRef(null);

  var __pageSizesRef = useRef(null);

  var __infoTextRef = useRef(null);

  var __pagesRef = useRef(null);

  var [__state_infoTextVisible, __state_setInfoTextVisible] = useState(true);
  var [__state_isLargeDisplayMode, __state_setIsLargeDisplayMode] = useState(true);
  var elementsWidth = useRef();
  var actualAdaptivityProps = useRef();

  var __updateAdaptivityProps = useCallback(function __updateAdaptivityProps() {
    var currentElementsWidth = getElementsWidth({
      parent: __parentRef.current,
      pageSizes: __pageSizesRef.current,
      info: __infoTextRef.current,
      pages: __pagesRef.current
    });

    if (isDefined(actualAdaptivityProps.current) && (actualAdaptivityProps.current.infoTextVisible !== __state_infoTextVisible || actualAdaptivityProps.current.isLargeDisplayMode !== __state_isLargeDisplayMode)) {
      return;
    }

    var isEmpty = !isDefined(elementsWidth.current);

    if (isEmpty) {
      elementsWidth.current = {};
    }

    if (isEmpty || __state_isLargeDisplayMode) {
      elementsWidth.current.pageSizes = currentElementsWidth.pageSizes;
      elementsWidth.current.pages = currentElementsWidth.pages;
    }

    if (isEmpty || __state_infoTextVisible) {
      elementsWidth.current.info = currentElementsWidth.info;
    }

    actualAdaptivityProps.current = calculateAdaptivityProps(_extends({
      parent: currentElementsWidth.parent
    }, elementsWidth.current));

    __state_setInfoTextVisible(__state_infoTextVisible => actualAdaptivityProps.current.infoTextVisible);

    __state_setIsLargeDisplayMode(__state_isLargeDisplayMode => actualAdaptivityProps.current.isLargeDisplayMode);
  }, [actualAdaptivityProps, __state_infoTextVisible, __state_isLargeDisplayMode, elementsWidth]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __contentAttributes = useCallback(function __contentAttributes() {
    var {
      className,
      displayMode,
      gridCompatibility,
      hasKnownLastPage,
      infoText,
      lightModeEnabled,
      maxPagesCount,
      onKeyDown,
      pageCount,
      pageIndex,
      pageIndexChange,
      pageSize,
      pageSizeChange,
      pageSizes,
      pagesCountText,
      pagesNavigatorVisible,
      rtlEnabled,
      showInfo,
      showNavigationButtons,
      showPageSizes,
      totalCount,
      visible
    } = props.pagerProps;
    return _extends({}, __restAttributes(), {
      pageSize,
      pageIndex,
      pageIndexChange,
      pageSizeChange,
      gridCompatibility,
      className,
      showInfo,
      infoText,
      lightModeEnabled,
      displayMode,
      maxPagesCount,
      pageCount,
      pagesCountText,
      visible,
      hasKnownLastPage,
      pagesNavigatorVisible,
      showPageSizes,
      pageSizes,
      rtlEnabled,
      showNavigationButtons,
      totalCount,
      onKeyDown
    });
  }, [props.pagerProps, __restAttributes]);

  useEffect(() => {
    var callback = () => {
      __updateAdaptivityProps();
    };

    resizeCallbacks.add(callback);
    return () => {
      resizeCallbacks.remove(callback);
    };
  }, [__updateAdaptivityProps]);
  useEffect(() => {
    var parentWidth = __parentRef.current ? getElementWidth(__parentRef.current) : 0;

    if (parentWidth > 0) {
      __updateAdaptivityProps();
    }
  });
  return viewFunction({
    props: _extends({}, props, {
      contentTemplate: getTemplate(props.contentTemplate, props.contentRender, props.contentComponent)
    }),
    infoTextVisible: __state_infoTextVisible,
    isLargeDisplayMode: __state_isLargeDisplayMode,
    parentRef: __parentRef,
    pageSizesRef: __pageSizesRef,
    infoTextRef: __infoTextRef,
    pagesRef: __pagesRef,
    contentAttributes: __contentAttributes(),
    updateAdaptivityProps: __updateAdaptivityProps,
    restAttributes: __restAttributes()
  });
}
ResizableContainer.defaultProps = ResizableContainerProps;