import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["appointmentComponent", "appointmentRender", "appointmentTemplate", "index", "onItemClick", "viewModel"];
import { getAppointmentStyles } from "./utils";
import { AppointmentContent } from "./content/layout";
import { Widget } from "../../common/widget";
import { combineClasses } from "../../../utils/combine_classes";
export var viewFunction = _ref => {
  var {
    classes,
    data,
    dateText,
    isReduced,
    onItemClick,
    props: {
      appointmentTemplate,
      index,
      viewModel: {
        info: {
          isRecurrent
        }
      }
    },
    ref,
    styles,
    text
  } = _ref;
  return React.createElement(Widget, _extends({
    onClick: onItemClick,
    rootElementRef: ref,
    style: normalizeStyles(styles),
    classes: classes,
    hint: text
  }, {
    role: "button"
  }), React.createElement(AppointmentContent, {
    text: text,
    isReduced: isReduced,
    dateText: dateText,
    isRecurrent: isRecurrent,
    index: index,
    data: data,
    appointmentTemplate: appointmentTemplate
  }));
};
export var AppointmentProps = {
  index: 0
};
import * as React from "react";
import { useCallback, useRef } from "react";
import { getTemplate } from "@devextreme/runtime/react";
var NUMBER_STYLES = new Set(["animationIterationCount", "borderImageOutset", "borderImageSlice", "border-imageWidth", "boxFlex", "boxFlexGroup", "boxOrdinalGroup", "columnCount", "fillOpacity", "flex", "flexGrow", "flexNegative", "flexOrder", "flexPositive", "flexShrink", "floodOpacity", "fontWeight", "gridColumn", "gridRow", "lineClamp", "lineHeight", "opacity", "order", "orphans", "stopOpacity", "strokeDasharray", "strokeDashoffset", "strokeMiterlimit", "strokeOpacity", "strokeWidth", "tabSize", "widows", "zIndex", "zoom"]);

var isNumeric = value => {
  if (typeof value === "number") return true;
  return !isNaN(Number(value));
};

var getNumberStyleValue = (style, value) => {
  return NUMBER_STYLES.has(style) ? value : "".concat(value, "px");
};

var normalizeStyles = styles => {
  if (!(styles instanceof Object)) return undefined;
  return Object.entries(styles).reduce((result, _ref2) => {
    var [key, value] = _ref2;
    result[key] = isNumeric(value) ? getNumberStyleValue(key, value) : value;
    return result;
  }, {});
};

export function Appointment(props) {
  var __ref = useRef(null);

  var __text = useCallback(function __text() {
    return props.viewModel.appointment.text;
  }, [props.viewModel]);

  var __styles = useCallback(function __styles() {
    return getAppointmentStyles(props.viewModel);
  }, [props.viewModel]);

  var __isReduced = useCallback(function __isReduced() {
    var {
      appointmentReduced
    } = props.viewModel.info;
    return !!appointmentReduced;
  }, []);

  var __dateText = useCallback(function __dateText() {
    return props.viewModel.info.dateText;
  }, [props.viewModel]);

  var __data = useCallback(function __data() {
    return {
      appointmentData: props.viewModel.info.appointment,
      targetedAppointmentData: props.viewModel.appointment
    };
  }, [props.viewModel]);

  var __onItemClick = useCallback(function __onItemClick() {
    var e = {
      data: [props.viewModel],
      target: __ref.current,
      index: props.index
    };
    props.onItemClick(e);
  }, [props.viewModel, props.index, props.onItemClick]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __classes = useCallback(function __classes() {
    var {
      allDay,
      appointmentReduced,
      direction,
      isRecurrent
    } = props.viewModel.info;
    var isVerticalDirection = direction === "vertical";
    return combineClasses({
      "dx-scheduler-appointment": true,
      "dx-scheduler-appointment-horizontal": !isVerticalDirection,
      "dx-scheduler-appointment-vertical": isVerticalDirection,
      "dx-scheduler-appointment-recurrence": isRecurrent,
      "dx-scheduler-all-day-appointment": allDay,
      "dx-scheduler-appointment-reduced": __isReduced(),
      "dx-scheduler-appointment-head": appointmentReduced === "head",
      "dx-scheduler-appointment-body": appointmentReduced === "body",
      "dx-scheduler-appointment-tail": appointmentReduced === "tail"
    });
  }, [__isReduced]);

  return viewFunction({
    props: _extends({}, props, {
      appointmentTemplate: getTemplate(props.appointmentTemplate, props.appointmentRender, props.appointmentComponent)
    }),
    ref: __ref,
    text: __text(),
    styles: __styles(),
    isReduced: __isReduced(),
    classes: __classes(),
    dateText: __dateText(),
    data: __data(),
    onItemClick: __onItemClick,
    restAttributes: __restAttributes()
  });
}
Appointment.defaultProps = AppointmentProps;