import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["agendaDuration", "currentDate", "currentView", "currentViewChange", "customizationFunction", "defaultCurrentView", "firstDayOfWeek", "intervalCount", "items", "max", "min", "onCurrentDateUpdate", "onCurrentViewUpdate", "startViewDate", "useDropDownViewSwitcher", "useShortDateFormat", "viewType", "views"];
import devices from 'devextreme/core/devices';
import { Toolbar } from "../../toolbar/toolbar";
import 'devextreme/ui/button_group';
import 'devextreme/ui/drop_down_button';
import dateUtils from 'devextreme/core/utils/date';
import { getCaption, nextWeek, getStep, getViewName, getNextIntervalDate } from 'devextreme/ui/scheduler/header/utils';
import { formToolbarItem, formatViews, isMonthView } from "./utils";
import { SchedulerProps } from "../props";
import { SchedulerCalendar } from "./calendar";
var {
  trimTime
} = dateUtils;
export var viewFunction = viewModel => {
  var {
    currentDate,
    firstDayOfWeek,
    max,
    min
  } = viewModel.props;
  var {
    calendarVisible,
    changeCalendarDate,
    changeCalendarVisible,
    items
  } = viewModel;
  return React.createElement("div", {
    className: "dx-scheduler-header"
  }, React.createElement(SchedulerCalendar, {
    currentDate: currentDate,
    onCurrentDateUpdate: changeCalendarDate,
    min: min,
    max: max,
    firstDayOfWeek: firstDayOfWeek,
    visible: calendarVisible,
    onVisibleUpdate: changeCalendarVisible
  }), React.createElement(Toolbar, {
    items: items
  }));
};
export var SchedulerToolbarBaseProps = {
  intervalCount: 1,
  firstDayOfWeek: 0,
  agendaDuration: 7,

  get useShortDateFormat() {
    return !devices.real().generic || devices.isSimulator();
  },

  viewType: "day"
};
export var SchedulerToolbarProps = {
  intervalCount: SchedulerToolbarBaseProps.intervalCount,
  firstDayOfWeek: SchedulerToolbarBaseProps.firstDayOfWeek,
  agendaDuration: SchedulerToolbarBaseProps.agendaDuration,
  useShortDateFormat: SchedulerToolbarBaseProps.useShortDateFormat,
  viewType: SchedulerToolbarBaseProps.viewType,
  useDropDownViewSwitcher: SchedulerProps.useDropDownViewSwitcher,
  defaultCurrentView: SchedulerProps.currentView,
  currentViewChange: () => {}
};
import { convertRulesToOptions } from 'devextreme/core/options/utils';
import * as React from "react";
import { useState, useCallback, useMemo } from "react";
export function SchedulerToolbar(props) {
  var [__state_currentView, __state_setCurrentView] = useState(() => props.currentView !== undefined ? props.currentView : props.defaultCurrentView);
  var [__state_calendarVisible, __state_setCalendarVisible] = useState(false);

  var __step = useCallback(function __step() {
    return getStep(props.viewType);
  }, [props.viewType]);

  var __displayedDate = useMemo(function __displayedDate() {
    var startViewDate = new Date(props.startViewDate);

    if (isMonthView(props.viewType)) {
      return nextWeek(startViewDate);
    }

    return startViewDate;
  }, [props.startViewDate, props.viewType]);

  var __views = useMemo(function __views() {
    return formatViews(props.views);
  }, [props.views]);

  var __selectedView = useCallback(function __selectedView() {
    return getViewName(props.currentView !== undefined ? props.currentView : __state_currentView);
  }, [props.currentView, __state_currentView]);

  var __setCurrentView = useCallback(function __setCurrentView(view) {
    if (view.name !== (props.currentView !== undefined ? props.currentView : __state_currentView)) {
      props.onCurrentViewUpdate(view.name);
    }
  }, [props.currentView, __state_currentView, props.onCurrentViewUpdate]);

  var __setCurrentDate = useCallback(function __setCurrentDate(date) {
    if (date.getTime() !== props.currentDate.getTime()) {
      props.onCurrentDateUpdate(new Date(date));
    }
  }, [props.currentDate, props.onCurrentDateUpdate]);

  var __changeCalendarVisible = useCallback(function __changeCalendarVisible(visible) {
    __state_setCalendarVisible(__state_calendarVisible => visible);
  }, []);

  var __restAttributes = useCallback(function __restAttributes() {
    var _props$currentView = _extends({}, props, {
      currentView: props.currentView !== undefined ? props.currentView : __state_currentView
    }),
        restProps = _objectWithoutPropertiesLoose(_props$currentView, _excluded);

    return restProps;
  }, [props, __state_currentView]);

  var __caption = useCallback(function __caption() {
    var options = {
      step: __step(),
      intervalCount: props.intervalCount,
      firstDayOfWeek: props.firstDayOfWeek,
      agendaDuration: props.agendaDuration,
      date: trimTime(__displayedDate)
    };
    return getCaption(options, props.useShortDateFormat, props.customizationFunction);
  }, [__step, props.intervalCount, props.firstDayOfWeek, props.agendaDuration, __displayedDate, props.useShortDateFormat, props.customizationFunction]);

  var __intervalOptions = useMemo(function __intervalOptions() {
    return {
      step: __step(),
      intervalCount: props.intervalCount,
      firstDayOfWeek: props.firstDayOfWeek,
      agendaDuration: props.agendaDuration
    };
  }, [__step, props.intervalCount, props.firstDayOfWeek, props.agendaDuration]);

  var __changeCalendarDate = useCallback(function __changeCalendarDate(date) {
    __state_setCalendarVisible(__state_calendarVisible => false);

    __setCurrentDate(date);
  }, [__setCurrentDate]);

  var __showCalendar = useCallback(function __showCalendar() {
    __changeCalendarVisible(true);
  }, [__changeCalendarVisible]);

  var __captionText = useCallback(function __captionText() {
    return __caption().text;
  }, [__caption]);

  var __getNextDate = useCallback(function __getNextDate(direction, initialDate) {
    var date = initialDate !== null && initialDate !== void 0 ? initialDate : props.currentDate;

    var options = _extends({}, __intervalOptions, {
      date
    });

    return getNextIntervalDate(options, direction);
  }, [props.currentDate, __intervalOptions]);

  var __updateDateByDirection = useCallback(function __updateDateByDirection(direction) {
    var date = __getNextDate(direction);

    __setCurrentDate(date);
  }, [__getNextDate, __setCurrentDate]);

  var __isPreviousButtonDisabled = useCallback(function __isPreviousButtonDisabled() {
    if (props.min === undefined) {
      return false;
    }

    var min = trimTime(new Date(props.min));

    var {
      endDate
    } = __caption();

    var previousDate = __getNextDate(-1, endDate);

    return previousDate < min;
  }, [props.min, __caption, __getNextDate]);

  var __isNextButtonDisabled = useCallback(function __isNextButtonDisabled() {
    if (props.max === undefined) {
      return false;
    }

    var max = new Date(new Date(props.max).setHours(23, 59, 59));

    var {
      startDate
    } = __caption();

    var nextDate = __getNextDate(1, startDate);

    return nextDate > max;
  }, [props.max, __caption, __getNextDate]);

  var __items = useMemo(function __items() {
    var options = {
      useDropDownViewSwitcher: props.useDropDownViewSwitcher,
      selectedView: __selectedView(),
      views: __views,
      setCurrentView: view => __setCurrentView(view),
      showCalendar: () => __showCalendar(),
      captionText: __captionText(),
      updateDateByDirection: direction => __updateDateByDirection(direction),
      isPreviousButtonDisabled: __isPreviousButtonDisabled(),
      isNextButtonDisabled: __isNextButtonDisabled()
    };
    return props.items.map(item => formToolbarItem(item, options));
  }, [props.useDropDownViewSwitcher, __selectedView, __views, __setCurrentView, __showCalendar, __captionText, __updateDateByDirection, __isPreviousButtonDisabled, __isNextButtonDisabled, props.items]);

  return viewFunction({
    props: _extends({}, props, {
      currentView: props.currentView !== undefined ? props.currentView : __state_currentView
    }),
    calendarVisible: __state_calendarVisible,
    step: __step(),
    displayedDate: __displayedDate,
    caption: __caption(),
    captionText: __captionText(),
    views: __views,
    selectedView: __selectedView(),
    setCurrentView: __setCurrentView,
    setCurrentDate: __setCurrentDate,
    intervalOptions: __intervalOptions,
    getNextDate: __getNextDate,
    updateDateByDirection: __updateDateByDirection,
    isPreviousButtonDisabled: __isPreviousButtonDisabled,
    isNextButtonDisabled: __isNextButtonDisabled,
    changeCalendarDate: __changeCalendarDate,
    changeCalendarVisible: __changeCalendarVisible,
    showCalendar: __showCalendar,
    items: __items,
    restAttributes: __restAttributes()
  });
}

function __processTwoWayProps(defaultProps) {
  var twoWayProps = ["currentView"];
  return Object.keys(defaultProps).reduce((props, propName) => {
    var propValue = defaultProps[propName];
    var defaultPropName = twoWayProps.some(p => p === propName) ? "default" + propName.charAt(0).toUpperCase() + propName.slice(1) : propName;
    props[defaultPropName] = propValue;
    return props;
  }, {});
}

SchedulerToolbar.defaultProps = SchedulerToolbarProps;
var __defaultOptionRules = [];
export function defaultOptions(rule) {
  __defaultOptionRules.push(rule);

  SchedulerToolbar.defaultProps = Object.create(Object.prototype, _extends(Object.getOwnPropertyDescriptors(SchedulerToolbar.defaultProps), Object.getOwnPropertyDescriptors(__processTwoWayProps(convertRulesToOptions(__defaultOptionRules)))));
}