import { SchedulerToolbarItem } from "./header/props";
import type { template } from 'devextreme/core/templates/template';
import DataSource from 'devextreme/data/data_source';
import type { DataSourceOptions } from 'devextreme/data/data_source';
import type {
    AppointmentDraggingAddEvent,
    AppointmentDraggingEndEvent,
    AppointmentDraggingMoveEvent,
    AppointmentDraggingStartEvent,
    AppointmentDraggingRemoveEvent,
    DateNavigatorTextInfo,
    Appointment,
    AppointmentAddingEvent,
    AppointmentClickEvent,
    AppointmentContextMenuEvent,
    AppointmentDblClickEvent,
    AppointmentDeletedEvent,
    AppointmentDeletingEvent,
    AppointmentFormOpeningEvent,
    AppointmentRenderedEvent,
    AppointmentUpdatedEvent,
    AppointmentUpdatingEvent,
    CellContextMenuEvent,
    AppointmentTooltipTemplateData,
    CellClickEvent,
} from 'devextreme/ui/scheduler';
import type { UserDefinedElement, DxElement } from 'devextreme/core/element';
import { ViewType } from "./types";
import { BaseWidgetProps } from "../common/base_props";
import { DataCellTemplateProps, DateTimeCellTemplateProps, ResourceCellTemplateProps } from "./workspaces/types";
import { AppointmentTemplateProps, OverflowIndicatorTemplateProps } from "./appointment/types";
export declare type ResourcePropsType = {
    allowMultiple?: boolean;
    dataSource?: string | unknown[] | DataSource | DataSourceOptions;
    label?: string;
    useColorAsDefault?: boolean;
    valueExpr?: string | ((data: unknown) => string);
    colorExpr?: string;
    displayExpr?: string | ((resource: unknown) => string);
    fieldExpr?: string;
};
export declare const ResourceProps: ResourcePropsType;
export declare type ViewPropsType = {
    endDayHour?: number;
    startDayHour?: number;
    firstDayOfWeek?: 0 | 1 | 2 | 3 | 4 | 5 | 6;
    cellDuration?: number;
    groups?: string[];
    maxAppointmentsPerCell?: number | "auto" | "unlimited";
    agendaDuration?: number;
    groupByDate?: boolean;
    groupOrientation?: "horizontal" | "vertical";
    intervalCount?: number;
    name?: string;
    startDate?: Date | number | string;
    type?: ViewType;
    scrolling?: typeof ScrollingProps;
    dataCellTemplate?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dateCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    resourceCellTemplate?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    appointmentCollectorTemplate?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    appointmentTemplate?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentTooltipTemplate?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    __defaultNestedValues?: any;
    dataCellRender?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dataCellComponent?: React.JSXElementConstructor<Partial<DataCellTemplateProps>>;
    dateCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    dateCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    timeCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    resourceCellRender?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    resourceCellComponent?: React.JSXElementConstructor<Partial<ResourceCellTemplateProps>>;
    appointmentCollectorRender?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    appointmentCollectorComponent?: React.JSXElementConstructor<Partial<OverflowIndicatorTemplateProps>>;
    appointmentRender?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentComponent?: React.JSXElementConstructor<Partial<AppointmentTemplateProps>>;
    appointmentTooltipRender?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentTooltipComponent?: React.JSXElementConstructor<Partial<AppointmentTemplateProps>>;
    children?: React.ReactNode;
};
export declare const ViewProps: ViewPropsType;
export declare type AppointmentEditingPropsType = {
    allowAdding?: boolean;
    allowDeleting?: boolean;
    allowDragging?: boolean;
    allowResizing?: boolean;
    allowTimeZoneEditing?: boolean;
    allowUpdating?: boolean;
    allowEditingTimeZones?: boolean;
};
export declare const AppointmentEditingProps: AppointmentEditingPropsType;
export declare type AppointmentDraggingPropsType = {
    autoScroll?: boolean;
    data?: unknown;
    group?: string;
    scrollSensitivity?: number;
    scrollSpeed?: number;
    onAdd?: (e: AppointmentDraggingAddEvent) => void;
    onDragEnd?: (e: AppointmentDraggingEndEvent) => void;
    onDragMove?: (e: AppointmentDraggingMoveEvent) => void;
    onDragStart?: (e: AppointmentDraggingStartEvent) => void;
    onRemove?: (e: AppointmentDraggingRemoveEvent) => void;
};
export declare const AppointmentDraggingProps: AppointmentDraggingPropsType;
export declare type ScrollingPropsType = {
    mode?: "standard" | "virtual";
};
export declare const ScrollingProps: ScrollingPropsType;
export declare type SchedulerPropsType = typeof BaseWidgetProps & {
    adaptivityEnabled: boolean;
    appointmentDragging?: typeof AppointmentDraggingProps;
    crossScrollingEnabled: boolean;
    currentDate: Date | number | string;
    currentView: string | "agenda" | "day" | "month" | "timelineDay" | "timelineMonth" | "timelineWeek" | "timelineWorkWeek" | "week" | "workWeek";
    dataSource?: string | Appointment[] | DataSource | DataSourceOptions;
    dateSerializationFormat?: string;
    descriptionExpr: string;
    editing?: typeof AppointmentEditingProps;
    focusStateEnabled: boolean;
    groupByDate: boolean;
    indicatorUpdateInterval: number;
    max?: Date | number | string;
    min?: Date | number | string;
    noDataText: any;
    recurrenceEditMode: "dialog" | "occurrence" | "series";
    remoteFiltering: boolean;
    resources?: typeof ResourceProps[];
    scrolling?: typeof ScrollingProps;
    selectedCellData: unknown[];
    shadeUntilCurrentTime: boolean;
    showAllDayPanel: boolean;
    showCurrentTimeIndicator: boolean;
    timeZone: string;
    useDropDownViewSwitcher: boolean;
    views?: ("day" | "week" | "workWeek" | "month" | "timelineDay" | "timelineWeek" | "timelineWorkWeek" | "timelineMonth" | "agenda" | typeof ViewProps)[];
    endDayHour: number;
    startDayHour: number;
    firstDayOfWeek: number;
    cellDuration: number;
    groups: string[];
    maxAppointmentsPerCell: number | "auto" | "unlimited";
    customizeDateNavigatorText?: (info: DateNavigatorTextInfo) => string;
    onAppointmentAdded?: (e: AppointmentDraggingAddEvent) => void;
    onAppointmentAdding?: (e: AppointmentAddingEvent) => void;
    onAppointmentClick?: ((e: AppointmentClickEvent) => void) | string;
    onAppointmentContextMenu?: ((e: AppointmentContextMenuEvent) => void) | string;
    onAppointmentDblClick?: ((e: AppointmentDblClickEvent) => void) | string;
    onAppointmentDeleted?: (e: AppointmentDeletedEvent) => void;
    onAppointmentDeleting?: (e: AppointmentDeletingEvent) => void;
    onAppointmentFormOpening?: (e: AppointmentFormOpeningEvent) => void;
    onAppointmentRendered?: (e: AppointmentRenderedEvent) => void;
    onAppointmentUpdated?: (e: AppointmentUpdatedEvent) => void;
    onAppointmentUpdating?: (e: AppointmentUpdatingEvent) => void;
    onCellClick?: ((e: CellClickEvent) => void) | string;
    onCellContextMenu?: ((e: CellContextMenuEvent) => void) | string;
    recurrenceExceptionExpr: string;
    recurrenceRuleExpr: string;
    startDateExpr: string;
    startDateTimeZoneExpr: string;
    endDateExpr: string;
    endDateTimeZoneExpr: string;
    allDayExpr: string;
    textExpr: string;
    dataCellTemplate?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dateCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    resourceCellTemplate?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    appointmentCollectorTemplate?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    appointmentTemplate?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentTooltipTemplate?: template | ((model: AppointmentTooltipTemplateData, itemIndex: number, contentElement: DxElement) => string | UserDefinedElement);
    toolbar: typeof SchedulerToolbarItem[];
    __defaultNestedValues?: any;
    defaultCurrentDate: Date | number | string;
    currentDateChange?: (currentDate: Date | number | string) => void;
    defaultCurrentView: string | "agenda" | "day" | "month" | "timelineDay" | "timelineMonth" | "timelineWeek" | "timelineWorkWeek" | "week" | "workWeek";
    currentViewChange?: (currentView: string | "agenda" | "day" | "month" | "timelineDay" | "timelineMonth" | "timelineWeek" | "timelineWorkWeek" | "week" | "workWeek") => void;
    dataCellRender?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dataCellComponent?: React.JSXElementConstructor<Partial<DataCellTemplateProps>>;
    dateCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    dateCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    timeCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    resourceCellRender?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    resourceCellComponent?: React.JSXElementConstructor<Partial<ResourceCellTemplateProps>>;
    appointmentCollectorRender?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    appointmentCollectorComponent?: React.JSXElementConstructor<Partial<OverflowIndicatorTemplateProps>>;
    appointmentRender?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentComponent?: React.JSXElementConstructor<Partial<AppointmentTemplateProps>>;
    appointmentTooltipRender?: template | ((model: AppointmentTooltipTemplateData, itemIndex: number, contentElement: DxElement) => string | UserDefinedElement);
    appointmentTooltipComponent?: template | ((model: AppointmentTooltipTemplateData, itemIndex: number, contentElement: DxElement) => string | UserDefinedElement);
    children?: React.ReactNode;
};
export declare const SchedulerProps: SchedulerPropsType;
export declare type DataAccessorsPropsType = {
    dateSerializationFormat?: string;
    descriptionExpr: string;
    resources?: typeof ResourceProps[];
    recurrenceExceptionExpr: string;
    recurrenceRuleExpr: string;
    startDateExpr: string;
    startDateTimeZoneExpr: string;
    endDateExpr: string;
    endDateTimeZoneExpr: string;
    allDayExpr: string;
    textExpr: string;
    __defaultNestedValues?: any;
    children?: React.ReactNode;
};
export declare const DataAccessorsProps: DataAccessorsPropsType;
export declare type CurrentViewConfigPropsType = {
    crossScrollingEnabled: boolean;
    groupByDate: boolean;
    indicatorUpdateInterval: number;
    scrolling?: typeof ScrollingProps;
    shadeUntilCurrentTime: boolean;
    showAllDayPanel: boolean;
    showCurrentTimeIndicator: boolean;
    endDayHour: number;
    startDayHour: number;
    firstDayOfWeek: number;
    cellDuration: number;
    maxAppointmentsPerCell: number | "auto" | "unlimited";
    dataCellTemplate?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dateCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    resourceCellTemplate?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    appointmentCollectorTemplate?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    appointmentTemplate?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    height?: string | number | (() => string | number);
    width?: string | number | (() => string | number);
    __defaultNestedValues?: any;
    dataCellRender?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dataCellComponent?: React.JSXElementConstructor<Partial<DataCellTemplateProps>>;
    dateCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    dateCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    timeCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    resourceCellRender?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    resourceCellComponent?: React.JSXElementConstructor<Partial<ResourceCellTemplateProps>>;
    appointmentCollectorRender?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    appointmentCollectorComponent?: React.JSXElementConstructor<Partial<OverflowIndicatorTemplateProps>>;
    appointmentRender?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentComponent?: React.JSXElementConstructor<Partial<AppointmentTemplateProps>>;
    children?: React.ReactNode;
};
export declare const CurrentViewConfigProps: CurrentViewConfigPropsType;
