import { TimeZoneCalculator } from "./timeZoneCalculator/utils";
import { Appointment } from 'devextreme/ui/scheduler';
import { ViewProps, SchedulerProps } from "./props";
import { UserDefinedElement } from 'devextreme/core/element';
import DataSource from 'devextreme/data/data_source';
import { CurrentViewConfigType } from "./workspaces/props";
import { Group, ViewMetaData } from "./workspaces/types";
import type { AppointmentDataItem, DataAccessorType } from "./types";
import { AppointmentsViewModelType, AppointmentViewModel, AppointmentClickData } from "./appointment/types";
import { AppointmentsConfigType } from "./model/types";
export declare const viewFunction: ({ appointmentsViewModel, changeTooltipVisible, currentViewConfig, loadedResources, onViewRendered, props: { accessKey, activeStateEnabled, className, currentView, customizeDateNavigatorText, disabled, focusStateEnabled, height, hint, hoverStateEnabled, max, min, rtlEnabled, tabIndex, toolbar: toolbarItems, useDropDownViewSwitcher, views, visible, width, }, restAttributes, setCurrentDate, setCurrentView, showTooltip, startViewDate, tooltipData, tooltipTarget, tooltipVisible, workSpaceKey, }: Scheduler) => any;
import * as React from "react";
import { ResourceProps, AppointmentEditingProps, AppointmentDraggingProps, ScrollingProps } from "./props";
export declare const Resource: React.FunctionComponent<typeof ResourceProps> & {
    propName: string;
};
export declare const View: React.FunctionComponent<typeof ViewProps> & {
    propName: string;
};
export declare const Editing: React.FunctionComponent<typeof AppointmentEditingProps> & {
    propName: string;
};
export declare const AppointmentDragging: React.FunctionComponent<typeof AppointmentDraggingProps> & {
    propName: string;
};
export declare const Scrolling: React.FunctionComponent<typeof ScrollingProps> & {
    propName: string;
};
export declare const ViewScrolling: React.FunctionComponent<typeof ScrollingProps> & {
    propName: string;
};
export declare type SchedulerRef = {
    addAppointment: (_appointment: Appointment) => void;
    deleteAppointment: (_appointment: Appointment) => void;
    updateAppointment: (_target: Appointment, _appointment: Appointment) => void;
    getDataSource: () => DataSource;
    getEndViewDate: () => Date;
    getStartViewDate: () => Date;
    hideAppointmentPopup: (_saveChanges?: boolean) => void;
    hideAppointmentTooltip: () => void;
    scrollTo: (_date: Date, _group?: Record<string, unknown>, _allDay?: boolean) => void;
    scrollToTime: (_hours: number, _minutes: number, _date?: Date) => void;
    showAppointmentPopup: (_appointmentData?: Appointment, _createNewAppointment?: boolean, _currentAppointmentData?: Appointment) => void;
    showAppointmentTooltip: (_appointmentData: Appointment, _target: string | UserDefinedElement, _currentAppointmentData?: Appointment) => void;
};
declare type RestProps = {
    className?: string;
    style?: {
        [name: string]: any;
    };
    key?: any;
    ref?: any;
};
interface Scheduler {
    props: typeof SchedulerProps & RestProps;
    __getNestedViews: ("day" | "week" | "workWeek" | "month" | "timelineDay" | "timelineWeek" | "timelineWorkWeek" | "timelineMonth" | "agenda" | typeof ViewProps)[];
    __getNestedScrolling: typeof ScrollingProps;
    __getNestedResources: typeof ResourceProps[];
    __getNestedEditing: typeof AppointmentEditingProps;
    __getNestedAppointmentDragging: typeof AppointmentDraggingProps | undefined;
    workSpaceViewModel?: ViewMetaData;
    resourcePromisesMap: Map<string, Promise<Group[]>>;
    loadedResources?: Group[];
    dataItems: Appointment[];
    tooltipTarget: HTMLElement;
    tooltipVisible: boolean;
    tooltipData: AppointmentViewModel[];
    lastViewDateByEndDayHour?: Date;
    currentViewProps: Partial<typeof ViewProps>;
    currentViewConfig: typeof CurrentViewConfigType;
    isValidViewDataProvider: boolean;
    dataAccessors: DataAccessorType;
    startViewDate: Date;
    isVirtualScrolling: boolean;
    timeZoneCalculator: TimeZoneCalculator;
    internalDataSource: DataSource;
    appointmentsConfig: AppointmentsConfigType | undefined;
    preparedDataItems: AppointmentDataItem[];
    filteredItems: Appointment[];
    appointmentsViewModel: AppointmentsViewModelType;
    workSpaceKey: string;
    onViewRendered: (viewMetaData: ViewMetaData) => void;
    setCurrentView: (view: string) => void;
    setCurrentDate: (date: Date) => void;
    showTooltip: (e: AppointmentClickData) => void;
    hideTooltip: () => void;
    changeTooltipVisible: (value: boolean) => void;
    restAttributes: RestProps;
}
declare const Scheduler: React.FC<import("../common/base_props").BaseWidgetPropsType & {
    adaptivityEnabled: boolean;
    appointmentDragging?: import("./props").AppointmentDraggingPropsType;
    crossScrollingEnabled: boolean;
    currentDate: string | number | Date;
    currentView: string;
    dataSource?: string | DataSource<any, any> | import('devextreme/ui/scheduler').dxSchedulerAppointment[] | import('devextreme/data/data_source').DataSourceOptions<any, any, any, any>;
    dateSerializationFormat?: string;
    descriptionExpr: string;
    editing?: import("./props").AppointmentEditingPropsType;
    focusStateEnabled: boolean;
    groupByDate: boolean;
    indicatorUpdateInterval: number;
    max?: string | number | Date;
    min?: string | number | Date;
    noDataText: any;
    recurrenceEditMode: "dialog" | "occurrence" | "series";
    remoteFiltering: boolean;
    resources?: import("./props").ResourcePropsType[];
    scrolling?: import("./props").ScrollingPropsType;
    selectedCellData: unknown[];
    shadeUntilCurrentTime: boolean;
    showAllDayPanel: boolean;
    showCurrentTimeIndicator: boolean;
    timeZone: string;
    useDropDownViewSwitcher: boolean;
    views?: ("agenda" | "day" | "month" | "timelineDay" | "timelineMonth" | "timelineWeek" | "timelineWorkWeek" | "week" | "workWeek" | import("./props").ViewPropsType)[];
    endDayHour: number;
    startDayHour: number;
    firstDayOfWeek: number;
    cellDuration: number;
    groups: string[];
    maxAppointmentsPerCell: number | "auto" | "unlimited";
    customizeDateNavigatorText?: (info: import('devextreme/ui/scheduler').DateNavigatorTextInfo) => string;
    onAppointmentAdded?: (e: import('devextreme/ui/scheduler').AppointmentDraggingAddEvent) => void;
    onAppointmentAdding?: (e: import('devextreme/ui/scheduler').AppointmentAddingEvent) => void;
    onAppointmentClick?: string | ((e: import('devextreme/ui/scheduler').AppointmentClickEvent) => void);
    onAppointmentContextMenu?: string | ((e: import('devextreme/ui/scheduler').AppointmentContextMenuEvent) => void);
    onAppointmentDblClick?: string | ((e: import('devextreme/ui/scheduler').AppointmentDblClickEvent) => void);
    onAppointmentDeleted?: (e: import('devextreme/ui/scheduler').AppointmentDeletedEvent) => void;
    onAppointmentDeleting?: (e: import('devextreme/ui/scheduler').AppointmentDeletingEvent) => void;
    onAppointmentFormOpening?: (e: import('devextreme/ui/scheduler').AppointmentFormOpeningEvent) => void;
    onAppointmentRendered?: (e: import('devextreme/ui/scheduler').AppointmentRenderedEvent) => void;
    onAppointmentUpdated?: (e: import('devextreme/ui/scheduler').AppointmentUpdatedEvent) => void;
    onAppointmentUpdating?: (e: import('devextreme/ui/scheduler').AppointmentUpdatingEvent) => void;
    onCellClick?: string | ((e: import('devextreme/ui/scheduler').CellClickEvent) => void);
    onCellContextMenu?: string | ((e: import('devextreme/ui/scheduler').CellContextMenuEvent) => void);
    recurrenceExceptionExpr: string;
    recurrenceRuleExpr: string;
    startDateExpr: string;
    startDateTimeZoneExpr: string;
    endDateExpr: string;
    endDateTimeZoneExpr: string;
    allDayExpr: string;
    textExpr: string;
    dataCellTemplate?: React.FunctionComponent<Partial<import("./workspaces/types").DataCellTemplateProps>>;
    dateCellTemplate?: React.FunctionComponent<Partial<import("./workspaces/types").DateTimeCellTemplateProps>>;
    timeCellTemplate?: React.FunctionComponent<Partial<import("./workspaces/types").DateTimeCellTemplateProps>>;
    resourceCellTemplate?: React.FunctionComponent<Partial<import("./workspaces/types").ResourceCellTemplateProps>>;
    appointmentCollectorTemplate?: React.FunctionComponent<Partial<import("./appointment/types").OverflowIndicatorTemplateProps>>;
    appointmentTemplate?: React.FunctionComponent<Partial<import("./appointment/types").AppointmentTemplateProps>>;
    appointmentTooltipTemplate?: import('devextreme/core/templates/template').template | ((model: import('devextreme/ui/scheduler').TargetedAppointmentInfo, itemIndex: number, contentElement: HTMLElement) => string | Element);
    toolbar: import("./header/props").SchedulerToolbarItemType[];
    __defaultNestedValues?: any;
    defaultCurrentDate: string | number | Date;
    currentDateChange?: (currentDate: string | number | Date) => void;
    defaultCurrentView: string;
    currentViewChange?: (currentView: string) => void;
    dataCellRender?: React.FunctionComponent<Partial<import("./workspaces/types").DataCellTemplateProps>>;
    dataCellComponent?: React.JSXElementConstructor<Partial<import("./workspaces/types").DataCellTemplateProps>>;
    dateCellRender?: React.FunctionComponent<Partial<import("./workspaces/types").DateTimeCellTemplateProps>>;
    dateCellComponent?: React.JSXElementConstructor<Partial<import("./workspaces/types").DateTimeCellTemplateProps>>;
    timeCellRender?: React.FunctionComponent<Partial<import("./workspaces/types").DateTimeCellTemplateProps>>;
    timeCellComponent?: React.JSXElementConstructor<Partial<import("./workspaces/types").DateTimeCellTemplateProps>>;
    resourceCellRender?: React.FunctionComponent<Partial<import("./workspaces/types").ResourceCellTemplateProps>>;
    resourceCellComponent?: React.JSXElementConstructor<Partial<import("./workspaces/types").ResourceCellTemplateProps>>;
    appointmentCollectorRender?: React.FunctionComponent<Partial<import("./appointment/types").OverflowIndicatorTemplateProps>>;
    appointmentCollectorComponent?: React.JSXElementConstructor<Partial<import("./appointment/types").OverflowIndicatorTemplateProps>>;
    appointmentRender?: React.FunctionComponent<Partial<import("./appointment/types").AppointmentTemplateProps>>;
    appointmentComponent?: React.JSXElementConstructor<Partial<import("./appointment/types").AppointmentTemplateProps>>;
    appointmentTooltipRender?: import('devextreme/core/templates/template').template | ((model: import('devextreme/ui/scheduler').TargetedAppointmentInfo, itemIndex: number, contentElement: HTMLElement) => string | Element);
    appointmentTooltipComponent?: import('devextreme/core/templates/template').template | ((model: import('devextreme/ui/scheduler').TargetedAppointmentInfo, itemIndex: number, contentElement: HTMLElement) => string | Element);
    children?: React.ReactNode;
} & RestProps & {
    ref?: React.Ref<SchedulerRef>;
}> & {
    defaultProps: typeof SchedulerProps;
};
export { Scheduler };
export default Scheduler;
