import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["__defaultNestedValues", "accessKey", "activeStateEnabled", "adaptivityEnabled", "allDayExpr", "appointmentCollectorComponent", "appointmentCollectorRender", "appointmentCollectorTemplate", "appointmentComponent", "appointmentDragging", "appointmentRender", "appointmentTemplate", "appointmentTooltipComponent", "appointmentTooltipRender", "appointmentTooltipTemplate", "cellDuration", "children", "className", "crossScrollingEnabled", "currentDate", "currentDateChange", "currentView", "currentViewChange", "customizeDateNavigatorText", "dataCellComponent", "dataCellRender", "dataCellTemplate", "dataSource", "dateCellComponent", "dateCellRender", "dateCellTemplate", "dateSerializationFormat", "defaultCurrentDate", "defaultCurrentView", "descriptionExpr", "disabled", "editing", "endDateExpr", "endDateTimeZoneExpr", "endDayHour", "firstDayOfWeek", "focusStateEnabled", "groupByDate", "groups", "height", "hint", "hoverStateEnabled", "indicatorUpdateInterval", "max", "maxAppointmentsPerCell", "min", "noDataText", "onAppointmentAdded", "onAppointmentAdding", "onAppointmentClick", "onAppointmentContextMenu", "onAppointmentDblClick", "onAppointmentDeleted", "onAppointmentDeleting", "onAppointmentFormOpening", "onAppointmentRendered", "onAppointmentUpdated", "onAppointmentUpdating", "onCellClick", "onCellContextMenu", "onClick", "onKeyDown", "recurrenceEditMode", "recurrenceExceptionExpr", "recurrenceRuleExpr", "remoteFiltering", "resourceCellComponent", "resourceCellRender", "resourceCellTemplate", "resources", "rtlEnabled", "scrolling", "selectedCellData", "shadeUntilCurrentTime", "showAllDayPanel", "showCurrentTimeIndicator", "startDateExpr", "startDateTimeZoneExpr", "startDayHour", "tabIndex", "textExpr", "timeCellComponent", "timeCellRender", "timeCellTemplate", "timeZone", "toolbar", "useDropDownViewSwitcher", "views", "visible", "width"];
import { ViewProps, SchedulerProps } from "./props";
import { Widget } from "../common/widget";
import DataSource from 'devextreme/data/data_source';
import { getCurrentViewConfig, getCurrentViewProps, getValidGroups } from "./model/views";
import { WorkSpace } from "./workspaces/base/work_space";
import { SchedulerToolbar } from "./header/header";
import { getViewDataGeneratorByViewType } from 'devextreme/ui/scheduler/workspaces/view_model/utils';
import { createDataAccessors, createTimeZoneCalculator, isViewDataProviderConfigValid } from "./common";
import { getGroupCount, loadResources } from 'devextreme/ui/scheduler/resources/utils';
import { getAppointmentsViewModel } from "./view_model/appointments/appointments";
import { getAppointmentsConfig, getAppointmentsModel } from "./model/appointments";
import { AppointmentLayout } from "./appointment/layout";
import { AppointmentTooltip } from "./appointment/tooltip/appointment_tooltip";
import { getViewRenderConfigByType } from "./workspaces/base/work_space_config";
import { getPreparedDataItems, resolveDataItems } from "./utils/data";
import { getFilterStrategy } from "./utils/filtering/local";
import combineRemoteFilter from "./utils/filtering/remote";
export var viewFunction = _ref => {
  var {
    appointmentsViewModel,
    changeTooltipVisible,
    currentViewConfig,
    loadedResources,
    onViewRendered,
    props: {
      accessKey,
      activeStateEnabled,
      className,
      currentView,
      customizeDateNavigatorText,
      disabled,
      focusStateEnabled,
      height,
      hint,
      hoverStateEnabled,
      max,
      min,
      rtlEnabled,
      tabIndex,
      toolbar: toolbarItems,
      useDropDownViewSwitcher,
      views,
      visible,
      width
    },
    restAttributes,
    setCurrentDate,
    setCurrentView,
    showTooltip,
    startViewDate,
    tooltipData,
    tooltipTarget,
    tooltipVisible,
    workSpaceKey
  } = _ref;
  var {
    allDayPanelExpanded,
    allowMultipleCellSelection,
    appointmentCollectorTemplate,
    appointmentTemplate,
    cellDuration,
    crossScrollingEnabled,
    currentDate,
    dataCellTemplate,
    dateCellTemplate,
    endDayHour,
    firstDayOfWeek,
    groupByDate,
    groupOrientation,
    hoursInterval,
    indicatorUpdateInterval,
    intervalCount,
    resourceCellTemplate,
    scrolling,
    shadeUntilCurrentTime,
    showAllDayPanel,
    showCurrentTimeIndicator,
    startDate,
    startDayHour,
    timeCellTemplate,
    type
  } = currentViewConfig;
  return React.createElement(Widget, _extends({
    classes: "dx-scheduler dx-scheduler-native",
    accessKey: accessKey,
    activeStateEnabled: activeStateEnabled,
    disabled: disabled,
    focusStateEnabled: focusStateEnabled,
    height: height,
    hint: hint,
    hoverStateEnabled: hoverStateEnabled,
    rtlEnabled: rtlEnabled,
    tabIndex: tabIndex,
    visible: visible,
    width: width,
    className: className
  }, restAttributes), React.createElement("div", {
    className: "dx-scheduler-container"
  }, toolbarItems.length !== 0 && React.createElement(SchedulerToolbar, {
    items: toolbarItems,
    views: views,
    currentView: currentView,
    onCurrentViewUpdate: setCurrentView,
    currentDate: currentDate,
    onCurrentDateUpdate: setCurrentDate,
    startViewDate: startViewDate,
    min: min,
    max: max,
    intervalCount: intervalCount,
    firstDayOfWeek: firstDayOfWeek,
    useDropDownViewSwitcher: useDropDownViewSwitcher,
    customizationFunction: customizeDateNavigatorText,
    viewType: type
  }), React.createElement(WorkSpace, {
    firstDayOfWeek: firstDayOfWeek,
    startDayHour: startDayHour,
    endDayHour: endDayHour,
    cellDuration: cellDuration,
    groupByDate: groupByDate,
    scrolling: scrolling,
    currentDate: currentDate,
    intervalCount: intervalCount,
    groupOrientation: groupOrientation,
    startDate: startDate,
    showAllDayPanel: showAllDayPanel,
    showCurrentTimeIndicator: showCurrentTimeIndicator,
    indicatorUpdateInterval: indicatorUpdateInterval,
    shadeUntilCurrentTime: shadeUntilCurrentTime,
    crossScrollingEnabled: crossScrollingEnabled,
    hoursInterval: hoursInterval,
    groups: loadedResources,
    type: type,
    schedulerHeight: height,
    schedulerWidth: width,
    allowMultipleCellSelection: allowMultipleCellSelection,
    allDayPanelExpanded: allDayPanelExpanded,
    onViewRendered: onViewRendered,
    dataCellTemplate: dataCellTemplate,
    timeCellTemplate: timeCellTemplate,
    dateCellTemplate: dateCellTemplate,
    resourceCellTemplate: resourceCellTemplate,
    allDayAppointments: React.createElement(AppointmentLayout, {
      isAllDay: true,
      appointments: appointmentsViewModel.allDay,
      overflowIndicators: appointmentsViewModel.allDayCompact,
      appointmentTemplate: appointmentTemplate,
      overflowIndicatorTemplate: appointmentCollectorTemplate,
      onAppointmentClick: showTooltip
    }),
    appointments: React.createElement(AppointmentLayout, {
      appointments: appointmentsViewModel.regular,
      overflowIndicators: appointmentsViewModel.regularCompact,
      appointmentTemplate: appointmentTemplate,
      overflowIndicatorTemplate: appointmentCollectorTemplate,
      onAppointmentClick: showTooltip
    }),
    key: workSpaceKey
  }), React.createElement(AppointmentTooltip, {
    visible: tooltipVisible,
    onVisibleChange: changeTooltipVisible,
    target: tooltipTarget,
    dataList: tooltipData
  })));
};
import { __collectChildren, equalByValue } from "@devextreme/runtime/react";
import * as React from "react";
import { useState, useCallback, useMemo, useEffect, useRef, useImperativeHandle, forwardRef } from "react";
import { getTemplate } from "@devextreme/runtime/react";
import { ResourceProps, AppointmentEditingProps, AppointmentDraggingProps, ScrollingProps } from "./props";
export var Resource = () => null;
Resource.propName = "resources";
Resource.defaultProps = ResourceProps;
export var View = () => null;
View.propName = "views";
View.defaultProps = ViewProps;
export var Editing = () => null;
Editing.propName = "editing";
Editing.defaultProps = AppointmentEditingProps;
export var AppointmentDragging = () => null;
AppointmentDragging.propName = "appointmentDragging";
AppointmentDragging.defaultProps = AppointmentDraggingProps;
export var Scrolling = () => null;
Scrolling.propName = "scrolling";
Scrolling.defaultProps = ScrollingProps;
export var ViewScrolling = () => null;
ViewScrolling.propName = "scrolling";
ViewScrolling.defaultProps = ScrollingProps;
var Scheduler = forwardRef(function scheduler(props, ref) {
  var [__state_currentDate, __state_setCurrentDate] = useState(() => props.currentDate !== undefined ? props.currentDate : props.defaultCurrentDate);
  var [__state_currentView, __state_setCurrentView] = useState(() => props.currentView !== undefined ? props.currentView : props.defaultCurrentView);
  var [__state_workSpaceViewModel, __state_setWorkSpaceViewModel] = useState(undefined);
  var [__state_resourcePromisesMap, __state_setResourcePromisesMap] = useState(new Map());
  var [__state_loadedResources, __state_setLoadedResources] = useState(undefined);
  var [__state_dataItems, __state_setDataItems] = useState([]);
  var [__state_tooltipTarget, __state_setTooltipTarget] = useState(undefined);
  var [__state_tooltipVisible, __state_setTooltipVisible] = useState(false);
  var [__state_tooltipData, __state_setTooltipData] = useState([]);
  var [__state_lastViewDateByEndDayHour, __state_setLastViewDateByEndDayHour] = useState(undefined);
  var cachedNested = useRef(__collectChildren(props.children));

  var __getNestedViews = useMemo(function __getNestedViews() {
    var _props$__defaultNeste;

    var nested = __collectChildren(props.children);

    if (!equalByValue(cachedNested.current, nested)) cachedNested.current = nested;
    return props.views ? props.views : cachedNested.current.views ? cachedNested.current.views : props === null || props === void 0 ? void 0 : (_props$__defaultNeste = props.__defaultNestedValues) === null || _props$__defaultNeste === void 0 ? void 0 : _props$__defaultNeste.views;
  }, [props.views, props.children]);

  var __getNestedScrolling = useMemo(function __getNestedScrolling() {
    var _cachedNested$current, _props$__defaultNeste2;

    var nested = __collectChildren(props.children);

    if (!equalByValue(cachedNested.current, nested)) cachedNested.current = nested;
    return props.scrolling ? props.scrolling : cachedNested.current.scrolling ? (_cachedNested$current = cachedNested.current.scrolling) === null || _cachedNested$current === void 0 ? void 0 : _cachedNested$current[0] : props === null || props === void 0 ? void 0 : (_props$__defaultNeste2 = props.__defaultNestedValues) === null || _props$__defaultNeste2 === void 0 ? void 0 : _props$__defaultNeste2.scrolling;
  }, [props.scrolling, props.children]);

  var __getNestedResources = useMemo(function __getNestedResources() {
    var _props$__defaultNeste3;

    var nested = __collectChildren(props.children);

    if (!equalByValue(cachedNested.current, nested)) cachedNested.current = nested;
    return props.resources ? props.resources : cachedNested.current.resources ? cachedNested.current.resources : props === null || props === void 0 ? void 0 : (_props$__defaultNeste3 = props.__defaultNestedValues) === null || _props$__defaultNeste3 === void 0 ? void 0 : _props$__defaultNeste3.resources;
  }, [props.resources, props.children]);

  var __getNestedEditing = useMemo(function __getNestedEditing() {
    var _cachedNested$current2, _props$__defaultNeste4;

    var nested = __collectChildren(props.children);

    if (!equalByValue(cachedNested.current, nested)) cachedNested.current = nested;
    return props.editing ? props.editing : cachedNested.current.editing ? (_cachedNested$current2 = cachedNested.current.editing) === null || _cachedNested$current2 === void 0 ? void 0 : _cachedNested$current2[0] : props === null || props === void 0 ? void 0 : (_props$__defaultNeste4 = props.__defaultNestedValues) === null || _props$__defaultNeste4 === void 0 ? void 0 : _props$__defaultNeste4.editing;
  }, [props.editing, props.children]);

  var __getNestedAppointmentDragging = useMemo(function __getNestedAppointmentDragging() {
    var _cachedNested$current3;

    var nested = __collectChildren(props.children);

    if (!equalByValue(cachedNested.current, nested)) cachedNested.current = nested;
    return props.appointmentDragging ? props.appointmentDragging : cachedNested.current.appointmentDragging ? (_cachedNested$current3 = cachedNested.current.appointmentDragging) === null || _cachedNested$current3 === void 0 ? void 0 : _cachedNested$current3[0] : undefined;
  }, [props.appointmentDragging, props.children]);

  var __currentViewProps = useCallback(function __currentViewProps() {
    return getCurrentViewProps(props.currentView !== undefined ? props.currentView : __state_currentView, __getNestedViews);
  }, [props.currentView, __state_currentView, __getNestedViews]);

  var __dataAccessors = useMemo(function __dataAccessors() {
    return createDataAccessors({
      startDateExpr: props.startDateExpr,
      endDateExpr: props.endDateExpr,
      startDateTimeZoneExpr: props.startDateTimeZoneExpr,
      endDateTimeZoneExpr: props.endDateTimeZoneExpr,
      allDayExpr: props.allDayExpr,
      textExpr: props.textExpr,
      descriptionExpr: props.descriptionExpr,
      recurrenceRuleExpr: props.recurrenceRuleExpr,
      recurrenceExceptionExpr: props.recurrenceExceptionExpr,
      resources: __getNestedResources
    });
  }, [props.startDateExpr, props.endDateExpr, props.startDateTimeZoneExpr, props.endDateTimeZoneExpr, props.allDayExpr, props.textExpr, props.descriptionExpr, props.recurrenceRuleExpr, props.recurrenceExceptionExpr, __getNestedResources]);

  var __timeZoneCalculator = useMemo(function __timeZoneCalculator() {
    return createTimeZoneCalculator(props.timeZone);
  }, [props.timeZone]);

  var __internalDataSource = useCallback(function __internalDataSource() {
    if (props.dataSource instanceof DataSource) {
      return props.dataSource;
    }

    if (props.dataSource instanceof Array) {
      return new DataSource({
        store: {
          type: "array",
          data: props.dataSource
        },
        paginate: false
      });
    }

    return new DataSource(props.dataSource);
  }, [props.dataSource]);

  var __setCurrentView = useCallback(function __setCurrentView(view) {
    __state_setCurrentView(__state_currentView => view), props.currentViewChange(view);
  }, [props.currentViewChange]);

  var __setCurrentDate = useCallback(function __setCurrentDate(date) {
    __state_setCurrentDate(__state_currentDate => date), props.currentDateChange(date);
  }, [props.currentDateChange]);

  var __changeTooltipVisible = useCallback(function __changeTooltipVisible(value) {
    __state_setTooltipVisible(__state_tooltipVisible => value);
  }, []);

  var __restAttributes = useCallback(function __restAttributes() {
    var _props$appointmentDra = _extends({}, props, {
      appointmentDragging: __getNestedAppointmentDragging,
      currentDate: props.currentDate !== undefined ? props.currentDate : __state_currentDate,
      currentView: props.currentView !== undefined ? props.currentView : __state_currentView,
      editing: __getNestedEditing,
      resources: __getNestedResources,
      scrolling: __getNestedScrolling,
      views: __getNestedViews
    }),
        restProps = _objectWithoutPropertiesLoose(_props$appointmentDra, _excluded);

    return restProps;
  }, [props, __state_currentDate, __state_currentView]);

  var __addAppointment = useCallback(function __addAppointment(_appointment) {}, []);

  var __deleteAppointment = useCallback(function __deleteAppointment(_appointment) {}, []);

  var __updateAppointment = useCallback(function __updateAppointment(_target, _appointment) {}, []);

  var __getEndViewDate = useCallback(function __getEndViewDate() {
    return __state_workSpaceViewModel.viewDataProvider.getLastCellEndDate();
  }, [__state_workSpaceViewModel]);

  var __hideAppointmentPopup = useCallback(function __hideAppointmentPopup(_saveChanges) {}, []);

  var __scrollTo = useCallback(function __scrollTo(_date, _group, _allDay) {}, []);

  var __scrollToTime = useCallback(function __scrollToTime(_hours, _minutes, _date) {}, []);

  var __showAppointmentPopup = useCallback(function __showAppointmentPopup(_appointmentData, _createNewAppointment, _currentAppointmentData) {}, []);

  var __showAppointmentTooltip = useCallback(function __showAppointmentTooltip(_appointmentData, _target, _currentAppointmentData) {}, []);

  var __currentViewConfig = useMemo(function __currentViewConfig() {
    var {
      appointmentCollectorTemplate,
      appointmentTemplate,
      cellDuration,
      crossScrollingEnabled,
      dataCellTemplate,
      dateCellTemplate,
      endDayHour,
      firstDayOfWeek,
      groupByDate,
      height,
      indicatorUpdateInterval,
      maxAppointmentsPerCell,
      resourceCellTemplate,
      shadeUntilCurrentTime,
      showAllDayPanel,
      showCurrentTimeIndicator,
      startDayHour,
      timeCellTemplate,
      width
    } = props;
    return getCurrentViewConfig(__currentViewProps(), {
      firstDayOfWeek,
      startDayHour,
      endDayHour,
      cellDuration,
      groupByDate,
      scrolling: __getNestedScrolling,
      dataCellTemplate,
      timeCellTemplate,
      resourceCellTemplate,
      dateCellTemplate,
      appointmentTemplate,
      appointmentCollectorTemplate,
      maxAppointmentsPerCell,
      showAllDayPanel,
      showCurrentTimeIndicator,
      indicatorUpdateInterval,
      shadeUntilCurrentTime,
      crossScrollingEnabled,
      height,
      width
    }, props.currentDate !== undefined ? props.currentDate : __state_currentDate);
  }, [props.appointmentCollectorTemplate, props.appointmentTemplate, props.cellDuration, props.crossScrollingEnabled, props.dataCellTemplate, props.dateCellTemplate, props.endDayHour, props.firstDayOfWeek, props.groupByDate, props.height, props.indicatorUpdateInterval, props.maxAppointmentsPerCell, props.resourceCellTemplate, props.shadeUntilCurrentTime, props.showAllDayPanel, props.showCurrentTimeIndicator, props.startDayHour, props.timeCellTemplate, props.width, props.currentDate, __state_currentDate, __getNestedScrolling, __currentViewProps]);

  var __isVirtualScrolling = useCallback(function __isVirtualScrolling() {
    var _currentViewProps$sc;

    return __getNestedScrolling.mode === "virtual" || ((_currentViewProps$sc = __currentViewProps().scrolling) === null || _currentViewProps$sc === void 0 ? void 0 : _currentViewProps$sc.mode) === "virtual";
  }, [__getNestedScrolling, __currentViewProps]);

  var __showTooltip = useCallback(function __showTooltip(e) {
    __state_setTooltipData(__state_tooltipData => e.data);

    __state_setTooltipTarget(__state_tooltipTarget => e.target);

    __changeTooltipVisible(true);
  }, [__changeTooltipVisible]);

  var __hideTooltip = useCallback(function __hideTooltip() {
    __changeTooltipVisible(false);
  }, [__changeTooltipVisible]);

  var __getDataSource = useCallback(function __getDataSource() {
    return __internalDataSource();
  }, [__internalDataSource]);

  var __isValidViewDataProvider = useCallback(function __isValidViewDataProvider() {
    var {
      allDayPanelExpanded,
      cellDuration,
      crossScrollingEnabled,
      currentDate,
      endDayHour,
      firstDayOfWeek,
      groupByDate,
      groupOrientation,
      hoursInterval,
      intervalCount,
      scrolling,
      showAllDayPanel,
      startDate,
      startDayHour,
      type
    } = __currentViewConfig;
    return isViewDataProviderConfigValid(__state_workSpaceViewModel === null || __state_workSpaceViewModel === void 0 ? void 0 : __state_workSpaceViewModel.viewDataProviderValidationOptions, {
      intervalCount: intervalCount !== null && intervalCount !== void 0 ? intervalCount : 1,
      currentDate,
      type,
      hoursInterval,
      startDayHour,
      endDayHour,
      groupOrientation,
      groupByDate,
      crossScrollingEnabled,
      firstDayOfWeek,
      startDate,
      showAllDayPanel,
      allDayPanelExpanded,
      scrolling,
      cellDuration,
      groups: __state_loadedResources
    });
  }, [__currentViewConfig, __state_workSpaceViewModel, __state_loadedResources]);

  var __startViewDate = useMemo(function __startViewDate() {
    var {
      currentDate,
      firstDayOfWeek,
      intervalCount,
      startDate,
      startDayHour,
      type
    } = __currentViewConfig;
    var options = {
      currentDate,
      startDayHour,
      startDate,
      intervalCount,
      firstDayOfWeek
    };
    var viewDataGenerator = getViewDataGeneratorByViewType(type);
    var startViewDate = viewDataGenerator.getStartViewDate(options);
    return startViewDate;
  }, [__currentViewConfig]);

  var __preparedDataItems = useMemo(function __preparedDataItems() {
    return getPreparedDataItems(__state_dataItems, __dataAccessors, __currentViewConfig.cellDuration, __timeZoneCalculator);
  }, [__state_dataItems, __dataAccessors, __currentViewConfig, __timeZoneCalculator]);

  var __workSpaceKey = useCallback(function __workSpaceKey() {
    var {
      crossScrollingEnabled,
      groupOrientation,
      intervalCount
    } = __currentViewConfig;

    if (!crossScrollingEnabled) {
      return "";
    }

    var groupCount = getGroupCount(__state_loadedResources !== null && __state_loadedResources !== void 0 ? __state_loadedResources : []);
    return "".concat(props.currentView !== undefined ? props.currentView : __state_currentView, "_").concat(groupOrientation, "_").concat(intervalCount, "_").concat(groupCount);
  }, [props.currentView, __state_currentView, __currentViewConfig, __state_loadedResources]);

  var __onViewRendered = useCallback(function __onViewRendered(viewMetaData) {
    __state_setWorkSpaceViewModel(__state_workSpaceViewModel => viewMetaData);

    var {
      viewDataProvider
    } = viewMetaData;
    var lastViewDate = viewDataProvider.getLastViewDateByEndDayHour(__currentViewConfig.endDayHour);

    if (lastViewDate.getTime() !== (__state_lastViewDateByEndDayHour === null || __state_lastViewDateByEndDayHour === void 0 ? void 0 : __state_lastViewDateByEndDayHour.getTime())) {
      __state_setLastViewDateByEndDayHour(__state_lastViewDateByEndDayHour => lastViewDate);
    }
  }, [__currentViewConfig, __state_lastViewDateByEndDayHour]);

  var __hideAppointmentTooltip = useCallback(function __hideAppointmentTooltip() {
    __hideTooltip();
  }, [__hideTooltip]);

  var __appointmentsConfig = useMemo(function __appointmentsConfig() {
    if (!__isValidViewDataProvider() || !__state_loadedResources) {
      return undefined;
    }

    var renderConfig = getViewRenderConfigByType(__currentViewConfig.type, __currentViewConfig.crossScrollingEnabled, __currentViewConfig.intervalCount, __state_loadedResources, __currentViewConfig.groupOrientation);
    var validGroups = getValidGroups(props.groups, __currentViewProps().groups);
    return getAppointmentsConfig({
      adaptivityEnabled: props.adaptivityEnabled,
      rtlEnabled: props.rtlEnabled,
      resources: __getNestedResources,
      timeZone: props.timeZone,
      groups: validGroups
    }, {
      startDayHour: __currentViewConfig.startDayHour,
      endDayHour: __currentViewConfig.endDayHour,
      currentDate: __currentViewConfig.currentDate,
      scrolling: __currentViewConfig.scrolling,
      intervalCount: __currentViewConfig.intervalCount,
      hoursInterval: __currentViewConfig.hoursInterval,
      showAllDayPanel: __currentViewConfig.showAllDayPanel,
      firstDayOfWeek: __currentViewConfig.firstDayOfWeek,
      type: __currentViewConfig.type,
      cellDuration: __currentViewConfig.cellDuration,
      maxAppointmentsPerCell: __currentViewConfig.maxAppointmentsPerCell
    }, __state_loadedResources, __state_workSpaceViewModel.viewDataProvider, renderConfig.isAllDayPanelSupported);
  }, [__isValidViewDataProvider, __state_loadedResources, __currentViewConfig, props.groups, __currentViewProps, props.adaptivityEnabled, props.rtlEnabled, __getNestedResources, props.timeZone, __state_workSpaceViewModel]);

  var __getStartViewDate = useCallback(function __getStartViewDate() {
    return __startViewDate;
  }, [__startViewDate]);

  var __filteredItems = useMemo(function __filteredItems() {
    if (!__appointmentsConfig) {
      return [];
    }

    var filterStrategy = getFilterStrategy(__appointmentsConfig.resources, __appointmentsConfig.startDayHour, __appointmentsConfig.endDayHour, __appointmentsConfig.cellDurationInMinutes, __appointmentsConfig.showAllDayPanel, __appointmentsConfig.supportAllDayRow, __appointmentsConfig.firstDayOfWeek, __appointmentsConfig.viewType, __appointmentsConfig.dateRange, __appointmentsConfig.groupCount, __appointmentsConfig.loadedResources, __appointmentsConfig.isVirtualScrolling, __timeZoneCalculator, __dataAccessors, __state_workSpaceViewModel.viewDataProvider);
    return filterStrategy.filter(__preparedDataItems);
  }, [__appointmentsConfig, __timeZoneCalculator, __dataAccessors, __state_workSpaceViewModel, __preparedDataItems]);

  var __appointmentsViewModel = useMemo(function __appointmentsViewModel() {
    if (!__appointmentsConfig || __filteredItems.length === 0) {
      return {
        allDay: [],
        allDayCompact: [],
        regular: [],
        regularCompact: []
      };
    }

    var model = getAppointmentsModel(__appointmentsConfig, __state_workSpaceViewModel.viewDataProvider, __timeZoneCalculator, __dataAccessors, __state_workSpaceViewModel.cellsMetaData);
    return getAppointmentsViewModel(model, __filteredItems);
  }, [__appointmentsConfig, __filteredItems, __state_workSpaceViewModel, __timeZoneCalculator, __dataAccessors]);

  useEffect(() => {
    var validGroups = getValidGroups(props.groups, __currentViewProps().groups);
    loadResources(validGroups, __getNestedResources, __state_resourcePromisesMap).then(loadedResources => {
      __state_setLoadedResources(__state_loadedResources => loadedResources);
    });
  }, [props.groups, __currentViewProps, __getNestedResources, __state_resourcePromisesMap]);
  useEffect(() => {
    if (!__internalDataSource().isLoaded() && !__internalDataSource().isLoading()) {
      if (props.remoteFiltering && __state_lastViewDateByEndDayHour) {
        var combinedFilter = combineRemoteFilter({
          dataAccessors: __dataAccessors,
          dataSourceFilter: __internalDataSource().filter(),
          min: __startViewDate,
          max: __state_lastViewDateByEndDayHour,
          dateSerializationFormat: props.dateSerializationFormat
        });

        __internalDataSource().filter(combinedFilter);
      }

      __internalDataSource().load().done(loadOptions => {
        __state_setDataItems(__state_dataItems => resolveDataItems(loadOptions));
      });
    }
  }, [__internalDataSource, props.remoteFiltering, __state_lastViewDateByEndDayHour, __dataAccessors, __startViewDate, props.dateSerializationFormat]);
  useImperativeHandle(ref, () => ({
    addAppointment: __addAppointment,
    deleteAppointment: __deleteAppointment,
    updateAppointment: __updateAppointment,
    getDataSource: __getDataSource,
    getEndViewDate: __getEndViewDate,
    getStartViewDate: __getStartViewDate,
    hideAppointmentPopup: __hideAppointmentPopup,
    hideAppointmentTooltip: __hideAppointmentTooltip,
    scrollTo: __scrollTo,
    scrollToTime: __scrollToTime,
    showAppointmentPopup: __showAppointmentPopup,
    showAppointmentTooltip: __showAppointmentTooltip
  }), [__addAppointment, __deleteAppointment, __updateAppointment, __getDataSource, __getEndViewDate, __getStartViewDate, __hideAppointmentPopup, __hideAppointmentTooltip, __scrollTo, __scrollToTime, __showAppointmentPopup, __showAppointmentTooltip]);
  return viewFunction({
    props: _extends({}, props, {
      currentDate: props.currentDate !== undefined ? props.currentDate : __state_currentDate,
      currentView: props.currentView !== undefined ? props.currentView : __state_currentView,
      dataCellTemplate: getTemplate(props.dataCellTemplate, props.dataCellRender, props.dataCellComponent),
      dateCellTemplate: getTemplate(props.dateCellTemplate, props.dateCellRender, props.dateCellComponent),
      timeCellTemplate: getTemplate(props.timeCellTemplate, props.timeCellRender, props.timeCellComponent),
      resourceCellTemplate: getTemplate(props.resourceCellTemplate, props.resourceCellRender, props.resourceCellComponent),
      appointmentCollectorTemplate: getTemplate(props.appointmentCollectorTemplate, props.appointmentCollectorRender, props.appointmentCollectorComponent),
      appointmentTemplate: getTemplate(props.appointmentTemplate, props.appointmentRender, props.appointmentComponent),
      appointmentTooltipTemplate: getTemplate(props.appointmentTooltipTemplate, props.appointmentTooltipRender, props.appointmentTooltipComponent),
      appointmentDragging: __getNestedAppointmentDragging,
      editing: __getNestedEditing,
      resources: __getNestedResources,
      scrolling: __getNestedScrolling,
      views: __getNestedViews
    }),
    workSpaceViewModel: __state_workSpaceViewModel,
    resourcePromisesMap: __state_resourcePromisesMap,
    loadedResources: __state_loadedResources,
    dataItems: __state_dataItems,
    tooltipTarget: __state_tooltipTarget,
    tooltipVisible: __state_tooltipVisible,
    tooltipData: __state_tooltipData,
    lastViewDateByEndDayHour: __state_lastViewDateByEndDayHour,
    __getNestedViews,
    __getNestedScrolling,
    __getNestedResources,
    __getNestedEditing,
    __getNestedAppointmentDragging,
    currentViewProps: __currentViewProps(),
    currentViewConfig: __currentViewConfig,
    isValidViewDataProvider: __isValidViewDataProvider(),
    dataAccessors: __dataAccessors,
    startViewDate: __startViewDate,
    isVirtualScrolling: __isVirtualScrolling(),
    timeZoneCalculator: __timeZoneCalculator,
    internalDataSource: __internalDataSource(),
    appointmentsConfig: __appointmentsConfig,
    preparedDataItems: __preparedDataItems,
    filteredItems: __filteredItems,
    appointmentsViewModel: __appointmentsViewModel,
    workSpaceKey: __workSpaceKey(),
    onViewRendered: __onViewRendered,
    setCurrentView: __setCurrentView,
    setCurrentDate: __setCurrentDate,
    showTooltip: __showTooltip,
    hideTooltip: __hideTooltip,
    changeTooltipVisible: __changeTooltipVisible,
    restAttributes: __restAttributes()
  });
});
export { Scheduler };
export default Scheduler;
Scheduler.defaultProps = SchedulerProps;