import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["bottomVirtualRowHeight", "children", "className", "height", "leftVirtualCellCount", "leftVirtualCellWidth", "rightVirtualCellCount", "rightVirtualCellWidth", "tableRef", "topVirtualRowHeight", "virtualCellsCount", "width"];
import { addHeightToStyle, addWidthToStyle } from "../utils";
import { VirtualRow } from "./virtual_row";
export var viewFunction = _ref => {
  var {
    hasBottomVirtualRow,
    hasTopVirtualRow,
    props: {
      bottomVirtualRowHeight,
      children,
      className,
      leftVirtualCellCount,
      leftVirtualCellWidth,
      rightVirtualCellCount,
      rightVirtualCellWidth,
      tableRef,
      topVirtualRowHeight,
      virtualCellsCount
    },
    style
  } = _ref;
  return React.createElement("table", {
    className: className,
    style: normalizeStyles(style),
    ref: tableRef
  }, React.createElement("tbody", null, hasTopVirtualRow && React.createElement(VirtualRow, {
    height: topVirtualRowHeight,
    cellsCount: virtualCellsCount,
    leftVirtualCellWidth: leftVirtualCellWidth,
    rightVirtualCellWidth: rightVirtualCellWidth,
    leftVirtualCellCount: leftVirtualCellCount,
    rightVirtualCellCount: rightVirtualCellCount
  }), children, hasBottomVirtualRow && React.createElement(VirtualRow, {
    height: bottomVirtualRowHeight,
    cellsCount: virtualCellsCount,
    leftVirtualCellWidth: leftVirtualCellWidth,
    rightVirtualCellWidth: rightVirtualCellWidth,
    leftVirtualCellCount: leftVirtualCellCount,
    rightVirtualCellCount: rightVirtualCellCount
  })));
};
export var TableProps = {
  className: "",
  topVirtualRowHeight: 0,
  bottomVirtualRowHeight: 0,
  leftVirtualCellWidth: 0,
  rightVirtualCellWidth: 0,
  virtualCellsCount: 0
};
import * as React from "react";
import { useCallback, useRef } from "react";
var NUMBER_STYLES = new Set(["animationIterationCount", "borderImageOutset", "borderImageSlice", "border-imageWidth", "boxFlex", "boxFlexGroup", "boxOrdinalGroup", "columnCount", "fillOpacity", "flex", "flexGrow", "flexNegative", "flexOrder", "flexPositive", "flexShrink", "floodOpacity", "fontWeight", "gridColumn", "gridRow", "lineClamp", "lineHeight", "opacity", "order", "orphans", "stopOpacity", "strokeDasharray", "strokeDashoffset", "strokeMiterlimit", "strokeOpacity", "strokeWidth", "tabSize", "widows", "zIndex", "zoom"]);

var isNumeric = value => {
  if (typeof value === "number") return true;
  return !isNaN(Number(value));
};

var getNumberStyleValue = (style, value) => {
  return NUMBER_STYLES.has(style) ? value : "".concat(value, "px");
};

var normalizeStyles = styles => {
  if (!(styles instanceof Object)) return undefined;
  return Object.entries(styles).reduce((result, _ref2) => {
    var [key, value] = _ref2;
    result[key] = isNumeric(value) ? getNumberStyleValue(key, value) : value;
    return result;
  }, {});
};

export function Table(props) {
  var __elementRef = useRef(null);

  var __hasTopVirtualRow = useCallback(function __hasTopVirtualRow() {
    var {
      topVirtualRowHeight
    } = props;
    return !!topVirtualRowHeight;
  }, [props.topVirtualRowHeight]);

  var __hasBottomVirtualRow = useCallback(function __hasBottomVirtualRow() {
    var {
      bottomVirtualRowHeight
    } = props;
    return !!bottomVirtualRowHeight;
  }, [props.bottomVirtualRowHeight]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __style = useCallback(function __style() {
    var {
      height,
      width
    } = props;

    var {
      style
    } = __restAttributes();

    var heightAdded = addHeightToStyle(height, style);
    return addWidthToStyle(width, heightAdded);
  }, [props.height, props.width, __restAttributes]);

  return viewFunction({
    props: _extends({}, props),
    elementRef: __elementRef,
    style: __style(),
    hasTopVirtualRow: __hasTopVirtualRow(),
    hasBottomVirtualRow: __hasBottomVirtualRow(),
    restAttributes: __restAttributes()
  });
}
Table.defaultProps = TableProps;