import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["accessKey", "activeStateEnabled", "allDayAppointments", "allDayPanelExpanded", "allowMultipleCellSelection", "appointmentCollectorComponent", "appointmentCollectorRender", "appointmentCollectorTemplate", "appointmentComponent", "appointmentRender", "appointmentTemplate", "appointments", "cellDuration", "className", "crossScrollingEnabled", "currentDate", "dataCellComponent", "dataCellRender", "dataCellTemplate", "dateCellComponent", "dateCellRender", "dateCellTemplate", "disabled", "endDayHour", "firstDayOfWeek", "focusStateEnabled", "groupByDate", "groupOrientation", "groups", "height", "hint", "hoursInterval", "hoverStateEnabled", "indicatorTime", "indicatorUpdateInterval", "intervalCount", "maxAppointmentsPerCell", "onClick", "onKeyDown", "onViewRendered", "resourceCellComponent", "resourceCellRender", "resourceCellTemplate", "rtlEnabled", "schedulerHeight", "schedulerWidth", "scrolling", "selectedCellData", "shadeUntilCurrentTime", "showAllDayPanel", "showCurrentTimeIndicator", "startDate", "startDayHour", "tabIndex", "timeCellComponent", "timeCellRender", "timeCellTemplate", "type", "visible", "width"];
import { subscribeToDXPointerDownEvent, subscribeToDXPointerMoveEvent, subscribeToScrollEvent } from "../../../../utils/subscribe_to_event";
import { combineClasses } from "../../../../utils/combine_classes";
import { OrdinaryLayout } from "./ordinary_layout";
import { VirtualScrollingDispatcher } from 'devextreme/ui/scheduler/workspaces/ui.scheduler.virtual_scrolling';
import ViewDataProvider from 'devextreme/ui/scheduler/workspaces/view_model/view_data_provider';
import { createCellElementMetaData, createVirtualScrollingOptions, DATE_TABLE_MIN_CELL_WIDTH, getCellIndices, getDateTableWidth, getSelectedCells, isCellAllDay } from "./utils";
import { WorkSpaceProps } from "../props";
import { getViewRenderConfigByType } from "./work_space_config";
import { isGroupingByDate, isHorizontalGroupingApplied, isVerticalGroupingApplied } from "../utils";
import { CrossScrollingLayout } from "./cross_scrolling_layout";
import { getViewDataGeneratorByViewType } from 'devextreme/ui/scheduler/workspaces/view_model/utils';
import { calculateIsGroupedAllDayPanel } from "../../view_model/to_test/views/utils/base";
import { DateHeaderDataGenerator } from 'devextreme/ui/scheduler/workspaces/view_model/date_header_data_generator';
import { TimePanelDataGenerator } from 'devextreme/ui/scheduler/workspaces/view_model/time_panel_data_generator';
import { CellsSelectionController } from 'devextreme/ui/scheduler/workspaces/cells_selection_controller';
import { getGroupPanelData } from "../../view_model/group_panel/utils";
import { getWindow } from 'devextreme/core/utils/window';
import domAdapter from 'devextreme/core/dom_adapter';
import { ConfigContext } from "../../../../common/config_context";
import pointerEvents from 'devextreme/events/pointer';
import eventsEngine from 'devextreme/events/core/events_engine';
import { isMouseEvent } from 'devextreme/events/utils';
import { ALL_DAY_PANEL_CELL_CLASS, DATE_TABLE_CELL_CLASS } from "../const";
import { DiagnosticUtils } from "../../../../utils/diagnostic";
var DATA_CELL_SELECTOR = ".".concat(DATE_TABLE_CELL_CLASS, ", .").concat(ALL_DAY_PANEL_CELL_CLASS);
var defaultVirtualScrollingMetaData = {
  cellHeight: 50,
  cellWidth: DATE_TABLE_MIN_CELL_WIDTH,
  viewWidth: 300,
  viewHeight: 300,
  scrollableWidth: 300,
  windowHeight: 300,
  windowWidth: 300
};

var calculateDefaultVirtualScrollingState = options => {
  var completeColumnCount = options.completeViewDataMap[0].length;
  var completeRowCount = options.completeViewDataMap.length;
  options.virtualScrollingDispatcher.setViewOptions(createVirtualScrollingOptions({
    cellHeight: defaultVirtualScrollingMetaData.cellHeight,
    cellWidth: defaultVirtualScrollingMetaData.cellWidth,
    schedulerHeight: options.schedulerHeight,
    schedulerWidth: options.schedulerWidth,
    viewHeight: defaultVirtualScrollingMetaData.viewHeight,
    viewWidth: defaultVirtualScrollingMetaData.viewWidth,
    scrolling: options.scrolling,
    scrollableWidth: defaultVirtualScrollingMetaData.scrollableWidth,
    groups: options.groups,
    isVerticalGrouping: options.isVerticalGrouping,
    completeRowCount,
    completeColumnCount,
    windowHeight: defaultVirtualScrollingMetaData.windowHeight,
    windowWidth: defaultVirtualScrollingMetaData.windowWidth,
    rtlEnabled: options.rtlEnabled
  }));
  options.virtualScrollingDispatcher.createVirtualScrolling();
  options.virtualScrollingDispatcher.updateDimensions(true);
  return options.virtualScrollingDispatcher.getRenderState();
};

export var prepareGenerationOptions = (workSpaceProps, renderConfig, isAllDayPanelVisible, virtualStartIndices) => {
  var {
    cellDuration,
    currentDate,
    endDayHour,
    firstDayOfWeek,
    groupByDate,
    groupOrientation,
    groups,
    hoursInterval,
    intervalCount,
    startDate,
    startDayHour,
    type
  } = workSpaceProps;
  var {
    getDateForHeaderText,
    headerCellTextFormat,
    isGenerateWeekDaysHeaderData,
    isProvideVirtualCellsWidth,
    isRenderTimePanel
  } = renderConfig;
  return {
    startRowIndex: virtualStartIndices.startRowIndex,
    startCellIndex: virtualStartIndices.startCellIndex,
    groupOrientation,
    groupByDate,
    groups,
    isProvideVirtualCellsWidth,
    isAllDayPanelVisible,
    selectedCells: undefined,
    focusedCell: undefined,
    headerCellTextFormat,
    getDateForHeaderText,
    startDayHour,
    endDayHour,
    cellDuration,
    viewType: type,
    intervalCount,
    hoursInterval,
    currentDate,
    startDate,
    firstDayOfWeek,
    isGenerateTimePanelData: isRenderTimePanel,
    isGenerateWeekDaysHeaderData
  };
};
export var viewFunction = _ref => {
  var {
    allDayPanelRef,
    classes,
    dateHeaderData,
    dateTableRef,
    groupOrientation,
    groupPanelData,
    groupPanelHeight,
    groupPanelRef,
    headerEmptyCellWidth,
    isAllDayPanelVisible,
    isGroupedByDate,
    isRenderHeaderEmptyCell,
    isStandaloneAllDayPanel,
    isVerticalGrouping,
    layoutRef,
    onScrollableScroll,
    props: {
      allDayAppointments,
      allDayPanelExpanded,
      appointments,
      dataCellTemplate,
      dateCellTemplate,
      groups,
      intervalCount,
      resourceCellTemplate,
      timeCellTemplate
    },
    renderConfig: {
      groupPanelClassName,
      isCreateCrossScrolling,
      isRenderDateHeader,
      isRenderTimePanel,
      isUseMonthDateTable,
      isUseTimelineHeader,
      scrollingDirection
    },
    tablesWidth,
    timePanelData,
    timePanelRef,
    viewData,
    widgetElementRef
  } = _ref;
  var Layout = isCreateCrossScrolling ? CrossScrollingLayout : OrdinaryLayout;
  return React.createElement(Layout, {
    ref: layoutRef,
    viewData: viewData,
    dateHeaderData: dateHeaderData,
    timePanelData: timePanelData,
    groupPanelData: groupPanelData,
    dataCellTemplate: dataCellTemplate,
    dateCellTemplate: dateCellTemplate,
    timeCellTemplate: timeCellTemplate,
    resourceCellTemplate: resourceCellTemplate,
    groups: groups,
    groupByDate: isGroupedByDate,
    groupOrientation: groupOrientation,
    groupPanelClassName: groupPanelClassName,
    intervalCount: intervalCount,
    isUseMonthDateTable: isUseMonthDateTable,
    isUseTimelineHeader: isUseTimelineHeader,
    isRenderTimePanel: isRenderTimePanel,
    isAllDayPanelCollapsed: !allDayPanelExpanded,
    isAllDayPanelVisible: isAllDayPanelVisible,
    isRenderDateHeader: isRenderDateHeader,
    isRenderHeaderEmptyCell: isRenderHeaderEmptyCell,
    isRenderGroupPanel: isVerticalGrouping,
    isStandaloneAllDayPanel: isStandaloneAllDayPanel,
    scrollingDirection: scrollingDirection,
    groupPanelHeight: groupPanelHeight,
    headerEmptyCellWidth: headerEmptyCellWidth,
    tablesWidth: tablesWidth,
    onScroll: onScrollableScroll,
    className: classes,
    dateTableRef: dateTableRef,
    allDayPanelRef: allDayPanelRef,
    timePanelRef: timePanelRef,
    groupPanelRef: groupPanelRef,
    widgetElementRef: widgetElementRef,
    appointments: appointments,
    allDayAppointments: allDayAppointments
  });
};
import * as React from "react";
import { useState, useContext, useCallback, useMemo, useEffect, useRef } from "react";
import { getTemplate } from "@devextreme/runtime/react";
export function WorkSpace(props) {
  var __dateTableRef = useRef(null);

  var __allDayPanelRef = useRef(null);

  var __timePanelRef = useRef(null);

  var __groupPanelRef = useRef(null);

  var __widgetElementRef = useRef(null);

  var __layoutRef = useRef(null);

  var [__state_groupPanelHeight, __state_setGroupPanelHeight] = useState(undefined);
  var [__state_headerEmptyCellWidth, __state_setHeaderEmptyCellWidth] = useState(undefined);
  var [__state_tablesWidth, __state_setTablesWidth] = useState(undefined);
  var [__state_virtualScrolling, __state_setVirtualScrolling] = useState(new VirtualScrollingDispatcher());
  var [__state_virtualScrollingData, __state_setVirtualScrollingData] = useState(undefined);
  var [__state_cellsSelectionState, __state_setCellsSelectionState] = useState(null);
  var [__state_isPointerDown, __state_setIsPointerDown] = useState(false);
  var config = useContext(ConfigContext);

  var __renderConfig = useMemo(function __renderConfig() {
    return getViewRenderConfigByType(props.type, props.crossScrollingEnabled, props.intervalCount, props.groups, props.groupOrientation);
  }, [props.type, props.crossScrollingEnabled, props.intervalCount, props.groups, props.groupOrientation]);

  var __viewDataGenerator = useMemo(function __viewDataGenerator() {
    return getViewDataGeneratorByViewType(props.type);
  }, [props.type]);

  var __startViewDate = useMemo(function __startViewDate() {
    var {
      currentDate,
      firstDayOfWeek,
      intervalCount,
      startDate,
      startDayHour,
      type
    } = props;
    var options = {
      currentDate,
      startDayHour,
      startDate,
      intervalCount,
      firstDayOfWeek
    };
    var viewDataGenerator = getViewDataGeneratorByViewType(type);
    var startViewDate = viewDataGenerator.getStartViewDate(options);
    return startViewDate;
  }, [props.currentDate, props.firstDayOfWeek, props.intervalCount, props.startDate, props.startDayHour, props.type]);

  var __isWorkSpaceWithOddCells = useCallback(function __isWorkSpaceWithOddCells() {
    return false;
  }, []);

  var __createDateTableElementsMeta = useCallback(function __createDateTableElementsMeta(totalCellCount) {
    var dateTableCells = __dateTableRef.current.querySelectorAll("td:not(.dx-scheduler-virtual-cell)");

    var dateTableRect = __dateTableRef.current.getBoundingClientRect();

    var dateTableCellsMeta = [];
    dateTableCells.forEach((cellElement, index) => {
      if (index % totalCellCount === 0) {
        dateTableCellsMeta.push([]);
      }

      var cellRect = cellElement.getBoundingClientRect();
      var validCellRect = createCellElementMetaData(dateTableRect, cellRect);
      dateTableCellsMeta[dateTableCellsMeta.length - 1].push(validCellRect);
    });
    return dateTableCellsMeta;
  }, []);

  var __createAllDayPanelElementsMeta = useCallback(function __createAllDayPanelElementsMeta() {
    if (!__allDayPanelRef.current) {
      return [];
    }

    var allDayPanelCells = __allDayPanelRef.current.querySelectorAll("td");

    var allDayPanelRect = __allDayPanelRef.current.getBoundingClientRect();

    var allDayPanelCellsMeta = [];
    allDayPanelCells.forEach(cellElement => {
      var cellRect = cellElement.getBoundingClientRect();
      allDayPanelCellsMeta.push(createCellElementMetaData(allDayPanelRect, cellRect));
    });
    return allDayPanelCellsMeta;
  }, []);

  var __onScroll = useCallback(function __onScroll(scrollOffset) {
    __state_virtualScrolling.handleOnScrollEvent(scrollOffset);

    var nextState = __state_virtualScrolling.getRenderState();

    var isUpdateState = Object.entries(nextState).some(_ref2 => {
      var [key, value] = _ref2;
      return value !== __state_virtualScrollingData.state[key];
    });

    if (isUpdateState) {
      __state_setVirtualScrollingData(__state_virtualScrollingData => ({
        state: nextState,
        sizes: __state_virtualScrollingData.sizes
      }));
    }
  }, [__state_virtualScrolling, __state_virtualScrollingData]);

  var __onPointerUp = useCallback(function __onPointerUp(e) {
    if (isMouseEvent(e) && e.button === 0) {
      __state_setIsPointerDown(__state_isPointerDown => false);
    }
  }, []);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __groupOrientation = useCallback(function __groupOrientation() {
    var {
      groupOrientation
    } = props;
    var {
      defaultGroupOrientation
    } = __renderConfig;
    return groupOrientation !== null && groupOrientation !== void 0 ? groupOrientation : defaultGroupOrientation;
  }, [props.groupOrientation, __renderConfig]);

  var __isAllDayPanelVisible = useCallback(function __isAllDayPanelVisible() {
    var {
      showAllDayPanel
    } = props;
    var {
      isAllDayPanelSupported
    } = __renderConfig;
    return isAllDayPanelSupported && showAllDayPanel;
  }, [props.showAllDayPanel, __renderConfig]);

  var __dateHeaderDataGenerator = useMemo(function __dateHeaderDataGenerator() {
    return new DateHeaderDataGenerator(__viewDataGenerator);
  }, [__viewDataGenerator]);

  var __timePanelDataGenerator = useMemo(function __timePanelDataGenerator() {
    return new TimePanelDataGenerator(__viewDataGenerator);
  }, [__viewDataGenerator]);

  var __isCalculateTablesWidth = useCallback(function __isCalculateTablesWidth() {
    return props.crossScrollingEnabled && __renderConfig.defaultGroupOrientation !== "vertical";
  }, [props.crossScrollingEnabled, __renderConfig]);

  var __onWindowScroll = useCallback(function __onWindowScroll() {
    var {
      scrollX,
      scrollY
    } = getWindow();

    __onScroll({
      top: scrollY,
      left: scrollX
    });
  }, [__onScroll]);

  var __onScrollableScroll = useCallback(function __onScrollableScroll(event) {
    if (props.scrolling.mode === "virtual") {
      __onScroll(event.scrollOffset);
    }
  }, [props.scrolling, __onScroll]);

  var __isVerticalGrouping = useCallback(function __isVerticalGrouping() {
    return isVerticalGroupingApplied(props.groups, __groupOrientation());
  }, [props.groups, __groupOrientation]);

  var __isHorizontalGrouping = useCallback(function __isHorizontalGrouping() {
    return isHorizontalGroupingApplied(props.groups, __groupOrientation());
  }, [props.groups, __groupOrientation]);

  var __isGroupedByDate = useCallback(function __isGroupedByDate() {
    return isGroupingByDate(props.groups, __groupOrientation(), props.groupByDate);
  }, [props.groups, __groupOrientation, props.groupByDate]);

  var __isStandaloneAllDayPanel = useCallback(function __isStandaloneAllDayPanel() {
    var {
      groups
    } = props;
    return !isVerticalGroupingApplied(groups, __groupOrientation()) && __isAllDayPanelVisible();
  }, [props.groups, __groupOrientation, __isAllDayPanelVisible]);

  var __completeViewDataMap = useMemo(function __completeViewDataMap() {
    var {
      cellDuration,
      currentDate,
      endDayHour,
      firstDayOfWeek,
      groupByDate,
      groups,
      hoursInterval,
      intervalCount,
      startDate,
      startDayHour,
      type
    } = props;
    return __viewDataGenerator.getCompleteViewDataMap({
      currentDate,
      startDate,
      startDayHour,
      endDayHour,
      groupByDate,
      groups,
      intervalCount,
      firstDayOfWeek,
      hoursInterval,
      cellDuration,
      startViewDate: __startViewDate,
      groupOrientation: __groupOrientation(),
      isVerticalGrouping: __isVerticalGrouping(),
      isHorizontalGrouping: __isHorizontalGrouping(),
      isGroupedByDate: __isGroupedByDate(),
      isAllDayPanelVisible: __isAllDayPanelVisible(),
      viewType: type,
      interval: __viewDataGenerator.getInterval(hoursInterval)
    });
  }, [props.cellDuration, props.currentDate, props.endDayHour, props.firstDayOfWeek, props.groupByDate, props.groups, props.hoursInterval, props.intervalCount, props.startDate, props.startDayHour, props.type, __viewDataGenerator, __startViewDate, __groupOrientation, __isVerticalGrouping, __isHorizontalGrouping, __isGroupedByDate, __isAllDayPanelVisible]);

  var __groupPanelData = useMemo(function __groupPanelData() {
    var {
      currentDate,
      endDayHour,
      groups,
      hoursInterval,
      intervalCount,
      startDayHour,
      type
    } = props;

    var columnCountPerGroup = __viewDataGenerator.getCellCount({
      intervalCount,
      hoursInterval,
      currentDate,
      startDayHour,
      endDayHour,
      viewType: type
    });

    var groupPanelData = getGroupPanelData(groups, columnCountPerGroup, __isGroupedByDate(), __isGroupedByDate() ? 1 : columnCountPerGroup);
    return groupPanelData;
  }, [props.currentDate, props.endDayHour, props.groups, props.hoursInterval, props.intervalCount, props.startDayHour, props.type, __viewDataGenerator, __isGroupedByDate]);

  var __isRenderHeaderEmptyCell = useCallback(function __isRenderHeaderEmptyCell() {
    return __isVerticalGrouping() || !!__renderConfig.isRenderTimePanel;
  }, [__isVerticalGrouping, __renderConfig]);

  var __classes = useCallback(function __classes() {
    var {
      allDayPanelExpanded,
      groups,
      intervalCount,
      scrolling
    } = props;
    return combineClasses({
      [__renderConfig.className]: true,
      "dx-scheduler-work-space-count": intervalCount > 1,
      "dx-scheduler-work-space-odd-cells": !!__isWorkSpaceWithOddCells(),
      "dx-scheduler-work-space-all-day-collapsed": !allDayPanelExpanded && __isAllDayPanelVisible(),
      "dx-scheduler-work-space-all-day": __isAllDayPanelVisible(),
      "dx-scheduler-work-space-group-by-date": __isGroupedByDate(),
      "dx-scheduler-work-space-grouped": groups.length > 0,
      "dx-scheduler-work-space-vertical-grouped": __isVerticalGrouping() && __renderConfig.defaultGroupOrientation !== "vertical",
      "dx-scheduler-work-space-horizontal-grouped": isHorizontalGroupingApplied(groups, __groupOrientation()) && __renderConfig.defaultGroupOrientation === "vertical",
      "dx-scheduler-group-column-count-one": __isVerticalGrouping() && groups.length === 1,
      "dx-scheduler-group-column-count-two": __isVerticalGrouping() && groups.length === 2,
      "dx-scheduler-group-column-count-three": __isVerticalGrouping() && groups.length === 3,
      "dx-scheduler-work-space-both-scrollbar": props.crossScrollingEnabled,
      "dx-scheduler-work-space-virtual": scrolling.mode === "virtual",
      "dx-scheduler-work-space": true
    });
  }, [props.allDayPanelExpanded, props.groups, props.intervalCount, props.scrolling, __renderConfig, __isWorkSpaceWithOddCells, __isAllDayPanelVisible, __isGroupedByDate, __isVerticalGrouping, __groupOrientation, props.crossScrollingEnabled]);

  var __correctedVirtualScrollingState = useMemo(function __correctedVirtualScrollingState() {
    var result = __state_virtualScrollingData === null || __state_virtualScrollingData === void 0 ? void 0 : __state_virtualScrollingData.state;

    if (!result) {
      var {
        groups,
        schedulerHeight,
        schedulerWidth,
        scrolling
      } = props;
      result = calculateDefaultVirtualScrollingState({
        virtualScrollingDispatcher: __state_virtualScrolling,
        scrolling,
        groups,
        completeViewDataMap: __completeViewDataMap,
        isVerticalGrouping: __isVerticalGrouping(),
        schedulerHeight,
        schedulerWidth,
        rtlEnabled: false
      });
    }

    return _extends({
      startCellIndex: 0,
      startRowIndex: 0
    }, result);
  }, [__state_virtualScrollingData, props.groups, props.schedulerHeight, props.schedulerWidth, props.scrolling, __state_virtualScrolling, __completeViewDataMap, __isVerticalGrouping]);

  var __completeDateHeaderData = useMemo(function __completeDateHeaderData() {
    var {
      currentDate,
      endDayHour,
      groups,
      hoursInterval,
      intervalCount,
      startDayHour,
      type: viewType
    } = props;
    return __dateHeaderDataGenerator.getCompleteDateHeaderMap({
      isGenerateWeekDaysHeaderData: __renderConfig.isGenerateWeekDaysHeaderData,
      isGroupedByDate: __isGroupedByDate(),
      groups,
      groupOrientation: __groupOrientation(),
      isHorizontalGrouping: __isHorizontalGrouping(),
      startDayHour,
      endDayHour,
      hoursInterval,
      intervalCount,
      headerCellTextFormat: __renderConfig.headerCellTextFormat,
      getDateForHeaderText: __renderConfig.getDateForHeaderText,
      interval: __viewDataGenerator.getInterval(hoursInterval),
      startViewDate: __startViewDate,
      currentDate,
      viewType,
      today: new Date()
    }, __completeViewDataMap);
  }, [props.currentDate, props.endDayHour, props.groups, props.hoursInterval, props.intervalCount, props.startDayHour, props.type, __dateHeaderDataGenerator, __renderConfig, __isGroupedByDate, __groupOrientation, __isHorizontalGrouping, __viewDataGenerator, __startViewDate, __completeViewDataMap]);

  var __completeTimePanelData = useMemo(function __completeTimePanelData() {
    if (!__renderConfig.isRenderTimePanel) {
      return undefined;
    }

    var {
      cellDuration,
      currentDate,
      endDayHour,
      hoursInterval,
      intervalCount,
      startDayHour,
      type
    } = props;
    return __timePanelDataGenerator.getCompleteTimePanelMap({
      startViewDate: __startViewDate,
      cellDuration,
      startDayHour,
      endDayHour,
      isVerticalGrouping: __isVerticalGrouping(),
      intervalCount,
      currentDate,
      viewType: type,
      hoursInterval
    }, __completeViewDataMap);
  }, [__renderConfig, props.cellDuration, props.currentDate, props.endDayHour, props.hoursInterval, props.intervalCount, props.startDayHour, props.type, __timePanelDataGenerator, __startViewDate, __isVerticalGrouping, __completeViewDataMap]);

  var __viewDataMap = useMemo(function __viewDataMap() {
    return __viewDataGenerator.generateViewDataMap(__completeViewDataMap, _extends({}, __correctedVirtualScrollingState, {
      isVerticalGrouping: __isVerticalGrouping(),
      isAllDayPanelVisible: __isAllDayPanelVisible()
    }));
  }, [__viewDataGenerator, __completeViewDataMap, __correctedVirtualScrollingState, __isVerticalGrouping, __isAllDayPanelVisible]);

  var __dateHeaderData = useMemo(function __dateHeaderData() {
    var {
      endDayHour,
      groups,
      hoursInterval,
      startDayHour
    } = props;
    return __dateHeaderDataGenerator.generateDateHeaderData(__completeDateHeaderData, __completeViewDataMap, _extends({
      isGenerateWeekDaysHeaderData: __renderConfig.isGenerateWeekDaysHeaderData,
      isProvideVirtualCellsWidth: __renderConfig.isProvideVirtualCellsWidth,
      isMonthDateHeader: __renderConfig.isMonthDateHeader,
      startDayHour,
      endDayHour,
      hoursInterval,
      groups,
      groupOrientation: __groupOrientation(),
      isGroupedByDate: __isGroupedByDate()
    }, __correctedVirtualScrollingState));
  }, [props.endDayHour, props.groups, props.hoursInterval, props.startDayHour, __dateHeaderDataGenerator, __completeDateHeaderData, __completeViewDataMap, __renderConfig, __groupOrientation, __isGroupedByDate, __correctedVirtualScrollingState]);

  var __timePanelData = useMemo(function __timePanelData() {
    if (!__completeTimePanelData) {
      return undefined;
    }

    return __timePanelDataGenerator.generateTimePanelData(__completeTimePanelData, _extends({
      isGroupedAllDayPanel: calculateIsGroupedAllDayPanel(props.groups, __groupOrientation(), __isAllDayPanelVisible()),
      isVerticalGrouping: __isVerticalGrouping(),
      isAllDayPanelVisible: __isAllDayPanelVisible()
    }, __correctedVirtualScrollingState));
  }, [__completeTimePanelData, __timePanelDataGenerator, props.groups, __groupOrientation, __isAllDayPanelVisible, __isVerticalGrouping, __correctedVirtualScrollingState]);

  var __viewDataMapWithSelection = useMemo(function __viewDataMapWithSelection() {
    if (!__state_cellsSelectionState) {
      return __viewDataMap;
    }

    return __viewDataGenerator.markSelectedAndFocusedCells(__viewDataMap, __state_cellsSelectionState);
  }, [__state_cellsSelectionState, __viewDataMap, __viewDataGenerator]);

  var __viewDataProvider = useMemo(function __viewDataProvider() {
    var {
      cellDuration,
      currentDate,
      endDayHour,
      firstDayOfWeek,
      groups,
      hoursInterval,
      intervalCount,
      startDate,
      startDayHour,
      type
    } = props;
    var viewDataProvider = new ViewDataProvider(type);
    viewDataProvider.completeViewDataMap = __completeViewDataMap;
    viewDataProvider.viewDataMap = __viewDataMap;
    var generationOptions = prepareGenerationOptions({
      intervalCount,
      groups,
      groupByDate: __isGroupedByDate(),
      groupOrientation: __groupOrientation(),
      startDayHour,
      endDayHour,
      currentDate,
      startDate,
      firstDayOfWeek,
      hoursInterval,
      type,
      cellDuration
    }, __renderConfig, __isAllDayPanelVisible(), __correctedVirtualScrollingState);
    viewDataProvider.setViewOptions(generationOptions);
    viewDataProvider.createGroupedDataMapProvider();
    return viewDataProvider;
  }, [props.cellDuration, props.currentDate, props.endDayHour, props.firstDayOfWeek, props.groups, props.hoursInterval, props.intervalCount, props.startDate, props.startDayHour, props.type, __completeViewDataMap, __viewDataMap, __isGroupedByDate, __groupOrientation, __renderConfig, __isAllDayPanelVisible, __correctedVirtualScrollingState]);

  var __viewData = useMemo(function __viewData() {
    var {
      groups
    } = props;

    var result = __viewDataGenerator.getViewDataFromMap(__completeViewDataMap, __viewDataMapWithSelection, _extends({}, __correctedVirtualScrollingState, {
      isProvideVirtualCellsWidth: __renderConfig.isProvideVirtualCellsWidth,
      isVerticalGrouping: __isVerticalGrouping(),
      isAllDayPanelVisible: __isAllDayPanelVisible(),
      isGroupedAllDayPanel: calculateIsGroupedAllDayPanel(groups, __groupOrientation(), __isAllDayPanelVisible())
    }));

    return result;
  }, [props.groups, __viewDataGenerator, __completeViewDataMap, __viewDataMapWithSelection, __correctedVirtualScrollingState, __renderConfig, __isVerticalGrouping, __isAllDayPanelVisible, __groupOrientation]);

  var __onPointerDown = useCallback(function __onPointerDown(e) {
    var cell = e.target.closest(DATA_CELL_SELECTOR);

    if (cell && isMouseEvent(e) && e.button === 0) {
      var isAllDay = isCellAllDay(cell);
      var cellIndices = getCellIndices(cell);

      var cellData = __viewDataProvider.getCellData(cellIndices.rowIndex, cellIndices.columnIndex, isAllDay);

      eventsEngine.off(domAdapter.getDocument(), pointerEvents.up, __onPointerUp);
      eventsEngine.on(domAdapter.getDocument(), pointerEvents.up, __onPointerUp);

      __state_setCellsSelectionState(__state_cellsSelectionState => ({
        focusedCell: {
          cellData,
          position: cellIndices
        },
        selectedCells: [cellData],
        firstSelectedCell: cellData
      }));

      __state_setIsPointerDown(__state_isPointerDown => true);
    }
  }, [__viewDataProvider, __onPointerUp]);

  var __onPointerMove = useCallback(function __onPointerMove(e) {
    var cell = e.target.closest(DATA_CELL_SELECTOR);

    if (cell && __state_isPointerDown) {
      e.preventDefault();
      e.stopPropagation();
      var cellsSelectionController = new CellsSelectionController();
      var cellIndices = getCellIndices(cell);
      var isAllDay = isCellAllDay(cell);

      var cellData = __viewDataProvider.getCellData(cellIndices.rowIndex, cellIndices.columnIndex, isAllDay);

      var nextFocusedCell = cellsSelectionController.moveToCell({
        isMultiSelection: true,
        isMultiSelectionAllowed: true,
        focusedCellData: __state_cellsSelectionState.focusedCell.cellData,
        currentCellData: cellData
      });

      if (nextFocusedCell === cellData && __state_cellsSelectionState.focusedCell.cellData.index !== cellData.index) {
        var firstCell = __state_cellsSelectionState.firstSelectedCell;
        var lastCell = cellData;
        var selectedCells = getSelectedCells(__viewDataProvider, firstCell, lastCell, !!firstCell.allDay);

        __state_setCellsSelectionState(__state_cellsSelectionState => ({
          focusedCell: {
            cellData,
            position: cellIndices
          },
          selectedCells,
          firstSelectedCell: __state_cellsSelectionState.firstSelectedCell
        }));
      }
    }
  }, [__state_isPointerDown, __viewDataProvider, __state_cellsSelectionState]);

  useEffect(() => {
    DiagnosticUtils.incrementRenderCount("scheduler_workspace");
  });
  useEffect(() => {
    var _timePanelRef$curren, _timePanelRef$curren2, _groupPanelRef$curre, _groupPanelRef$curre2;

    var timePanelWidth = (_timePanelRef$curren = (_timePanelRef$curren2 = __timePanelRef.current) === null || _timePanelRef$curren2 === void 0 ? void 0 : _timePanelRef$curren2.getBoundingClientRect().width) !== null && _timePanelRef$curren !== void 0 ? _timePanelRef$curren : 0;
    var groupPanelWidth = (_groupPanelRef$curre = (_groupPanelRef$curre2 = __groupPanelRef.current) === null || _groupPanelRef$curre2 === void 0 ? void 0 : _groupPanelRef$curre2.getBoundingClientRect().width) !== null && _groupPanelRef$curre !== void 0 ? _groupPanelRef$curre : 0;

    __state_setHeaderEmptyCellWidth(__state_headerEmptyCellWidth => timePanelWidth + groupPanelWidth);
  });
  useEffect(() => {
    if (__isCalculateTablesWidth()) {
      var {
        currentDate,
        endDayHour,
        groups,
        hoursInterval,
        intervalCount,
        startDayHour,
        type: viewType
      } = props;

      __state_setTablesWidth(__state_tablesWidth => getDateTableWidth(__layoutRef.current.getScrollableWidth(), __dateTableRef.current, __viewDataProvider, {
        intervalCount,
        currentDate,
        viewType,
        hoursInterval,
        startDayHour,
        endDayHour,
        groups,
        groupOrientation: __groupOrientation()
      }));
    }
  });
  useEffect(() => {
    var dateTableCell = __dateTableRef.current.querySelector("td:not(.dx-scheduler-virtual-cell)");

    var cellRect = dateTableCell.getBoundingClientRect();
    var cellHeight = Math.floor(cellRect.height);
    var cellWidth = Math.floor(cellRect.width);

    var scrollableWidth = __layoutRef.current.getScrollableWidth();

    var widgetRect = __widgetElementRef.current.getBoundingClientRect();

    var viewHeight = widgetRect.height;
    var viewWidth = widgetRect.width;
    var windowHeight = getWindow().innerHeight;
    var windowWidth = getWindow().innerWidth;
    var nextSizes = {
      cellHeight,
      cellWidth,
      scrollableWidth,
      viewHeight,
      viewWidth,
      windowHeight,
      windowWidth
    };
    var isNextMetaDataNotEqualToCurrent = !__state_virtualScrollingData || Object.entries(nextSizes).some(_ref3 => {
      var [key, value] = _ref3;
      return value !== __state_virtualScrollingData.sizes[key];
    });

    if (isNextMetaDataNotEqualToCurrent) {
      var {
        groups,
        schedulerHeight,
        schedulerWidth,
        scrolling
      } = props;
      var completeColumnCount = __completeViewDataMap[0].length;
      var completeRowCount = __completeViewDataMap.length;

      __state_virtualScrolling.setViewOptions(createVirtualScrollingOptions({
        cellHeight: nextSizes.cellHeight,
        cellWidth: nextSizes.cellWidth,
        schedulerHeight,
        schedulerWidth,
        viewHeight: nextSizes.viewHeight,
        viewWidth: nextSizes.viewWidth,
        scrolling,
        scrollableWidth: nextSizes.scrollableWidth,
        groups,
        isVerticalGrouping: __isVerticalGrouping(),
        completeRowCount,
        completeColumnCount,
        windowHeight: nextSizes.windowHeight,
        windowWidth: nextSizes.windowWidth,
        rtlEnabled: !!(config !== null && config !== void 0 && config.rtlEnabled)
      }));

      __state_virtualScrolling.createVirtualScrolling();

      __state_virtualScrolling.updateDimensions(true);

      __state_setVirtualScrollingData(__state_virtualScrollingData => ({
        state: __state_virtualScrolling.getRenderState(),
        sizes: nextSizes
      }));
    }
  });
  useEffect(() => {
    __state_setGroupPanelHeight(__state_groupPanelHeight => {
      var _dateTableRef$curren;

      return (_dateTableRef$curren = __dateTableRef.current) === null || _dateTableRef$curren === void 0 ? void 0 : _dateTableRef$curren.getBoundingClientRect().height;
    });
  });
  useEffect(() => {
    if (__state_virtualScrolling.isAttachWindowScrollEvent()) {
      return subscribeToScrollEvent(domAdapter.getDocument(), __onWindowScroll);
    }

    return undefined;
  }, [__state_virtualScrolling, __onWindowScroll]);
  useEffect(() => {
    var disposePointerDown = subscribeToDXPointerDownEvent(__widgetElementRef.current, __onPointerDown);
    var disposePointerMove = subscribeToDXPointerMoveEvent(__widgetElementRef.current, __onPointerMove);
    return () => {
      disposePointerDown();
      disposePointerMove();
    };
  }, [__onPointerDown, __onPointerMove]);
  useEffect(() => {
    var {
      allDayPanelExpanded,
      cellDuration,
      crossScrollingEnabled,
      currentDate,
      endDayHour,
      firstDayOfWeek,
      groupByDate,
      groupOrientation,
      groups,
      hoursInterval,
      intervalCount,
      onViewRendered,
      scrolling,
      showAllDayPanel,
      startDate,
      startDayHour,
      type: viewType
    } = props;
    var tableWidths = getDateTableWidth(__layoutRef.current.getScrollableWidth(), __dateTableRef.current, __viewDataProvider, {
      intervalCount,
      currentDate,
      viewType,
      hoursInterval,
      startDayHour,
      endDayHour,
      groups,
      groupOrientation: __groupOrientation()
    });

    if (!__isCalculateTablesWidth() || tableWidths === __state_tablesWidth) {
      var columnCount = __viewDataMap.dateTableMap[0].length;

      var dateTableCellsMeta = __createDateTableElementsMeta(columnCount);

      var allDayPanelCellsMeta = __createAllDayPanelElementsMeta();

      onViewRendered({
        viewDataProvider: __viewDataProvider,
        cellsMetaData: {
          dateTableCellsMeta,
          allDayPanelCellsMeta
        },
        viewDataProviderValidationOptions: {
          intervalCount,
          currentDate,
          type: viewType,
          hoursInterval,
          startDayHour,
          endDayHour,
          groups,
          groupOrientation,
          groupByDate,
          crossScrollingEnabled,
          firstDayOfWeek,
          startDate,
          showAllDayPanel,
          allDayPanelExpanded,
          scrolling,
          cellDuration
        }
      });
    }
  }, [props.allDayPanelExpanded, props.cellDuration, props.crossScrollingEnabled, props.currentDate, props.endDayHour, props.firstDayOfWeek, props.groupByDate, props.groupOrientation, props.groups, props.hoursInterval, props.intervalCount, props.onViewRendered, props.scrolling, props.showAllDayPanel, props.startDate, props.startDayHour, props.type, __viewDataProvider, __groupOrientation, __isCalculateTablesWidth, __state_tablesWidth, __viewDataMap, __createDateTableElementsMeta, __createAllDayPanelElementsMeta]);
  useEffect(() => {
    eventsEngine.off(domAdapter.getDocument(), pointerEvents.up, __onPointerUp);
  }, []);
  return viewFunction({
    props: _extends({}, props, {
      dataCellTemplate: getTemplate(props.dataCellTemplate, props.dataCellRender, props.dataCellComponent),
      dateCellTemplate: getTemplate(props.dateCellTemplate, props.dateCellRender, props.dateCellComponent),
      timeCellTemplate: getTemplate(props.timeCellTemplate, props.timeCellRender, props.timeCellComponent),
      resourceCellTemplate: getTemplate(props.resourceCellTemplate, props.resourceCellRender, props.resourceCellComponent),
      appointmentTemplate: getTemplate(props.appointmentTemplate, props.appointmentRender, props.appointmentComponent),
      appointmentCollectorTemplate: getTemplate(props.appointmentCollectorTemplate, props.appointmentCollectorRender, props.appointmentCollectorComponent)
    }),
    groupPanelHeight: __state_groupPanelHeight,
    headerEmptyCellWidth: __state_headerEmptyCellWidth,
    tablesWidth: __state_tablesWidth,
    virtualScrolling: __state_virtualScrolling,
    virtualScrollingData: __state_virtualScrollingData,
    cellsSelectionState: __state_cellsSelectionState,
    isPointerDown: __state_isPointerDown,
    dateTableRef: __dateTableRef,
    allDayPanelRef: __allDayPanelRef,
    timePanelRef: __timePanelRef,
    groupPanelRef: __groupPanelRef,
    widgetElementRef: __widgetElementRef,
    layoutRef: __layoutRef,
    config,
    renderConfig: __renderConfig,
    groupOrientation: __groupOrientation(),
    isVerticalGrouping: __isVerticalGrouping(),
    isHorizontalGrouping: __isHorizontalGrouping(),
    isGroupedByDate: __isGroupedByDate(),
    isAllDayPanelVisible: __isAllDayPanelVisible(),
    viewDataGenerator: __viewDataGenerator,
    dateHeaderDataGenerator: __dateHeaderDataGenerator,
    timePanelDataGenerator: __timePanelDataGenerator,
    startViewDate: __startViewDate,
    completeViewDataMap: __completeViewDataMap,
    correctedVirtualScrollingState: __correctedVirtualScrollingState,
    viewDataMap: __viewDataMap,
    viewDataMapWithSelection: __viewDataMapWithSelection,
    viewData: __viewData,
    completeDateHeaderData: __completeDateHeaderData,
    dateHeaderData: __dateHeaderData,
    completeTimePanelData: __completeTimePanelData,
    timePanelData: __timePanelData,
    viewDataProvider: __viewDataProvider,
    groupPanelData: __groupPanelData,
    isRenderHeaderEmptyCell: __isRenderHeaderEmptyCell(),
    isWorkSpaceWithOddCells: __isWorkSpaceWithOddCells(),
    classes: __classes(),
    isStandaloneAllDayPanel: __isStandaloneAllDayPanel(),
    isCalculateTablesWidth: __isCalculateTablesWidth(),
    createDateTableElementsMeta: __createDateTableElementsMeta,
    createAllDayPanelElementsMeta: __createAllDayPanelElementsMeta,
    onWindowScroll: __onWindowScroll,
    onScrollableScroll: __onScrollableScroll,
    onScroll: __onScroll,
    onPointerDown: __onPointerDown,
    onPointerUp: __onPointerUp,
    onPointerMove: __onPointerMove,
    restAttributes: __restAttributes()
  });
}
WorkSpace.defaultProps = WorkSpaceProps;