import type { dxSchedulerScrolling } from 'devextreme/ui/scheduler';
import { GroupOrientation, ViewType } from "../types";
import { DataCellTemplateProps, DateTimeCellTemplateProps, Group, ResourceCellTemplateProps, ViewMetaData } from "./types";
import { BaseWidgetProps } from "../../common/base_props";
import { HeaderPanelLayoutProps } from "./base/header_panel/layout";
import { DateTableLayoutProps } from "./base/date_table/layout";
import { GetDateForHeaderText } from "../view_model/to_test/views/types";
import { ScrollableDirection } from "../../scroll_view/common/types";
import { AppointmentTemplateProps, OverflowIndicatorTemplateProps } from "../appointment/types";
export declare type WorkSpacePropsType = typeof BaseWidgetProps & {
    dataCellTemplate?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dateCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    resourceCellTemplate?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    appointmentTemplate?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentCollectorTemplate?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    intervalCount: number;
    groups: Group[];
    groupByDate: boolean;
    groupOrientation?: GroupOrientation;
    crossScrollingEnabled: boolean;
    startDayHour: number;
    endDayHour: number;
    firstDayOfWeek: number;
    currentDate: Date;
    startDate?: Date;
    hoursInterval: number;
    showAllDayPanel: boolean;
    allDayPanelExpanded: boolean;
    allowMultipleCellSelection: boolean;
    indicatorTime: any;
    indicatorUpdateInterval: any;
    shadeUntilCurrentTime: boolean;
    selectedCellData: any;
    scrolling: dxSchedulerScrolling;
    cellDuration: number;
    showCurrentTimeIndicator: boolean;
    schedulerHeight?: string | number | (() => string | number);
    schedulerWidth?: string | number | (() => string | number);
    type: ViewType;
    maxAppointmentsPerCell?: number | "auto" | "unlimited";
    onViewRendered: (viewMetaData: ViewMetaData) => void;
    appointments?: React.ReactNode;
    allDayAppointments?: React.ReactNode;
    dataCellRender?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dataCellComponent?: React.JSXElementConstructor<Partial<DataCellTemplateProps>>;
    dateCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    dateCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    timeCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    resourceCellRender?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    resourceCellComponent?: React.JSXElementConstructor<Partial<ResourceCellTemplateProps>>;
    appointmentRender?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentComponent?: React.JSXElementConstructor<Partial<AppointmentTemplateProps>>;
    appointmentCollectorRender?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    appointmentCollectorComponent?: React.JSXElementConstructor<Partial<OverflowIndicatorTemplateProps>>;
};
export declare const WorkSpaceProps: WorkSpacePropsType;
export declare type CurrentViewConfigTypeType = {
    dataCellTemplate?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dateCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellTemplate?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    resourceCellTemplate?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    appointmentTemplate?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentCollectorTemplate?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    intervalCount: number;
    groupByDate: boolean;
    groupOrientation?: GroupOrientation;
    crossScrollingEnabled: boolean;
    startDayHour: number;
    endDayHour: number;
    firstDayOfWeek: number;
    currentDate: Date;
    startDate?: Date;
    hoursInterval: number;
    showAllDayPanel: boolean;
    allDayPanelExpanded: boolean;
    allowMultipleCellSelection: boolean;
    indicatorUpdateInterval: any;
    shadeUntilCurrentTime: boolean;
    scrolling: dxSchedulerScrolling;
    cellDuration: number;
    showCurrentTimeIndicator: boolean;
    schedulerHeight?: string | number | (() => string | number);
    schedulerWidth?: string | number | (() => string | number);
    type: ViewType;
    maxAppointmentsPerCell?: number | "auto" | "unlimited";
    accessKey?: string;
    focusStateEnabled?: boolean;
    tabIndex?: number;
    dataCellRender?: React.FunctionComponent<Partial<DataCellTemplateProps>>;
    dataCellComponent?: React.JSXElementConstructor<Partial<DataCellTemplateProps>>;
    dateCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    dateCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    timeCellRender?: React.FunctionComponent<Partial<DateTimeCellTemplateProps>>;
    timeCellComponent?: React.JSXElementConstructor<Partial<DateTimeCellTemplateProps>>;
    resourceCellRender?: React.FunctionComponent<Partial<ResourceCellTemplateProps>>;
    resourceCellComponent?: React.JSXElementConstructor<Partial<ResourceCellTemplateProps>>;
    appointmentRender?: React.FunctionComponent<Partial<AppointmentTemplateProps>>;
    appointmentComponent?: React.JSXElementConstructor<Partial<AppointmentTemplateProps>>;
    appointmentCollectorRender?: React.FunctionComponent<Partial<OverflowIndicatorTemplateProps>>;
    appointmentCollectorComponent?: React.JSXElementConstructor<Partial<OverflowIndicatorTemplateProps>>;
};
export declare const CurrentViewConfigType: CurrentViewConfigTypeType;
export interface ViewRenderConfig {
    headerPanelTemplate: React.FunctionComponent<Partial<Omit<typeof HeaderPanelLayoutProps, "dateHeaderData">> & Required<Pick<typeof HeaderPanelLayoutProps, "dateHeaderData">>>;
    dateTableTemplate: React.FunctionComponent<Partial<typeof DateTableLayoutProps>>;
    className: string;
    isAllDayPanelSupported: boolean;
    isProvideVirtualCellsWidth: boolean;
    isRenderTimePanel: boolean;
    groupPanelClassName: "dx-scheduler-work-space-vertical-group-table" | "dx-scheduler-group-table";
    headerCellTextFormat: string | ((date: Date) => string);
    getDateForHeaderText: GetDateForHeaderText;
    isRenderDateHeader: boolean;
    isGenerateWeekDaysHeaderData: boolean;
    scrollingDirection: ScrollableDirection;
    isCreateCrossScrolling: boolean;
    defaultGroupOrientation: GroupOrientation;
    isMonthDateHeader: boolean;
    isUseMonthDateTable: boolean;
    isUseTimelineHeader: boolean;
}
