import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["pocketState", "pocketTop", "pullDownIconAngle", "pullDownOpacity", "pullDownTranslateTop", "pulledDownText", "pullingDownText", "refreshStrategy", "refreshingText", "topPocketRef", "topPocketTranslateTop", "visible"];
import { LoadIndicator } from "../../../load_indicator";
import { combineClasses } from "../../../../utils/combine_classes";
import messageLocalization from 'devextreme/localization/message';
import { PULLDOWN_ICON_CLASS, SCROLLVIEW_PULLDOWN, SCROLLVIEW_PULLDOWN_IMAGE_CLASS, SCROLLVIEW_PULLDOWN_INDICATOR_CLASS, SCROLLVIEW_PULLDOWN_READY_CLASS, SCROLLVIEW_PULLDOWN_LOADING_CLASS, SCROLLVIEW_PULLDOWN_TEXT_CLASS, SCROLLVIEW_PULLDOWN_VISIBLE_TEXT_CLASS, SCROLLVIEW_TOP_POCKET_CLASS, TopPocketState } from "../../common/consts";
import { current, isMaterial } from 'devextreme/ui/themes';
export var viewFunction = viewModel => {
  var {
    props: {
      pulledDownText,
      pullingDownText,
      refreshStrategy,
      refreshingText,
      topPocketRef
    },
    pullDownClasses,
    pullDownIconStyles,
    pullDownRef,
    pullDownStyles,
    readyVisibleClass,
    refreshVisibleClass,
    releaseVisibleClass,
    topPocketClasses,
    topPocketStyles
  } = viewModel;
  return React.createElement("div", {
    ref: topPocketRef,
    className: topPocketClasses,
    style: normalizeStyles(topPocketStyles)
  }, React.createElement("div", {
    ref: pullDownRef,
    className: pullDownClasses,
    style: normalizeStyles(pullDownStyles)
  }, refreshStrategy !== "swipeDown" && React.createElement("div", {
    className: SCROLLVIEW_PULLDOWN_IMAGE_CLASS
  }), refreshStrategy === "swipeDown" && React.createElement("div", {
    className: PULLDOWN_ICON_CLASS,
    style: normalizeStyles(pullDownIconStyles)
  }), React.createElement("div", {
    className: SCROLLVIEW_PULLDOWN_INDICATOR_CLASS
  }, React.createElement(LoadIndicator, null)), refreshStrategy !== "swipeDown" && React.createElement("div", {
    className: SCROLLVIEW_PULLDOWN_TEXT_CLASS
  }, React.createElement("div", {
    className: releaseVisibleClass
  }, pullingDownText), React.createElement("div", {
    className: readyVisibleClass
  }, pulledDownText), React.createElement("div", {
    className: refreshVisibleClass
  }, refreshingText))));
};
export var TopPocketProps = {
  get pullingDownText() {
    return isMaterial(current()) ? "" : messageLocalization.format("dxScrollView-pullingDownText");
  },

  get pulledDownText() {
    return isMaterial(current()) ? "" : messageLocalization.format("dxScrollView-pulledDownText");
  },

  get refreshingText() {
    return isMaterial(current()) ? "" : messageLocalization.format("dxScrollView-refreshingText");
  },

  get pocketState() {
    return TopPocketState.STATE_RELEASED;
  },

  pullDownTranslateTop: 0,
  pullDownIconAngle: 0,
  pullDownOpacity: 0,
  pocketTop: 0,
  topPocketTranslateTop: 0,
  visible: true
};
import * as React from "react";
import { useCallback, useMemo, useRef } from "react";
var NUMBER_STYLES = new Set(["animationIterationCount", "borderImageOutset", "borderImageSlice", "border-imageWidth", "boxFlex", "boxFlexGroup", "boxOrdinalGroup", "columnCount", "fillOpacity", "flex", "flexGrow", "flexNegative", "flexOrder", "flexPositive", "flexShrink", "floodOpacity", "fontWeight", "gridColumn", "gridRow", "lineClamp", "lineHeight", "opacity", "order", "orphans", "stopOpacity", "strokeDasharray", "strokeDashoffset", "strokeMiterlimit", "strokeOpacity", "strokeWidth", "tabSize", "widows", "zIndex", "zoom"]);

var isNumeric = value => {
  if (typeof value === "number") return true;
  return !isNaN(Number(value));
};

var getNumberStyleValue = (style, value) => {
  return NUMBER_STYLES.has(style) ? value : "".concat(value, "px");
};

var normalizeStyles = styles => {
  if (!(styles instanceof Object)) return undefined;
  return Object.entries(styles).reduce((result, _ref) => {
    var [key, value] = _ref;
    result[key] = isNumeric(value) ? getNumberStyleValue(key, value) : value;
    return result;
  }, {});
};

export function TopPocket(props) {
  var __pullDownRef = useRef(null);

  var __releaseVisibleClass = useCallback(function __releaseVisibleClass() {
    return props.pocketState === TopPocketState.STATE_RELEASED ? SCROLLVIEW_PULLDOWN_VISIBLE_TEXT_CLASS : undefined;
  }, [props.pocketState]);

  var __readyVisibleClass = useCallback(function __readyVisibleClass() {
    return props.pocketState === TopPocketState.STATE_READY ? SCROLLVIEW_PULLDOWN_VISIBLE_TEXT_CLASS : undefined;
  }, [props.pocketState]);

  var __refreshVisibleClass = useCallback(function __refreshVisibleClass() {
    return props.pocketState === TopPocketState.STATE_REFRESHING ? SCROLLVIEW_PULLDOWN_VISIBLE_TEXT_CLASS : undefined;
  }, [props.pocketState]);

  var __pullDownClasses = useCallback(function __pullDownClasses() {
    var {
      pocketState,
      visible
    } = props;
    var classesMap = {
      [SCROLLVIEW_PULLDOWN]: true,
      [SCROLLVIEW_PULLDOWN_READY_CLASS]: pocketState === TopPocketState.STATE_READY,
      [SCROLLVIEW_PULLDOWN_LOADING_CLASS]: pocketState === TopPocketState.STATE_REFRESHING,
      "dx-state-invisible": !visible
    };
    return combineClasses(classesMap);
  }, [props.pocketState, props.visible]);

  var __topPocketClasses = useCallback(function __topPocketClasses() {
    var classesMap = {
      [SCROLLVIEW_TOP_POCKET_CLASS]: true,
      "dx-state-invisible": !props.visible
    };
    return combineClasses(classesMap);
  }, [props.visible]);

  var __pullDownStyles = useMemo(function __pullDownStyles() {
    if (props.refreshStrategy === "swipeDown") {
      return {
        opacity: props.pullDownOpacity,
        transform: "translate(0px, ".concat(props.pullDownTranslateTop, "px)")
      };
    }

    return undefined;
  }, [props.refreshStrategy, props.pullDownOpacity, props.pullDownTranslateTop]);

  var __topPocketStyles = useMemo(function __topPocketStyles() {
    if (props.refreshStrategy === "pullDown") {
      return {
        top: "".concat(-props.pocketTop, "px"),
        transform: "translate(0px, ".concat(props.topPocketTranslateTop, "px)")
      };
    }

    return undefined;
  }, [props.refreshStrategy, props.pocketTop, props.topPocketTranslateTop]);

  var __pullDownIconStyles = useMemo(function __pullDownIconStyles() {
    return {
      transform: "rotate(".concat(props.pullDownIconAngle, "deg)")
    };
  }, [props.pullDownIconAngle]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  return viewFunction({
    props: _extends({}, props),
    pullDownRef: __pullDownRef,
    releaseVisibleClass: __releaseVisibleClass(),
    readyVisibleClass: __readyVisibleClass(),
    refreshVisibleClass: __refreshVisibleClass(),
    pullDownClasses: __pullDownClasses(),
    topPocketClasses: __topPocketClasses(),
    pullDownStyles: __pullDownStyles,
    topPocketStyles: __topPocketStyles,
    pullDownIconStyles: __pullDownIconStyles,
    restAttributes: __restAttributes()
  });
}
TopPocket.defaultProps = TopPocketProps;