import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["addWidgetClass", "aria", "bounceEnabled", "children", "classes", "direction", "disabled", "forceGeneratePockets", "height", "inertiaEnabled", "loadPanelComponent", "loadPanelRender", "loadPanelTemplate", "needRenderScrollbars", "needScrollViewContentWrapper", "onBounce", "onEnd", "onPullDown", "onReachBottom", "onScroll", "onStart", "onUpdated", "onVisibilityChange", "pullDownEnabled", "pulledDownText", "pullingDownText", "reachBottomEnabled", "reachBottomText", "refreshStrategy", "refreshingText", "rtlEnabled", "scrollByContent", "scrollByThumb", "scrollLocationChange", "showScrollbar", "useKeyboard", "useNative", "useSimulatedScrollbar", "visible", "width"];
import { isDefined } from 'devextreme/core/utils/type';
import { Scrollable } from "./scrollable";
import { ScrollViewProps } from "./common/scrollview_props";
import { ScrollViewLoadPanel } from "./internal/load_panel";
export var viewFunction = viewModel => {
  var {
    props: {
      aria,
      bounceEnabled,
      children,
      direction,
      disabled,
      height,
      inertiaEnabled,
      onBounce,
      onEnd,
      onPullDown,
      onReachBottom,
      onScroll,
      onStart,
      onUpdated,
      pullDownEnabled,
      pulledDownText,
      pullingDownText,
      reachBottomText,
      refreshStrategy,
      refreshingText,
      rtlEnabled,
      scrollByContent,
      scrollByThumb,
      showScrollbar,
      useKeyboard,
      useNative,
      useSimulatedScrollbar,
      visible,
      width
    },
    reachBottomEnabled,
    restAttributes,
    scrollableRef
  } = viewModel;
  return React.createElement(Scrollable, _extends({
    useNative: useNative,
    classes: "dx-scrollview",
    ref: scrollableRef,
    aria: aria,
    width: width,
    height: height,
    disabled: disabled,
    visible: visible,
    rtlEnabled: rtlEnabled,
    direction: direction,
    showScrollbar: showScrollbar,
    scrollByThumb: scrollByThumb,
    pullDownEnabled: pullDownEnabled,
    reachBottomEnabled: reachBottomEnabled,
    onScroll: onScroll,
    onUpdated: onUpdated,
    onPullDown: onPullDown,
    onReachBottom: onReachBottom,
    refreshStrategy: refreshStrategy,
    pulledDownText: pulledDownText,
    pullingDownText: pullingDownText,
    refreshingText: refreshingText,
    reachBottomText: reachBottomText,
    forceGeneratePockets: true,
    needScrollViewContentWrapper: true,
    useSimulatedScrollbar: useSimulatedScrollbar,
    inertiaEnabled: inertiaEnabled,
    bounceEnabled: bounceEnabled,
    scrollByContent: scrollByContent,
    useKeyboard: useKeyboard,
    onStart: onStart,
    onEnd: onEnd,
    onBounce: onBounce,
    loadPanelTemplate: ScrollViewLoadPanel
  }, restAttributes), children);
};
import * as React from "react";
import { useState, useCallback, useRef, useImperativeHandle, forwardRef } from "react";
import { getTemplate } from "@devextreme/runtime/react";
var ScrollView = forwardRef(function scrollView(props, ref) {
  var __scrollableRef = useRef(null);

  var [__state_forceReachBottom, __state_setForceReachBottom] = useState(undefined);

  var __reachBottomEnabled = useCallback(function __reachBottomEnabled() {
    if (isDefined(__state_forceReachBottom)) {
      return __state_forceReachBottom;
    }

    return props.reachBottomEnabled;
  }, [__state_forceReachBottom, props.reachBottomEnabled]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __refresh = useCallback(function __refresh() {
    if (props.pullDownEnabled) {
      __scrollableRef.current.refresh();
    }
  }, [props.pullDownEnabled]);

  var __content = useCallback(function __content() {
    return __scrollableRef.current.content();
  }, []);

  var __container = useCallback(function __container() {
    return __scrollableRef.current.container();
  }, []);

  var __scrollBy = useCallback(function __scrollBy(distance) {
    __scrollableRef.current.scrollBy(distance);
  }, []);

  var __scrollTo = useCallback(function __scrollTo(targetLocation) {
    __scrollableRef.current.scrollTo(targetLocation);
  }, []);

  var __scrollToElement = useCallback(function __scrollToElement(element, offset) {
    __scrollableRef.current.scrollToElement(element, offset);
  }, []);

  var __scrollHeight = useCallback(function __scrollHeight() {
    return __scrollableRef.current.scrollHeight();
  }, []);

  var __scrollWidth = useCallback(function __scrollWidth() {
    return __scrollableRef.current.scrollWidth();
  }, []);

  var __scrollOffset = useCallback(function __scrollOffset() {
    return __scrollableRef.current.scrollOffset();
  }, []);

  var __scrollTop = useCallback(function __scrollTop() {
    return __scrollableRef.current.scrollTop();
  }, []);

  var __scrollLeft = useCallback(function __scrollLeft() {
    return __scrollableRef.current.scrollLeft();
  }, []);

  var __clientHeight = useCallback(function __clientHeight() {
    return __scrollableRef.current.clientHeight();
  }, []);

  var __clientWidth = useCallback(function __clientWidth() {
    return __scrollableRef.current.clientWidth();
  }, []);

  var __toggleLoading = useCallback(function __toggleLoading(showOrHide) {
    __state_setForceReachBottom(__state_forceReachBottom => showOrHide);
  }, []);

  var __startLoading = useCallback(function __startLoading() {
    __scrollableRef.current.startLoading();
  }, []);

  var __finishLoading = useCallback(function __finishLoading() {
    __scrollableRef.current.finishLoading();
  }, []);

  var __updateHandler = useCallback(function __updateHandler() {
    __scrollableRef.current.updateHandler();
  }, []);

  var __release = useCallback(function __release(preventScrollBottom) {
    if (preventScrollBottom !== undefined) {
      __toggleLoading(!preventScrollBottom);
    }

    __scrollableRef.current.release();
  }, [__toggleLoading]);

  useImperativeHandle(ref, () => ({
    release: __release,
    refresh: __refresh,
    content: __content,
    container: __container,
    scrollBy: __scrollBy,
    scrollTo: __scrollTo,
    scrollToElement: __scrollToElement,
    scrollHeight: __scrollHeight,
    scrollWidth: __scrollWidth,
    scrollOffset: __scrollOffset,
    scrollTop: __scrollTop,
    scrollLeft: __scrollLeft,
    clientHeight: __clientHeight,
    clientWidth: __clientWidth,
    toggleLoading: __toggleLoading,
    startLoading: __startLoading,
    finishLoading: __finishLoading,
    updateHandler: __updateHandler
  }), [__release, __refresh, __content, __container, __scrollBy, __scrollTo, __scrollToElement, __scrollHeight, __scrollWidth, __scrollOffset, __scrollTop, __scrollLeft, __clientHeight, __clientWidth, __toggleLoading, __startLoading, __finishLoading, __updateHandler]);
  return viewFunction({
    props: _extends({}, props, {
      loadPanelTemplate: getTemplate(props.loadPanelTemplate, props.loadPanelRender, props.loadPanelComponent)
    }),
    forceReachBottom: __state_forceReachBottom,
    scrollableRef: __scrollableRef,
    reachBottomEnabled: __reachBottomEnabled(),
    restAttributes: __restAttributes()
  });
});
export { ScrollView };
export default ScrollView;
ScrollView.defaultProps = ScrollViewProps;