import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["addWidgetClass", "aria", "bounceEnabled", "children", "classes", "direction", "disabled", "forceGeneratePockets", "height", "inertiaEnabled", "loadPanelComponent", "loadPanelRender", "loadPanelTemplate", "needRenderScrollbars", "needScrollViewContentWrapper", "onBounce", "onEnd", "onPullDown", "onReachBottom", "onScroll", "onStart", "onUpdated", "onVisibilityChange", "pullDownEnabled", "pulledDownText", "pullingDownText", "reachBottomEnabled", "reachBottomText", "refreshStrategy", "refreshingText", "rtlEnabled", "scrollByContent", "scrollByThumb", "scrollLocationChange", "showScrollbar", "useKeyboard", "useNative", "useSimulatedScrollbar", "visible", "width"];
import { ScrollableNative } from "./strategy/native";
import { ScrollableSimulated } from "./strategy/simulated";
import { getElementLocationInternal } from "./utils/get_element_location_internal";
import { convertToLocation } from "./utils/convert_location";
import { getOffsetDistance } from "./utils/get_offset_distance";
import { isDefined, isNumeric } from 'devextreme/core/utils/type';
import { hasWindow } from 'devextreme/core/utils/window';
import { DIRECTION_HORIZONTAL, DIRECTION_VERTICAL } from "./common/consts";
import { ScrollableProps } from "./common/scrollable_props";
import { resolveRtlEnabled } from "../../utils/resolve_rtl";
import { ConfigContext } from "../../common/config_context";
export var viewFunction = viewModel => {
  var {
    isServerSide,
    props: {
      aria,
      bounceEnabled,
      children,
      classes,
      direction,
      disabled,
      forceGeneratePockets,
      height,
      inertiaEnabled,
      loadPanelTemplate,
      needScrollViewContentWrapper,
      onBounce,
      onEnd,
      onPullDown,
      onReachBottom,
      onScroll,
      onStart,
      onUpdated,
      onVisibilityChange,
      pullDownEnabled,
      pulledDownText,
      pullingDownText,
      reachBottomEnabled,
      reachBottomText,
      refreshStrategy,
      refreshingText,
      scrollByContent,
      scrollByThumb,
      showScrollbar,
      useKeyboard,
      useNative,
      useSimulatedScrollbar,
      visible,
      width
    },
    restAttributes,
    rtlEnabled,
    scrollableNativeRef,
    scrollableSimulatedRef
  } = viewModel;
  return useNative ? React.createElement(ScrollableNative, _extends({
    ref: scrollableNativeRef,
    aria: aria,
    classes: classes,
    width: width,
    height: height,
    disabled: disabled,
    visible: visible,
    rtlEnabled: rtlEnabled,
    direction: direction,
    showScrollbar: showScrollbar,
    pullDownEnabled: pullDownEnabled,
    reachBottomEnabled: reachBottomEnabled,
    forceGeneratePockets: forceGeneratePockets && !isServerSide,
    needScrollViewContentWrapper: needScrollViewContentWrapper,
    loadPanelTemplate: !isServerSide ? loadPanelTemplate : undefined,
    needRenderScrollbars: !isServerSide,
    onScroll: onScroll,
    onUpdated: onUpdated,
    onPullDown: onPullDown,
    onReachBottom: onReachBottom,
    refreshStrategy: refreshStrategy,
    pulledDownText: pulledDownText,
    pullingDownText: pullingDownText,
    refreshingText: refreshingText,
    reachBottomText: reachBottomText,
    useSimulatedScrollbar: useSimulatedScrollbar
  }, restAttributes), children) : React.createElement(ScrollableSimulated, _extends({
    ref: scrollableSimulatedRef,
    aria: aria,
    classes: classes,
    width: width,
    height: height,
    disabled: disabled,
    visible: visible,
    rtlEnabled: rtlEnabled,
    direction: direction,
    showScrollbar: showScrollbar,
    scrollByThumb: scrollByThumb,
    pullDownEnabled: pullDownEnabled,
    reachBottomEnabled: reachBottomEnabled,
    forceGeneratePockets: forceGeneratePockets && !isServerSide,
    needScrollViewContentWrapper: needScrollViewContentWrapper,
    loadPanelTemplate: !isServerSide ? loadPanelTemplate : undefined,
    needRenderScrollbars: !isServerSide,
    onScroll: onScroll,
    onUpdated: onUpdated,
    onPullDown: onPullDown,
    onReachBottom: onReachBottom,
    refreshStrategy: "simulated",
    pulledDownText: pulledDownText,
    pullingDownText: pullingDownText,
    refreshingText: refreshingText,
    reachBottomText: reachBottomText,
    onVisibilityChange: onVisibilityChange,
    inertiaEnabled: inertiaEnabled,
    bounceEnabled: bounceEnabled,
    scrollByContent: scrollByContent,
    useKeyboard: useKeyboard,
    onStart: onStart,
    onEnd: onEnd,
    onBounce: onBounce
  }, restAttributes), children);
};
import * as React from "react";
import { useContext, useCallback, useRef, useImperativeHandle, forwardRef } from "react";
import { getTemplate } from "@devextreme/runtime/react";
var Scrollable = forwardRef(function scrollable(props, ref) {
  var __scrollableNativeRef = useRef(null);

  var __scrollableSimulatedRef = useRef(null);

  var config = useContext(ConfigContext);

  var __scrollableRef = useCallback(function __scrollableRef() {
    if (props.useNative) {
      return __scrollableNativeRef.current;
    }

    return __scrollableSimulatedRef.current;
  }, [props.useNative]);

  var __rtlEnabled = useCallback(function __rtlEnabled() {
    var {
      rtlEnabled
    } = props;
    return !!resolveRtlEnabled(rtlEnabled, config);
  }, [props.rtlEnabled, config]);

  var __isServerSide = useCallback(function __isServerSide() {
    return !hasWindow();
  }, []);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __validate = useCallback(function __validate(event) {
    return __scrollableRef().validate(event);
  }, [__scrollableRef]);

  var __content = useCallback(function __content() {
    return __scrollableRef().content();
  }, [__scrollableRef]);

  var __container = useCallback(function __container() {
    return __scrollableRef().container();
  }, [__scrollableRef]);

  var __scrollBy = useCallback(function __scrollBy(distance) {
    var {
      left,
      top
    } = convertToLocation(distance, props.direction);

    if (!isDefined(top) || !isNumeric(top)) {
      top = 0;
    }

    if (!isDefined(left) || !isNumeric(top)) {
      left = 0;
    }

    if (top === 0 && left === 0) {
      return;
    }

    __scrollableRef().scrollByLocation({
      top,
      left
    });
  }, [props.direction, __scrollableRef]);

  var __updateHandler = useCallback(function __updateHandler() {
    __scrollableRef().updateHandler();
  }, [__scrollableRef]);

  var __release = useCallback(function __release() {
    if (!__isServerSide()) {
      __scrollableRef().release();
    }
  }, [__isServerSide, __scrollableRef]);

  var __refresh = useCallback(function __refresh() {
    if (!__isServerSide()) {
      __scrollableRef().refresh();
    }
  }, [__isServerSide, __scrollableRef]);

  var __scrollHeight = useCallback(function __scrollHeight() {
    return __scrollableRef().scrollHeight();
  }, [__scrollableRef]);

  var __scrollWidth = useCallback(function __scrollWidth() {
    return __scrollableRef().scrollWidth();
  }, [__scrollableRef]);

  var __scrollOffset = useCallback(function __scrollOffset() {
    return __scrollableRef().scrollOffset();
  }, [__scrollableRef]);

  var __scrollTop = useCallback(function __scrollTop() {
    return __scrollableRef().scrollTop();
  }, [__scrollableRef]);

  var __scrollLeft = useCallback(function __scrollLeft() {
    return __scrollableRef().scrollLeft();
  }, [__scrollableRef]);

  var __clientHeight = useCallback(function __clientHeight() {
    return __scrollableRef().clientHeight();
  }, [__scrollableRef]);

  var __clientWidth = useCallback(function __clientWidth() {
    return __scrollableRef().clientWidth();
  }, [__scrollableRef]);

  var __startLoading = useCallback(function __startLoading() {
    __scrollableRef().startLoading();
  }, [__scrollableRef]);

  var __finishLoading = useCallback(function __finishLoading() {
    if (!__isServerSide()) {
      __scrollableRef().finishLoading();
    }
  }, [__isServerSide, __scrollableRef]);

  var __scrollTo = useCallback(function __scrollTo(targetLocation) {
    !props.useNative && __updateHandler();
    var currentScrollOffset = props.useNative ? __scrollOffset() : {
      top: __container().scrollTop,
      left: __container().scrollLeft
    };
    var distance = getOffsetDistance(convertToLocation(targetLocation, props.direction), currentScrollOffset);

    __scrollBy(distance);
  }, [props.useNative, __updateHandler, __scrollOffset, __container, props.direction, __scrollBy]);

  var __getScrollElementPosition = useCallback(function __getScrollElementPosition(targetElement, direction, offset) {
    var scrollOffset = __scrollOffset();

    return getElementLocationInternal(targetElement, direction, __container(), scrollOffset, offset);
  }, [__scrollOffset, __container]);

  var __scrollToElement = useCallback(function __scrollToElement(element, offset) {
    if (!__content().contains(element)) {
      return;
    }

    var scrollPosition = {
      top: 0,
      left: 0
    };
    var {
      direction
    } = props;

    if (direction !== DIRECTION_VERTICAL) {
      scrollPosition.left = __getScrollElementPosition(element, DIRECTION_HORIZONTAL, offset);
    }

    if (direction !== DIRECTION_HORIZONTAL) {
      scrollPosition.top = __getScrollElementPosition(element, DIRECTION_VERTICAL, offset);
    }

    __scrollTo(scrollPosition);
  }, [__content, props.direction, __getScrollElementPosition, __scrollTo]);

  useImperativeHandle(ref, () => ({
    content: __content,
    container: __container,
    scrollTo: __scrollTo,
    scrollBy: __scrollBy,
    updateHandler: __updateHandler,
    release: __release,
    refresh: __refresh,
    scrollToElement: __scrollToElement,
    scrollHeight: __scrollHeight,
    scrollWidth: __scrollWidth,
    scrollOffset: __scrollOffset,
    scrollTop: __scrollTop,
    scrollLeft: __scrollLeft,
    clientHeight: __clientHeight,
    clientWidth: __clientWidth,
    getScrollElementPosition: __getScrollElementPosition,
    startLoading: __startLoading,
    finishLoading: __finishLoading
  }), [__content, __container, __scrollTo, __scrollBy, __updateHandler, __release, __refresh, __scrollToElement, __scrollHeight, __scrollWidth, __scrollOffset, __scrollTop, __scrollLeft, __clientHeight, __clientWidth, __getScrollElementPosition, __startLoading, __finishLoading]);
  return viewFunction({
    props: _extends({}, props, {
      loadPanelTemplate: getTemplate(props.loadPanelTemplate, props.loadPanelRender, props.loadPanelComponent)
    }),
    scrollableNativeRef: __scrollableNativeRef,
    scrollableSimulatedRef: __scrollableSimulatedRef,
    config,
    validate: __validate,
    scrollableRef: __scrollableRef(),
    rtlEnabled: __rtlEnabled(),
    isServerSide: __isServerSide(),
    restAttributes: __restAttributes()
  });
});
export { Scrollable };
export default Scrollable;
Scrollable.defaultProps = ScrollableProps;