import { DxMouseEvent } from "../common/types";
export declare const OUT_BOUNDS_ACCELERATION = 0.5;
export declare const ACCELERATION = 0.92;
export declare const MIN_VELOCITY_LIMIT = 1;
export declare const BOUNCE_MIN_VELOCITY_LIMIT: number;
export declare const BOUNCE_ACCELERATION_SUM: number;
export declare const viewFunction: (viewModel: AnimatedScrollbar) => any;
import { EventCallback } from "../../common/event_callback";
import { ScrollableShowScrollbar, ScrollLocationChangeArgs } from "../common/types";
export declare type AnimatedScrollbarPropsTypeType = {
    pulledDown: boolean;
    bottomPocketSize: number;
    contentPaddingBottom: number;
    onBounce?: EventCallback;
    onLock?: EventCallback;
    onUnlock?: EventCallback;
    onPullDown?: EventCallback;
    onReachBottom?: EventCallback;
    onEnd?: EventCallback<string>;
    direction: "vertical" | "horizontal";
    containerSize: number;
    contentSize: number;
    visible: boolean;
    containerHasSizes: boolean;
    scrollLocation: number;
    minOffset: number;
    maxOffset: number;
    inertiaEnabled: boolean;
    showScrollbar: ScrollableShowScrollbar;
    scrollByThumb: any;
    scrollLocationChange?: EventCallback<ScrollLocationChangeArgs>;
    bounceEnabled: any;
    pullDownEnabled: boolean;
    reachBottomEnabled: boolean;
    forceGeneratePockets: boolean;
};
declare const AnimatedScrollbarPropsType: AnimatedScrollbarPropsTypeType;
import * as React from "react";
export declare type AnimatedScrollbarRef = {
    isThumb: (element: EventTarget | null) => boolean;
    isScrollbar: (element: EventTarget | null) => boolean;
    reachedMin: () => boolean;
    reachedMax: () => boolean;
    initHandler: (event: DxMouseEvent, crossThumbScrolling: boolean, offset: number) => void;
    moveHandler: (delta: number, isDxMouseWheel: boolean) => void;
    endHandler: (receivedVelocity: number, needRiseEnd: boolean) => void;
    stopHandler: () => void;
    scrollTo: (value: number) => void;
    releaseHandler: () => void;
};
declare type RestProps = {
    className?: string;
    style?: {
        [name: string]: any;
    };
    key?: any;
    ref?: any;
};
interface AnimatedScrollbar {
    props: typeof AnimatedScrollbarPropsType & RestProps;
    scrollbarRef: any;
    canceled: boolean;
    newScrollLocation: number;
    forceAnimationToBottomBound: boolean;
    pendingRefreshing: boolean;
    pendingLoading: boolean;
    pendingBounceAnimator: boolean;
    pendingInertiaAnimator: boolean;
    needRiseEnd: boolean;
    wasRelease: boolean;
    isReadyToStart: boolean;
    distanceToNearestBoundary: number;
    suppressVelocityBeforeBoundary: () => void;
    scrollToNextStep: () => void;
    setActiveState: () => void;
    moveTo: (value: number) => void;
    moveToMouseLocation: (event: DxMouseEvent, offset: number) => void;
    resetThumbScrolling: () => void;
    stop: () => void;
    cancel: () => void;
    calcThumbScrolling: (event: DxMouseEvent, currentCrossThumbScrolling: boolean, isScrollbarClicked: boolean) => void;
    distanceToMin: number;
    distanceToMax: number;
    pendingRelease: boolean;
    inProgress: boolean;
    inRange: boolean;
    isReachBottom: boolean;
    finished: boolean;
    acceleration: number;
    maxOffset: number;
    restAttributes: RestProps;
}
declare const AnimatedScrollbar: React.FC<AnimatedScrollbarPropsTypeType & RestProps & {
    ref?: React.Ref<AnimatedScrollbarRef>;
}> & {
    defaultProps: typeof AnimatedScrollbarPropsType;
};
export { AnimatedScrollbar };
export default AnimatedScrollbar;
