import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["bottomPocketSize", "bounceEnabled", "containerHasSizes", "containerSize", "contentPaddingBottom", "contentSize", "direction", "forceGeneratePockets", "inertiaEnabled", "maxOffset", "minOffset", "onBounce", "onEnd", "onLock", "onPullDown", "onReachBottom", "onUnlock", "pullDownEnabled", "pulledDown", "reachBottomEnabled", "scrollByThumb", "scrollLocation", "scrollLocationChange", "showScrollbar", "visible"];
import { Scrollbar } from "./scrollbar";
import { requestAnimationFrame, cancelAnimationFrame } from 'devextreme/animation/frame';
import { ScrollableSimulatedProps } from "../common/simulated_strategy_props";
import { inRange } from 'devextreme/core/utils/math';
import { clampIntoRange } from "../utils/clamp_into_range";
import { AnimatedScrollbarProps } from "../common/animated_scrollbar_props";
import { isDxMouseWheelEvent } from 'devextreme/events/utils';
export var OUT_BOUNDS_ACCELERATION = 0.5;
export var ACCELERATION = 0.92;
export var MIN_VELOCITY_LIMIT = 1;
export var BOUNCE_MIN_VELOCITY_LIMIT = MIN_VELOCITY_LIMIT / 5;
var FRAME_DURATION = 17;
var BOUNCE_DURATION = 400;
var BOUNCE_FRAMES = BOUNCE_DURATION / FRAME_DURATION;
export var BOUNCE_ACCELERATION_SUM = (1 - ACCELERATION ** BOUNCE_FRAMES) / (1 - ACCELERATION);
export var viewFunction = viewModel => {
  var {
    newScrollLocation,
    props: {
      bounceEnabled,
      containerHasSizes,
      containerSize,
      contentSize,
      direction,
      maxOffset,
      minOffset,
      scrollByThumb,
      scrollLocationChange,
      showScrollbar,
      visible
    },
    scrollbarRef
  } = viewModel;
  return React.createElement(Scrollbar, {
    ref: scrollbarRef,
    direction: direction,
    contentSize: contentSize,
    containerSize: containerSize,
    visible: visible,
    minOffset: minOffset,
    maxOffset: maxOffset,
    scrollLocation: newScrollLocation,
    scrollLocationChange: scrollLocationChange,
    scrollByThumb: scrollByThumb,
    bounceEnabled: bounceEnabled,
    showScrollbar: showScrollbar,
    containerHasSizes: containerHasSizes
  });
};
var AnimatedScrollbarPropsType = {
  pulledDown: AnimatedScrollbarProps.pulledDown,
  bottomPocketSize: AnimatedScrollbarProps.bottomPocketSize,
  contentPaddingBottom: AnimatedScrollbarProps.contentPaddingBottom,
  direction: AnimatedScrollbarProps.direction,
  containerSize: AnimatedScrollbarProps.containerSize,
  contentSize: AnimatedScrollbarProps.contentSize,
  visible: AnimatedScrollbarProps.visible,
  containerHasSizes: AnimatedScrollbarProps.containerHasSizes,
  scrollLocation: AnimatedScrollbarProps.scrollLocation,
  minOffset: AnimatedScrollbarProps.minOffset,
  maxOffset: AnimatedScrollbarProps.maxOffset,
  inertiaEnabled: ScrollableSimulatedProps.inertiaEnabled,
  showScrollbar: ScrollableSimulatedProps.showScrollbar,
  scrollByThumb: ScrollableSimulatedProps.scrollByThumb,
  bounceEnabled: ScrollableSimulatedProps.bounceEnabled,
  pullDownEnabled: ScrollableSimulatedProps.pullDownEnabled,
  reachBottomEnabled: ScrollableSimulatedProps.reachBottomEnabled,
  forceGeneratePockets: ScrollableSimulatedProps.forceGeneratePockets
};
import * as React from "react";
import { useState, useCallback, useEffect, useRef, useImperativeHandle, forwardRef } from "react";
var AnimatedScrollbar = forwardRef(function animatedScrollbar(props, ref) {
  var __scrollbarRef = useRef(null);

  var [__state_canceled, __state_setCanceled] = useState(false);
  var [__state_newScrollLocation, __state_setNewScrollLocation] = useState(0);
  var [__state_forceAnimationToBottomBound, __state_setForceAnimationToBottomBound] = useState(false);
  var [__state_pendingRefreshing, __state_setPendingRefreshing] = useState(false);
  var [__state_pendingLoading, __state_setPendingLoading] = useState(false);
  var [__state_pendingBounceAnimator, __state_setPendingBounceAnimator] = useState(false);
  var [__state_pendingInertiaAnimator, __state_setPendingInertiaAnimator] = useState(false);
  var [__state_needRiseEnd, __state_setNeedRiseEnd] = useState(false);
  var [__state_wasRelease, __state_setWasRelease] = useState(false);
  var thumbScrolling = useRef(false);
  var crossThumbScrolling = useRef(false);
  var stepAnimationFrame = useRef(0);
  var velocity = useRef(0);
  var refreshing = useRef(false);
  var loading = useRef(false);

  var __setActiveState = useCallback(function __setActiveState() {
    __scrollbarRef.current.setActiveState();
  }, []);

  var __moveTo = useCallback(function __moveTo(value) {
    __scrollbarRef.current.moveTo(value);
  }, []);

  var __moveToMouseLocation = useCallback(function __moveToMouseLocation(event, offset) {
    __scrollbarRef.current.moveToMouseLocation(event, offset);
  }, []);

  var __resetThumbScrolling = useCallback(function __resetThumbScrolling() {
    thumbScrolling.current = false;
    crossThumbScrolling.current = false;
  }, []);

  var __stop = useCallback(function __stop() {
    velocity.current = 0;

    __state_setPendingBounceAnimator(__state_pendingBounceAnimator => false);

    __state_setPendingInertiaAnimator(__state_pendingInertiaAnimator => false);
  }, []);

  var __distanceToMin = useCallback(function __distanceToMin() {
    return props.minOffset - props.scrollLocation;
  }, [props.minOffset, props.scrollLocation]);

  var __inProgress = useCallback(function __inProgress() {
    return __state_pendingBounceAnimator || __state_pendingInertiaAnimator;
  }, [__state_pendingBounceAnimator, __state_pendingInertiaAnimator]);

  var __isReachBottom = useCallback(function __isReachBottom() {
    return props.reachBottomEnabled && Math.round(props.scrollLocation - props.maxOffset) <= 1;
  }, [props.reachBottomEnabled, props.scrollLocation, props.maxOffset]);

  var __finished = useCallback(function __finished() {
    if (__state_pendingBounceAnimator) {
      return Math.abs(velocity.current) <= BOUNCE_MIN_VELOCITY_LIMIT;
    }

    return Math.abs(velocity.current) <= MIN_VELOCITY_LIMIT;
  }, [__state_pendingBounceAnimator, velocity]);

  var __maxOffset = useCallback(function __maxOffset() {
    if (props.forceGeneratePockets && props.reachBottomEnabled && !__state_forceAnimationToBottomBound) {
      return props.maxOffset - props.bottomPocketSize - props.contentPaddingBottom;
    }

    return props.maxOffset;
  }, [props.forceGeneratePockets, props.reachBottomEnabled, __state_forceAnimationToBottomBound, props.maxOffset, props.bottomPocketSize, props.contentPaddingBottom]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __isThumb = useCallback(function __isThumb(element) {
    return __scrollbarRef.current.isThumb(element);
  }, []);

  var __isScrollbar = useCallback(function __isScrollbar(element) {
    return __scrollbarRef.current.isScrollbar(element);
  }, []);

  var __reachedMax = useCallback(function __reachedMax() {
    return props.scrollLocation >= props.minOffset;
  }, [props.scrollLocation, props.minOffset]);

  var __isReadyToStart = useCallback(function __isReadyToStart() {
    return __state_needRiseEnd && !__inProgress() && !(__state_pendingRefreshing || __state_pendingLoading);
  }, [__state_needRiseEnd, __inProgress, __state_pendingRefreshing, __state_pendingLoading]);

  var __cancel = useCallback(function __cancel() {
    __state_setCanceled(__state_canceled => true);

    __stop();

    cancelAnimationFrame(stepAnimationFrame.current);
  }, [__stop, stepAnimationFrame]);

  var __calcThumbScrolling = useCallback(function __calcThumbScrolling(event, currentCrossThumbScrolling, isScrollbarClicked) {
    var {
      target
    } = event.originalEvent;
    thumbScrolling.current = isScrollbarClicked || props.scrollByThumb && __isThumb(target);
    crossThumbScrolling.current = !thumbScrolling.current && currentCrossThumbScrolling;
  }, [props.scrollByThumb, __isThumb, thumbScrolling]);

  var __distanceToMax = useCallback(function __distanceToMax() {
    return __maxOffset() - props.scrollLocation;
  }, [__maxOffset, props.scrollLocation]);

  var __pendingRelease = useCallback(function __pendingRelease() {
    return (props.pulledDown && props.pullDownEnabled || __isReachBottom() && props.reachBottomEnabled) && !__state_wasRelease;
  }, [props.pulledDown, props.pullDownEnabled, __isReachBottom, props.reachBottomEnabled, __state_wasRelease]);

  var __inRange = useCallback(function __inRange() {
    return inRange(props.scrollLocation, __maxOffset(), props.minOffset);
  }, [props.scrollLocation, __maxOffset, props.minOffset]);

  var __reachedMin = useCallback(function __reachedMin() {
    return props.scrollLocation <= __maxOffset();
  }, [props.scrollLocation, __maxOffset]);

  var __moveHandler = useCallback(function __moveHandler(delta, isDxMouseWheel) {
    if (crossThumbScrolling.current) {
      return;
    }

    var resultDelta = delta;

    if (thumbScrolling.current) {
      resultDelta = -Math.round(delta / (props.containerSize / props.contentSize));
    }

    var isOutBounds = !inRange(props.scrollLocation, __maxOffset(), props.minOffset);

    if (isOutBounds) {
      resultDelta *= OUT_BOUNDS_ACCELERATION;
    }

    var scrollValue = props.scrollLocation + resultDelta;

    __moveTo(props.bounceEnabled && !isDxMouseWheel ? scrollValue : clampIntoRange(scrollValue, props.minOffset, __maxOffset()));
  }, [crossThumbScrolling, thumbScrolling, props.containerSize, props.contentSize, props.scrollLocation, __maxOffset, props.minOffset, __moveTo, props.bounceEnabled]);

  var __endHandler = useCallback(function __endHandler(receivedVelocity, needRiseEnd) {
    velocity.current = props.inertiaEnabled && !thumbScrolling.current ? receivedVelocity : 0;

    __state_setNeedRiseEnd(__state_needRiseEnd => needRiseEnd);

    __resetThumbScrolling();
  }, [props.inertiaEnabled, thumbScrolling, __resetThumbScrolling]);

  var __stopHandler = useCallback(function __stopHandler() {
    if (thumbScrolling.current) {
      __state_setNeedRiseEnd(__state_needRiseEnd => true);
    }

    __resetThumbScrolling();
  }, [thumbScrolling, __resetThumbScrolling]);

  var __scrollTo = useCallback(function __scrollTo(value) {
    loading.current = false;
    refreshing.current = false;

    __moveTo(-clampIntoRange(value, -__maxOffset(), 0));

    __state_setNeedRiseEnd(__state_needRiseEnd => true);
  }, [__moveTo, __maxOffset]);

  var __releaseHandler = useCallback(function __releaseHandler() {
    if (props.forceGeneratePockets && props.reachBottomEnabled && inRange(props.scrollLocation, __maxOffset(), props.maxOffset)) {
      __state_setForceAnimationToBottomBound(__state_forceAnimationToBottomBound => true);
    }

    __state_setWasRelease(__state_wasRelease => true);

    __state_setNeedRiseEnd(__state_needRiseEnd => true);

    __resetThumbScrolling();

    __state_setPendingRefreshing(__state_pendingRefreshing => false);

    __state_setPendingLoading(__state_pendingLoading => false);
  }, [props.forceGeneratePockets, props.reachBottomEnabled, props.scrollLocation, __maxOffset, props.maxOffset, __resetThumbScrolling]);

  var __distanceToNearestBoundary = useCallback(function __distanceToNearestBoundary() {
    return Math.min(Math.abs(__distanceToMin()), Math.abs(__distanceToMax()));
  }, [__distanceToMin, __distanceToMax]);

  var __suppressVelocityBeforeBoundary = useCallback(function __suppressVelocityBeforeBoundary() {
    if (Math.abs(__distanceToMin()) - Math.abs(velocity.current) <= 0) {
      velocity.current = __distanceToMin();
    }

    if (Math.abs(__distanceToMax()) - Math.abs(velocity.current) <= 0) {
      velocity.current = __distanceToMax();
    }
  }, [__distanceToMin, velocity, __distanceToMax]);

  var __acceleration = useCallback(function __acceleration() {
    return __state_pendingBounceAnimator || __inRange() ? ACCELERATION : OUT_BOUNDS_ACCELERATION;
  }, [__state_pendingBounceAnimator, __inRange]);

  var __initHandler = useCallback(function __initHandler(event, crossThumbScrolling, offset) {
    __cancel();

    refreshing.current = false;
    loading.current = false;

    if (!isDxMouseWheelEvent(event.originalEvent)) {
      var {
        target
      } = event.originalEvent;

      var scrollbarClicked = props.scrollByThumb && __isScrollbar(target);

      __calcThumbScrolling(event, crossThumbScrolling, scrollbarClicked);

      if (scrollbarClicked) {
        __moveToMouseLocation(event, offset);
      }

      if (thumbScrolling.current) {
        __setActiveState();
      }
    }
  }, [__cancel, props.scrollByThumb, __isScrollbar, __calcThumbScrolling, __moveToMouseLocation, thumbScrolling, __setActiveState]);

  var __scrollToNextStep = useCallback(function __scrollToNextStep() {
    cancelAnimationFrame(stepAnimationFrame.current);
    stepAnimationFrame.current = requestAnimationFrame(() => {
      __state_setNewScrollLocation(__state_newScrollLocation => props.scrollLocation + velocity.current);

      velocity.current *= __acceleration();
    });
  }, [stepAnimationFrame, props.scrollLocation, velocity, __acceleration]);

  useEffect(() => {
    return () => {
      __cancel();
    };
  }, []);
  useEffect(() => {
    if (props.forceGeneratePockets && __isReadyToStart() && __inRange() && props.pulledDown && !refreshing.current) {
      var _props$onPullDown;

      refreshing.current = true;

      __state_setPendingRefreshing(__state_pendingRefreshing => true);

      (_props$onPullDown = props.onPullDown) === null || _props$onPullDown === void 0 ? void 0 : _props$onPullDown.call(props);
    }
  }, [props.forceGeneratePockets, __isReadyToStart, __inRange, props.pulledDown, refreshing, props.onPullDown]);
  useEffect(() => {
    var isInsideBounds = inRange(props.scrollLocation, props.maxOffset, 0);

    if (isInsideBounds && __isReadyToStart() && __finished() && !__pendingRelease()) {
      var _props$onEnd;

      __state_setNeedRiseEnd(__state_needRiseEnd => false);

      __state_setWasRelease(__state_wasRelease => false);

      __state_setForceAnimationToBottomBound(__state_forceAnimationToBottomBound => false);

      (_props$onEnd = props.onEnd) === null || _props$onEnd === void 0 ? void 0 : _props$onEnd.call(props, props.direction);
    }
  }, [props.scrollLocation, props.maxOffset, __isReadyToStart, __finished, __pendingRelease, props.onEnd, props.direction]);
  useEffect(() => {
    if (props.forceGeneratePockets && __isReadyToStart() && __inRange() && __isReachBottom() && !loading.current && __finished()) {
      var _props$onReachBottom;

      loading.current = true;

      __state_setPendingLoading(__state_pendingLoading => true);

      (_props$onReachBottom = props.onReachBottom) === null || _props$onReachBottom === void 0 ? void 0 : _props$onReachBottom.call(props);
    }
  }, [props.forceGeneratePockets, __isReadyToStart, __inRange, __isReachBottom, loading, __finished, props.onReachBottom]);
  useEffect(() => {
    if (__isReadyToStart()) {
      __state_setCanceled(__state_canceled => false);

      if (!__inRange() && props.bounceEnabled) {
        var _props$onBounce;

        var distanceToBound = clampIntoRange(props.scrollLocation, props.minOffset, __maxOffset()) - props.scrollLocation;
        velocity.current = distanceToBound / BOUNCE_ACCELERATION_SUM;
        (_props$onBounce = props.onBounce) === null || _props$onBounce === void 0 ? void 0 : _props$onBounce.call(props);

        __state_setPendingBounceAnimator(__state_pendingBounceAnimator => true);
      }

      if (__inRange() && props.inertiaEnabled) {
        if (thumbScrolling.current || !thumbScrolling.current && crossThumbScrolling.current) {
          velocity.current = 0;
        }

        __state_setPendingInertiaAnimator(__state_pendingInertiaAnimator => true);
      }
    }
  }, [__isReadyToStart, __inRange, props.bounceEnabled, props.scrollLocation, props.minOffset, __maxOffset, props.onBounce, props.inertiaEnabled, thumbScrolling, crossThumbScrolling]);
  useEffect(() => {
    if (!__inProgress()) {
      __state_setNewScrollLocation(__state_newScrollLocation => props.scrollLocation);
    }
  }, [__inProgress, props.scrollLocation]);
  useEffect(() => {
    if (__state_pendingInertiaAnimator) {
      if (__state_canceled) {
        __state_setNeedRiseEnd(__state_needRiseEnd => false);

        __stop();

        return;
      }

      if (__finished() || !props.bounceEnabled && __distanceToNearestBoundary() === 0) {
        __stop();

        return;
      }

      if (!props.bounceEnabled) {
        __suppressVelocityBeforeBoundary();
      }

      __scrollToNextStep();
    }

    if (__state_pendingBounceAnimator) {
      if (__distanceToNearestBoundary() === 0) {
        __stop();

        return;
      }

      __suppressVelocityBeforeBoundary();

      __scrollToNextStep();
    }
  }, [__state_pendingInertiaAnimator, __state_canceled, __stop, __finished, props.bounceEnabled, __distanceToNearestBoundary, __suppressVelocityBeforeBoundary, __scrollToNextStep, __state_pendingBounceAnimator]);
  useEffect(() => {
    if (__state_pendingBounceAnimator || __state_pendingRefreshing || __state_pendingLoading) {
      var _props$onLock;

      (_props$onLock = props.onLock) === null || _props$onLock === void 0 ? void 0 : _props$onLock.call(props);
    } else {
      var _props$onUnlock;

      (_props$onUnlock = props.onUnlock) === null || _props$onUnlock === void 0 ? void 0 : _props$onUnlock.call(props);
    }
  }, [__state_pendingBounceAnimator, __state_pendingRefreshing, __state_pendingLoading, props.onLock, props.onUnlock]);
  useImperativeHandle(ref, () => ({
    isThumb: __isThumb,
    isScrollbar: __isScrollbar,
    reachedMin: __reachedMin,
    reachedMax: __reachedMax,
    initHandler: __initHandler,
    moveHandler: __moveHandler,
    endHandler: __endHandler,
    stopHandler: __stopHandler,
    scrollTo: __scrollTo,
    releaseHandler: __releaseHandler
  }), [__isThumb, __isScrollbar, __reachedMin, __reachedMax, __initHandler, __moveHandler, __endHandler, __stopHandler, __scrollTo, __releaseHandler]);
  return viewFunction({
    props: _extends({}, props),
    canceled: __state_canceled,
    newScrollLocation: __state_newScrollLocation,
    forceAnimationToBottomBound: __state_forceAnimationToBottomBound,
    pendingRefreshing: __state_pendingRefreshing,
    pendingLoading: __state_pendingLoading,
    pendingBounceAnimator: __state_pendingBounceAnimator,
    pendingInertiaAnimator: __state_pendingInertiaAnimator,
    needRiseEnd: __state_needRiseEnd,
    wasRelease: __state_wasRelease,
    scrollbarRef: __scrollbarRef,
    isReadyToStart: __isReadyToStart(),
    distanceToNearestBoundary: __distanceToNearestBoundary(),
    suppressVelocityBeforeBoundary: __suppressVelocityBeforeBoundary,
    scrollToNextStep: __scrollToNextStep,
    setActiveState: __setActiveState,
    moveTo: __moveTo,
    moveToMouseLocation: __moveToMouseLocation,
    resetThumbScrolling: __resetThumbScrolling,
    stop: __stop,
    cancel: __cancel,
    calcThumbScrolling: __calcThumbScrolling,
    distanceToMin: __distanceToMin(),
    distanceToMax: __distanceToMax(),
    pendingRelease: __pendingRelease(),
    inProgress: __inProgress(),
    inRange: __inRange(),
    isReachBottom: __isReachBottom(),
    finished: __finished(),
    acceleration: __acceleration(),
    maxOffset: __maxOffset(),
    restAttributes: __restAttributes()
  });
});
export { AnimatedScrollbar };
export default AnimatedScrollbar;
AnimatedScrollbar.defaultProps = AnimatedScrollbarPropsType;