import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["bounceEnabled", "containerHasSizes", "containerSize", "contentSize", "direction", "maxOffset", "minOffset", "scrollByThumb", "scrollLocation", "scrollLocationChange", "showScrollbar", "visible"];
import { combineClasses } from "../../../utils/combine_classes";
import domAdapter from 'devextreme/core/dom_adapter';
import { DIRECTION_HORIZONTAL, SCROLLABLE_SCROLLBAR_CLASS, SCROLLABLE_SCROLL_CLASS, SCROLLABLE_SCROLL_CONTENT_CLASS, SCROLLABLE_SCROLLBAR_ACTIVE_CLASS, HOVER_ENABLED_STATE, ShowScrollbarMode } from "../common/consts";
import { subscribeToDXPointerDownEvent, subscribeToDXPointerUpEvent, subscribeToMouseEnterEvent, subscribeToMouseLeaveEvent } from "../../../utils/subscribe_to_event";
import { ScrollbarProps } from "../common/scrollbar_props";
import { ScrollableSimulatedProps } from "../common/simulated_strategy_props";
import { ConfigContext } from "../../../common/config_context";
export var THUMB_MIN_SIZE = 15;
export var viewFunction = viewModel => {
  var {
    hidden,
    scrollbarClasses,
    scrollbarRef,
    thumbClasses,
    thumbRef,
    thumbStyles
  } = viewModel;
  return React.createElement("div", {
    className: scrollbarClasses,
    ref: scrollbarRef,
    hidden: hidden
  }, React.createElement("div", {
    className: thumbClasses,
    style: normalizeStyles(thumbStyles),
    ref: thumbRef
  }, React.createElement("div", {
    className: SCROLLABLE_SCROLL_CONTENT_CLASS
  })));
};
export var ScrollbarPropsType = {
  direction: ScrollbarProps.direction,
  containerSize: ScrollbarProps.containerSize,
  contentSize: ScrollbarProps.contentSize,
  visible: ScrollbarProps.visible,
  containerHasSizes: ScrollbarProps.containerHasSizes,
  scrollLocation: ScrollbarProps.scrollLocation,
  minOffset: ScrollbarProps.minOffset,
  maxOffset: ScrollbarProps.maxOffset,
  showScrollbar: ScrollableSimulatedProps.showScrollbar,
  scrollByThumb: ScrollableSimulatedProps.scrollByThumb,
  bounceEnabled: ScrollableSimulatedProps.bounceEnabled
};
import * as React from "react";
import { useState, useContext, useCallback, useMemo, useEffect, useRef, useImperativeHandle, forwardRef } from "react";
var NUMBER_STYLES = new Set(["animationIterationCount", "borderImageOutset", "borderImageSlice", "border-imageWidth", "boxFlex", "boxFlexGroup", "boxOrdinalGroup", "columnCount", "fillOpacity", "flex", "flexGrow", "flexNegative", "flexOrder", "flexPositive", "flexShrink", "floodOpacity", "fontWeight", "gridColumn", "gridRow", "lineClamp", "lineHeight", "opacity", "order", "orphans", "stopOpacity", "strokeDasharray", "strokeDashoffset", "strokeMiterlimit", "strokeOpacity", "strokeWidth", "tabSize", "widows", "zIndex", "zoom"]);

var isNumeric = value => {
  if (typeof value === "number") return true;
  return !isNaN(Number(value));
};

var getNumberStyleValue = (style, value) => {
  return NUMBER_STYLES.has(style) ? value : "".concat(value, "px");
};

var normalizeStyles = styles => {
  if (!(styles instanceof Object)) return undefined;
  return Object.entries(styles).reduce((result, _ref) => {
    var [key, value] = _ref;
    result[key] = isNumeric(value) ? getNumberStyleValue(key, value) : value;
    return result;
  }, {});
};

var Scrollbar = forwardRef(function scrollbar(props, ref) {
  var __scrollbarRef = useRef(null);

  var __scrollRef = useRef(null);

  var __thumbRef = useRef(null);

  var [__state_hovered, __state_setHovered] = useState(false);
  var [__state_active, __state_setActive] = useState(false);
  var rightScrollLocation = useRef(0);
  var prevScrollLocation = useRef(0);
  var prevMaxOffset = useRef(0);
  var config = useContext(ConfigContext);

  var __isHorizontal = useCallback(function __isHorizontal() {
    return props.direction === DIRECTION_HORIZONTAL;
  }, [props.direction]);

  var __containerToContentRatio = useCallback(function __containerToContentRatio() {
    return props.contentSize ? props.containerSize / props.contentSize : props.containerSize;
  }, [props.contentSize, props.containerSize]);

  var __isHoverMode = useCallback(function __isHoverMode() {
    return props.showScrollbar === ShowScrollbarMode.HOVER;
  }, [props.showScrollbar]);

  var __isAlwaysMode = useCallback(function __isAlwaysMode() {
    return props.showScrollbar === ShowScrollbarMode.ALWAYS;
  }, [props.showScrollbar]);

  var __isNeverMode = useCallback(function __isNeverMode() {
    return props.showScrollbar === ShowScrollbarMode.NEVER;
  }, [props.showScrollbar]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __isThumb = useCallback(function __isThumb(element) {
    return __scrollbarRef.current.querySelector(".".concat(SCROLLABLE_SCROLL_CLASS)) === element || __scrollbarRef.current.querySelector(".".concat(SCROLLABLE_SCROLL_CONTENT_CLASS)) === element;
  }, []);

  var __isScrollbar = useCallback(function __isScrollbar(element) {
    return element === __scrollbarRef.current;
  }, []);

  var __setActiveState = useCallback(function __setActiveState() {
    __state_setActive(__state_active => true);
  }, []);

  var __axis = useCallback(function __axis() {
    return __isHorizontal() ? "x" : "y";
  }, [__isHorizontal]);

  var __fullScrollProp = useCallback(function __fullScrollProp() {
    return __isHorizontal() ? "scrollLeft" : "scrollTop";
  }, [__isHorizontal]);

  var __dimension = useCallback(function __dimension() {
    return __isHorizontal() ? "width" : "height";
  }, [__isHorizontal]);

  var __scrollSize = useCallback(function __scrollSize() {
    return Math.max(props.containerSize * __containerToContentRatio(), THUMB_MIN_SIZE);
  }, [props.containerSize, __containerToContentRatio]);

  var __hidden = useCallback(function __hidden() {
    return __isNeverMode() || props.maxOffset === 0 || props.containerSize < 15;
  }, [__isNeverMode, props.maxOffset, props.containerSize]);

  var __isExpandable = useCallback(function __isExpandable() {
    return (__isHoverMode() || __isAlwaysMode()) && props.scrollByThumb;
  }, [__isHoverMode, __isAlwaysMode, props.scrollByThumb]);

  var __scrollRatio = useCallback(function __scrollRatio() {
    var scrollOffsetMax = Math.abs(props.maxOffset);

    if (scrollOffsetMax) {
      return (props.containerSize - __scrollSize()) / scrollOffsetMax;
    }

    return 1;
  }, [props.maxOffset, props.containerSize, __scrollSize]);

  var __scrollbarClasses = useCallback(function __scrollbarClasses() {
    var classesMap = {
      [SCROLLABLE_SCROLLBAR_CLASS]: true,
      ["dx-scrollbar-".concat(props.direction)]: true,
      [SCROLLABLE_SCROLLBAR_ACTIVE_CLASS]: __state_active,
      [HOVER_ENABLED_STATE]: __isExpandable(),
      "dx-state-invisible": __hidden(),
      "dx-state-hover": __isExpandable() && __state_hovered
    };
    return combineClasses(classesMap);
  }, [props.direction, __state_active, __isExpandable, __hidden, __state_hovered]);

  var __isThumbVisible = useCallback(function __isThumbVisible() {
    if (__hidden()) {
      return false;
    }

    if (__isHoverMode()) {
      return props.visible || __state_hovered || __state_active;
    }

    if (__isAlwaysMode()) {
      return true;
    }

    return props.visible;
  }, [__hidden, __isHoverMode, props.visible, __state_hovered, __state_active, __isAlwaysMode]);

  var __moveTo = useCallback(function __moveTo(location) {
    var _props$scrollLocation;

    var scrollDelta = Math.abs(prevScrollLocation.current - location);
    prevScrollLocation.current = location;
    rightScrollLocation.current = props.maxOffset - location;
    (_props$scrollLocation = props.scrollLocationChange) === null || _props$scrollLocation === void 0 ? void 0 : _props$scrollLocation.call(props, {
      fullScrollProp: __fullScrollProp(),
      location: -location,
      needFireScroll: scrollDelta > 0
    });
  }, [prevScrollLocation, props.maxOffset, props.scrollLocationChange, __fullScrollProp]);

  var __thumbTransform = useCallback(function __thumbTransform() {
    var translateValue = -props.scrollLocation * __scrollRatio();

    if (__isHorizontal()) {
      return "translate(".concat(translateValue, "px, 0px)");
    }

    return "translate(0px, ".concat(translateValue, "px)");
  }, [props.scrollLocation, __scrollRatio, __isHorizontal]);

  var __thumbClasses = useCallback(function __thumbClasses() {
    return combineClasses({
      [SCROLLABLE_SCROLL_CLASS]: true,
      "dx-state-invisible": !__isThumbVisible()
    });
  }, [__isThumbVisible]);

  var __moveToMouseLocation = useCallback(function __moveToMouseLocation(event, offset) {
    var mouseLocation = event["page".concat(__axis().toUpperCase())] - offset;
    var delta = mouseLocation / __containerToContentRatio() - props.containerSize / 2;

    __moveTo(Math.round(-delta));
  }, [__axis, __containerToContentRatio, props.containerSize, __moveTo]);

  var __thumbStyles = useMemo(function __thumbStyles() {
    return {
      [__dimension()]: Math.round(__scrollSize()) || THUMB_MIN_SIZE,
      transform: __isNeverMode() ? "none" : __thumbTransform()
    };
  }, [__dimension, __scrollSize, __isNeverMode, __thumbTransform]);

  useEffect(() => {
    return subscribeToDXPointerDownEvent(__thumbRef.current, () => {
      __state_setActive(__state_active => true);
    });
  }, []);
  useEffect(() => {
    return subscribeToDXPointerUpEvent(domAdapter.getDocument(), () => {
      __state_setActive(__state_active => false);
    });
  }, []);
  useEffect(() => {
    if (__isExpandable()) {
      return subscribeToMouseEnterEvent(__scrollbarRef.current, () => {
        __state_setHovered(__state_hovered => true);
      });
    }

    return undefined;
  }, [__isExpandable]);
  useEffect(() => {
    if (__isExpandable()) {
      return subscribeToMouseLeaveEvent(__scrollbarRef.current, () => {
        __state_setHovered(__state_hovered => false);
      });
    }

    return undefined;
  }, [__isExpandable]);
  useEffect(() => {
    if (props.containerHasSizes) {
      var newScrollLocation = props.scrollLocation;
      var maxOffsetChanged = Math.abs(props.maxOffset - prevMaxOffset.current) > 0;
      prevMaxOffset.current = props.maxOffset;

      if (maxOffsetChanged && __isHorizontal() && config !== null && config !== void 0 && config.rtlEnabled) {
        if (props.maxOffset === 0) {
          rightScrollLocation.current = 0;
        }

        newScrollLocation = props.maxOffset - rightScrollLocation.current;
      }

      if (prevScrollLocation.current !== newScrollLocation) {
        __moveTo(newScrollLocation);
      }
    }
  }, [props.containerHasSizes, props.scrollLocation, props.maxOffset, prevMaxOffset, __isHorizontal, config, rightScrollLocation, prevScrollLocation, __moveTo]);
  useImperativeHandle(ref, () => ({
    isThumb: __isThumb,
    isScrollbar: __isScrollbar,
    setActiveState: __setActiveState,
    moveToMouseLocation: __moveToMouseLocation,
    moveTo: __moveTo
  }), [__isThumb, __isScrollbar, __setActiveState, __moveToMouseLocation, __moveTo]);
  return viewFunction({
    props: _extends({}, props),
    hovered: __state_hovered,
    active: __state_active,
    scrollbarRef: __scrollbarRef,
    scrollRef: __scrollRef,
    thumbRef: __thumbRef,
    config,
    axis: __axis(),
    fullScrollProp: __fullScrollProp(),
    dimension: __dimension(),
    isHorizontal: __isHorizontal(),
    scrollSize: __scrollSize(),
    containerToContentRatio: __containerToContentRatio(),
    scrollRatio: __scrollRatio(),
    scrollbarClasses: __scrollbarClasses(),
    thumbStyles: __thumbStyles,
    thumbTransform: __thumbTransform(),
    thumbClasses: __thumbClasses(),
    hidden: __hidden(),
    isThumbVisible: __isThumbVisible(),
    isExpandable: __isExpandable(),
    isHoverMode: __isHoverMode(),
    isAlwaysMode: __isAlwaysMode(),
    isNeverMode: __isNeverMode(),
    restAttributes: __restAttributes()
  });
});
export { Scrollbar };
export default Scrollbar;
Scrollbar.defaultProps = ScrollbarPropsType;