import 'devextreme/events/gesture/emitter.gesture.scroll';
import { ScrollableSimulatedProps } from "../common/simulated_strategy_props";
import { ScrollOffset, ScrollEventArgs, ScrollableDirection, DxMouseEvent, DxMouseWheelEvent, DxKeyboardEvent, ScrollLocationChangeArgs } from "../common/types";
export declare const viewFunction: (viewModel: ScrollableSimulated) => any;
import * as React from "react";
export declare type ScrollableSimulatedRef = {
    content: () => HTMLDivElement;
    container: () => HTMLDivElement;
    refresh: () => void;
    release: () => void;
    updateHandler: () => void;
    scrollHeight: () => number;
    scrollWidth: () => number;
    scrollOffset: () => ScrollOffset;
    scrollTop: () => number;
    scrollLeft: () => number;
    clientHeight: () => number;
    clientWidth: () => number;
    validate: (event: DxMouseEvent) => boolean;
    moveIsAllowed: (event: DxMouseEvent | DxMouseWheelEvent) => boolean;
    scrollByLocation: (location: ScrollOffset) => void;
};
declare type RestProps = {
    className?: string;
    style?: {
        [name: string]: any;
    };
    key?: any;
    ref?: any;
};
interface ScrollableSimulated {
    props: typeof ScrollableSimulatedProps & RestProps;
    scrollableRef: any;
    topPocketRef: any;
    bottomPocketRef: any;
    wrapperRef: any;
    contentRef: any;
    scrollViewContentRef: any;
    containerRef: any;
    vScrollbarRef: any;
    hScrollbarRef: any;
    active: boolean;
    hovered: boolean;
    scrolling: boolean;
    containerClientWidth: number;
    containerClientHeight: number;
    contentScrollWidth: number;
    contentScrollHeight: number;
    contentClientWidth: number;
    contentClientHeight: number;
    contentPaddingBottom: number;
    topPocketHeight: number;
    bottomPocketHeight: number;
    topPocketState: any;
    isLoadPanelVisible: boolean;
    vScrollLocation: number;
    hScrollLocation: number;
    pulledDown: boolean;
    handleScroll: () => void;
    syncScrollbarsWithContent: () => void;
    startLoading: () => void;
    finishLoading: () => void;
    getEventArgs: () => ScrollEventArgs;
    getInitEventData: () => {
        getDirection: (event: DxMouseWheelEvent) => ScrollableDirection | undefined;
        validate: (event: DxMouseEvent) => boolean;
        isNative: boolean;
        scrollTarget: HTMLDivElement | null;
    };
    onStart: () => void;
    onEnd: (direction: string) => void;
    restoreEndActionDirections: () => void;
    onUpdated: () => void;
    onBounce: () => void;
    onPullDown: () => void;
    onRelease: () => void;
    onReachBottom: () => void;
    scrollLocationChange: (eventData: ScrollLocationChangeArgs) => void;
    hScrollOffsetMax: number;
    vScrollOffsetMax: number;
    vScrollOffsetMin: number;
    onScroll: () => void;
    handleInit: (event: DxMouseEvent) => void;
    handleStart: (event: DxMouseEvent) => void;
    handleMove: (e: DxMouseEvent) => void;
    handleEnd: (event: DxMouseEvent) => void;
    handleStop: () => void;
    handleCancel: (event: DxMouseEvent) => void;
    isCrossThumbScrolling: (event: DxMouseEvent) => boolean;
    adjustDistance: (event: DxMouseEvent, property: "delta" | "velocity") => void;
    suppressDirections: (event: DxMouseEvent) => void;
    validateEvent: (isContent: boolean, isScrollbar: boolean, isThumb: boolean) => boolean;
    prepareDirections: (value: boolean) => void;
    isContent: (element: EventTarget | null) => boolean;
    tryGetAllowedDirection: (event: DxMouseWheelEvent) => ScrollableDirection | undefined;
    isLocked: () => boolean;
    validateWheel: (event: DxMouseWheelEvent) => boolean;
    clearWheelValidationTimer: () => void;
    validateMove: (event: DxMouseEvent) => boolean;
    handleKeyDown: (event: DxKeyboardEvent) => void;
    scrollByLine: (lines: {
        left: number;
        top: number;
    }) => void;
    scrollByPage: (page: number) => void;
    scrollByKey: (key: string) => void;
    lock: () => void;
    unlock: () => void;
    onVisibilityChangeHandler: (visible: boolean) => void;
    updateElementDimensions: () => void;
    setTopPocketDimensions: (topPocketEl: HTMLDivElement) => void;
    setBottomPocketDimensions: (bottomPocketEl: HTMLDivElement) => void;
    setContentDimensions: (contentEl: HTMLDivElement) => void;
    setContainerDimensions: (containerEl: HTMLDivElement) => void;
    contentHeight: number;
    contentWidth: number;
    containerHasSizes: boolean;
    contentStyles: {
        [key: string]: string;
    };
    contentTranslateY: number;
    contentTranslateX: number;
    containerStyles: {
        touchAction: string;
    } | undefined;
    cssClasses: string;
    direction: {
        isVertical: boolean;
        isHorizontal: boolean;
        isBoth: boolean;
    };
    permissibleDirection: ScrollableDirection | undefined;
    isHoverable: boolean;
    restAttributes: RestProps;
}
declare const ScrollableSimulated: React.FC<import("../common/base_scrollable_props").BaseScrollablePropsType & {
    inertiaEnabled: boolean;
    useKeyboard: boolean;
    showScrollbar: import("../common/types").ScrollableShowScrollbar;
    scrollByThumb: any;
    refreshStrategy: import("../common/types").RefreshStrategy;
    onVisibilityChange?: (visible: boolean) => void;
    onStart?: (value: ScrollEventArgs) => void;
    onEnd?: (value: ScrollEventArgs) => void;
    onBounce?: (value: ScrollEventArgs) => void;
    scrollLocationChange?: (value: ScrollLocationChangeArgs) => void;
} & RestProps & {
    ref?: React.Ref<ScrollableSimulatedRef>;
}> & {
    defaultProps: typeof ScrollableSimulatedProps;
};
export { ScrollableSimulated };
export default ScrollableSimulated;
