import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["addWidgetClass", "aria", "bounceEnabled", "children", "classes", "direction", "disabled", "forceGeneratePockets", "height", "inertiaEnabled", "loadPanelComponent", "loadPanelRender", "loadPanelTemplate", "needRenderScrollbars", "needScrollViewContentWrapper", "onBounce", "onEnd", "onPullDown", "onReachBottom", "onScroll", "onStart", "onUpdated", "onVisibilityChange", "pullDownEnabled", "pulledDownText", "pullingDownText", "reachBottomEnabled", "reachBottomText", "refreshStrategy", "refreshingText", "rtlEnabled", "scrollByContent", "scrollByThumb", "scrollLocationChange", "showScrollbar", "useKeyboard", "visible", "width"];
import 'devextreme/events/gesture/emitter.gesture.scroll';
import { subscribeToScrollEvent, subscribeToScrollInitEvent, subscribeToDXScrollStartEvent, subscribeToDXScrollMoveEvent, subscribeToDXScrollEndEvent, subscribeToDXScrollStopEvent, subscribeToDXScrollCancelEvent, subscribeToMouseEnterEvent, subscribeToMouseLeaveEvent, subscribeToDXPointerDownEvent, subscribeToDXPointerUpEvent } from "../../../utils/subscribe_to_event";
import { AnimatedScrollbar } from "../scrollbar/animated_scrollbar";
import { Widget } from "../../common/widget";
import { combineClasses } from "../../../utils/combine_classes";
import { getOffsetDistance } from "../utils/get_offset_distance";
import { getBoundaryProps } from "../utils/get_boundary_props";
import { permissibleWheelDirection } from "../utils/get_permissible_wheel_direction";
import { isDxMouseWheelEvent, normalizeKeyName, isCommandKeyPressed } from 'devextreme/events/utils';
import { isDefined } from 'devextreme/core/utils/type';
import { ScrollableSimulatedProps } from "../common/simulated_strategy_props";
import eventsEngine from 'devextreme/events/core/events_engine';
import { inRange } from 'devextreme/core/utils/math';
import { ScrollDirection } from "../utils/scroll_direction";
import { DIRECTION_VERTICAL, DIRECTION_HORIZONTAL, SCROLLABLE_SIMULATED_CLASS, SCROLLABLE_CONTAINER_CLASS, SCROLLABLE_CONTENT_CLASS, SCROLLABLE_WRAPPER_CLASS, SCROLLVIEW_CONTENT_CLASS, SCROLLABLE_DISABLED_CLASS, SCROLLABLE_SCROLLBARS_ALWAYSVISIBLE, SCROLL_LINE_HEIGHT, SCROLLABLE_SCROLLBAR_CLASS, KEY_CODES, VALIDATE_WHEEL_TIMEOUT, TopPocketState, DIRECTION_BOTH } from "../common/consts";
import { getElementOffset } from "../../../utils/get_element_offset";
import { getElementPadding, getElementOverflowX, getElementOverflowY } from "../utils/get_element_style";
import { TopPocket } from "../internal/pocket/top";
import { BottomPocket } from "../internal/pocket/bottom";
import { getDevicePixelRatio } from "../utils/get_device_pixel_ratio";
import { isElementVisible } from "../utils/is_element_visible";
import { clampIntoRange } from "../utils/clamp_into_range";
import { allowedDirection } from "../utils/get_allowed_direction";
import { subscribeToResize } from "../utils/subscribe_to_resize";
import domAdapter from 'devextreme/core/dom_adapter';
export var viewFunction = viewModel => {
  var {
    active,
    bottomPocketHeight,
    bottomPocketRef,
    containerClientHeight,
    containerClientWidth,
    containerHasSizes,
    containerRef,
    containerStyles,
    contentHeight,
    contentPaddingBottom,
    contentRef,
    contentStyles,
    contentWidth,
    cssClasses,
    direction,
    hScrollLocation,
    hScrollOffsetMax,
    hScrollbarRef,
    handleKeyDown,
    hovered,
    isLoadPanelVisible,
    lock,
    onBounce,
    onEnd,
    onPullDown,
    onReachBottom,
    onVisibilityChangeHandler,
    props: {
      aria,
      bounceEnabled,
      children,
      forceGeneratePockets,
      height,
      inertiaEnabled,
      loadPanelTemplate: LoadPanelTemplate,
      needRenderScrollbars,
      needScrollViewContentWrapper,
      pullDownEnabled,
      pulledDownText,
      pullingDownText,
      reachBottomEnabled,
      reachBottomText,
      refreshStrategy,
      refreshingText,
      rtlEnabled,
      scrollByThumb,
      showScrollbar,
      useKeyboard,
      visible,
      width
    },
    pulledDown,
    restAttributes,
    scrollLocationChange,
    scrollViewContentRef,
    scrollableRef,
    scrolling,
    topPocketRef,
    topPocketState,
    unlock,
    vScrollLocation,
    vScrollOffsetMax,
    vScrollOffsetMin,
    vScrollbarRef,
    wrapperRef
  } = viewModel;
  return React.createElement(Widget, _extends({
    rootElementRef: scrollableRef,
    focusStateEnabled: useKeyboard,
    aria: aria,
    addWidgetClass: false,
    classes: cssClasses,
    rtlEnabled: rtlEnabled,
    height: height,
    width: width,
    visible: visible,
    onVisibilityChange: onVisibilityChangeHandler
  }, restAttributes, {
    onKeyDown: useKeyboard ? handleKeyDown : undefined
  }), React.createElement("div", {
    className: SCROLLABLE_WRAPPER_CLASS,
    ref: wrapperRef
  }, React.createElement("div", {
    className: SCROLLABLE_CONTAINER_CLASS,
    ref: containerRef,
    style: normalizeStyles(containerStyles)
  }, React.createElement("div", {
    className: SCROLLABLE_CONTENT_CLASS,
    ref: contentRef,
    style: normalizeStyles(contentStyles)
  }, forceGeneratePockets && React.createElement(TopPocket, {
    topPocketRef: topPocketRef,
    pullingDownText: pullingDownText,
    pulledDownText: pulledDownText,
    refreshingText: refreshingText,
    refreshStrategy: refreshStrategy,
    pocketState: topPocketState,
    visible: !!pullDownEnabled
  }), needScrollViewContentWrapper ? React.createElement("div", {
    className: SCROLLVIEW_CONTENT_CLASS,
    ref: scrollViewContentRef
  }, children) : children, forceGeneratePockets && React.createElement(BottomPocket, {
    bottomPocketRef: bottomPocketRef,
    reachBottomText: reachBottomText,
    visible: !!reachBottomEnabled
  })), needRenderScrollbars && direction.isHorizontal && React.createElement(AnimatedScrollbar, {
    direction: "horizontal",
    ref: hScrollbarRef,
    contentSize: contentWidth,
    containerSize: containerClientWidth,
    visible: hovered || scrolling || active,
    minOffset: 0,
    maxOffset: hScrollOffsetMax,
    scrollLocation: hScrollLocation,
    scrollLocationChange: scrollLocationChange,
    scrollByThumb: scrollByThumb,
    bounceEnabled: bounceEnabled,
    showScrollbar: showScrollbar,
    inertiaEnabled: inertiaEnabled,
    onBounce: onBounce,
    onEnd: onEnd,
    containerHasSizes: containerHasSizes
  }), needRenderScrollbars && direction.isVertical && React.createElement(AnimatedScrollbar, {
    direction: "vertical",
    ref: vScrollbarRef,
    contentSize: contentHeight,
    containerSize: containerClientHeight,
    visible: hovered || scrolling || active,
    minOffset: vScrollOffsetMin,
    maxOffset: vScrollOffsetMax,
    scrollLocation: vScrollLocation,
    scrollLocationChange: scrollLocationChange,
    scrollByThumb: scrollByThumb,
    bounceEnabled: bounceEnabled,
    showScrollbar: showScrollbar,
    inertiaEnabled: inertiaEnabled,
    onBounce: onBounce,
    onEnd: onEnd,
    containerHasSizes: containerHasSizes,
    forceGeneratePockets: forceGeneratePockets,
    bottomPocketSize: bottomPocketHeight,
    contentPaddingBottom: contentPaddingBottom,
    pulledDown: pulledDown,
    onPullDown: onPullDown,
    onReachBottom: onReachBottom,
    pullDownEnabled: pullDownEnabled,
    reachBottomEnabled: reachBottomEnabled,
    onLock: lock,
    onUnlock: unlock
  }))), viewModel.props.loadPanelTemplate && LoadPanelTemplate({
    targetElement: scrollableRef,
    refreshingText: refreshingText,
    visible: isLoadPanelVisible
  }));
};
import * as React from "react";
import { useState, useCallback, useMemo, useEffect, useRef, useImperativeHandle, forwardRef } from "react";
import { getTemplate } from "@devextreme/runtime/react";
var NUMBER_STYLES = new Set(["animationIterationCount", "borderImageOutset", "borderImageSlice", "border-imageWidth", "boxFlex", "boxFlexGroup", "boxOrdinalGroup", "columnCount", "fillOpacity", "flex", "flexGrow", "flexNegative", "flexOrder", "flexPositive", "flexShrink", "floodOpacity", "fontWeight", "gridColumn", "gridRow", "lineClamp", "lineHeight", "opacity", "order", "orphans", "stopOpacity", "strokeDasharray", "strokeDashoffset", "strokeMiterlimit", "strokeOpacity", "strokeWidth", "tabSize", "widows", "zIndex", "zoom"]);

var isNumeric = value => {
  if (typeof value === "number") return true;
  return !isNaN(Number(value));
};

var getNumberStyleValue = (style, value) => {
  return NUMBER_STYLES.has(style) ? value : "".concat(value, "px");
};

var normalizeStyles = styles => {
  if (!(styles instanceof Object)) return undefined;
  return Object.entries(styles).reduce((result, _ref) => {
    var [key, value] = _ref;
    result[key] = isNumeric(value) ? getNumberStyleValue(key, value) : value;
    return result;
  }, {});
};

var ScrollableSimulated = forwardRef(function scrollableSimulated(props, ref) {
  var __wrapperRef = useRef(null);

  var __contentRef = useRef(null);

  var __scrollViewContentRef = useRef(null);

  var __containerRef = useRef(null);

  var __scrollableRef = useRef(null);

  var __topPocketRef = useRef(null);

  var __bottomPocketRef = useRef(null);

  var __vScrollbarRef = useRef(null);

  var __hScrollbarRef = useRef(null);

  var [__state_active, __state_setActive] = useState(false);
  var [__state_hovered, __state_setHovered] = useState(false);
  var [__state_scrolling, __state_setScrolling] = useState(false);
  var [__state_containerClientWidth, __state_setContainerClientWidth] = useState(0);
  var [__state_containerClientHeight, __state_setContainerClientHeight] = useState(0);
  var [__state_contentScrollWidth, __state_setContentScrollWidth] = useState(0);
  var [__state_contentScrollHeight, __state_setContentScrollHeight] = useState(0);
  var [__state_contentClientWidth, __state_setContentClientWidth] = useState(0);
  var [__state_contentClientHeight, __state_setContentClientHeight] = useState(0);
  var [__state_contentPaddingBottom, __state_setContentPaddingBottom] = useState(0);
  var [__state_topPocketHeight, __state_setTopPocketHeight] = useState(0);
  var [__state_bottomPocketHeight, __state_setBottomPocketHeight] = useState(0);
  var [__state_topPocketState, __state_setTopPocketState] = useState(TopPocketState.STATE_RELEASED);
  var [__state_isLoadPanelVisible, __state_setIsLoadPanelVisible] = useState(false);
  var [__state_vScrollLocation, __state_setVScrollLocation] = useState(0);
  var [__state_hScrollLocation, __state_setHScrollLocation] = useState(0);
  var validateWheelTimer = useRef();
  var locked = useRef(false);
  var loadingIndicatorEnabled = useRef(true);
  var eventForUserAction = useRef();
  var validDirections = useRef({});
  var endActionDirections = useRef({
    horizontal: false,
    vertical: false
  });
  var savedScrollOffset = useRef({
    scrollTop: 0,
    scrollLeft: 0
  });

  var __pulledDown = useCallback(function __pulledDown() {
    return props.pullDownEnabled && props.bounceEnabled && __state_topPocketHeight > 0 && __state_vScrollLocation - __state_topPocketHeight >= 0;
  }, [props.pullDownEnabled, props.bounceEnabled, __state_topPocketHeight, __state_vScrollLocation]);

  var __restoreEndActionDirections = useCallback(function __restoreEndActionDirections() {
    endActionDirections.current[DIRECTION_HORIZONTAL] = false;
    endActionDirections.current[DIRECTION_VERTICAL] = false;
  }, []);

  var __onScroll = useCallback(function __onScroll() {
    eventsEngine.triggerHandler(__containerRef.current, {
      type: "scroll"
    });
  }, []);

  var __handleStop = useCallback(function __handleStop() {
    var _hScrollbarRef$curre, _vScrollbarRef$curre;

    (_hScrollbarRef$curre = __hScrollbarRef.current) === null || _hScrollbarRef$curre === void 0 ? void 0 : _hScrollbarRef$curre.stopHandler();
    (_vScrollbarRef$curre = __vScrollbarRef.current) === null || _vScrollbarRef$curre === void 0 ? void 0 : _vScrollbarRef$curre.stopHandler();
  }, []);

  var __handleCancel = useCallback(function __handleCancel(event) {
    var _hScrollbarRef$curre2, _vScrollbarRef$curre2;

    eventForUserAction.current = event;
    (_hScrollbarRef$curre2 = __hScrollbarRef.current) === null || _hScrollbarRef$curre2 === void 0 ? void 0 : _hScrollbarRef$curre2.endHandler(0, false);
    (_vScrollbarRef$curre2 = __vScrollbarRef.current) === null || _vScrollbarRef$curre2 === void 0 ? void 0 : _vScrollbarRef$curre2.endHandler(0, false);
  }, []);

  var __adjustDistance = useCallback(function __adjustDistance(event, property) {
    var distance = event[property];
    distance.x *= validDirections.current[DIRECTION_HORIZONTAL] ? 1 : 0;
    distance.y *= validDirections.current[DIRECTION_VERTICAL] ? 1 : 0;

    if (isDxMouseWheelEvent(event.originalEvent)) {
      var devicePixelRatio = getDevicePixelRatio();
      distance.x = Math.round(distance.x / devicePixelRatio * 100) / 100;
      distance.y = Math.round(distance.y / devicePixelRatio * 100) / 100;
    }
  }, [validDirections]);

  var __validateEvent = useCallback(function __validateEvent(isContent, isScrollbar, isThumb) {
    return props.scrollByThumb && (isScrollbar || isThumb) || props.scrollByContent && isContent;
  }, [props.scrollByThumb, props.scrollByContent]);

  var __prepareDirections = useCallback(function __prepareDirections(value) {
    validDirections.current[DIRECTION_HORIZONTAL] = value;
    validDirections.current[DIRECTION_VERTICAL] = value;
  }, []);

  var __isContent = useCallback(function __isContent(element) {
    var closest = element.closest(".".concat(SCROLLABLE_SIMULATED_CLASS));

    if (isDefined(closest)) {
      return closest === __scrollableRef.current;
    }

    return false;
  }, []);

  var __isLocked = useCallback(function __isLocked() {
    return locked.current;
  }, [locked]);

  var __clearWheelValidationTimer = useCallback(function __clearWheelValidationTimer() {
    clearTimeout(validateWheelTimer.current);
    validateWheelTimer.current = undefined;
  }, [validateWheelTimer]);

  var __lock = useCallback(function __lock() {
    locked.current = true;
  }, []);

  var __unlock = useCallback(function __unlock() {
    if (!props.disabled) {
      locked.current = false;
    }
  }, [props.disabled]);

  var __setTopPocketDimensions = useCallback(function __setTopPocketDimensions(topPocketEl) {
    __state_setTopPocketHeight(__state_topPocketHeight => props.forceGeneratePockets && props.pullDownEnabled ? topPocketEl.clientHeight : 0);
  }, [props.forceGeneratePockets, props.pullDownEnabled]);

  var __setBottomPocketDimensions = useCallback(function __setBottomPocketDimensions(bottomPocketEl) {
    __state_setBottomPocketHeight(__state_bottomPocketHeight => props.forceGeneratePockets && props.reachBottomEnabled ? bottomPocketEl.clientHeight : 0);
  }, [props.forceGeneratePockets, props.reachBottomEnabled]);

  var __setContentDimensions = useCallback(function __setContentDimensions(contentEl) {
    __state_setContentClientHeight(__state_contentClientHeight => contentEl.clientHeight);

    __state_setContentScrollHeight(__state_contentScrollHeight => contentEl.scrollHeight);

    __state_setContentClientWidth(__state_contentClientWidth => contentEl.clientWidth);

    __state_setContentScrollWidth(__state_contentScrollWidth => contentEl.scrollWidth);

    __state_setContentPaddingBottom(__state_contentPaddingBottom => getElementPadding(__contentRef.current, "bottom"));
  }, []);

  var __setContainerDimensions = useCallback(function __setContainerDimensions(containerEl) {
    __state_setContainerClientHeight(__state_containerClientHeight => containerEl.clientHeight);

    __state_setContainerClientWidth(__state_containerClientWidth => containerEl.clientWidth);
  }, []);

  var __contentHeight = useCallback(function __contentHeight() {
    return getElementOverflowY(__contentRef === null || __contentRef === void 0 ? void 0 : __contentRef.current) === "hidden" ? __state_contentClientHeight : Math.max(__state_contentScrollHeight, __state_contentClientHeight);
  }, [__state_contentClientHeight, __state_contentScrollHeight]);

  var __contentWidth = useCallback(function __contentWidth() {
    return getElementOverflowX(__contentRef === null || __contentRef === void 0 ? void 0 : __contentRef.current) === "hidden" ? __state_contentClientWidth : Math.max(__state_contentScrollWidth, __state_contentClientWidth);
  }, [__state_contentClientWidth, __state_contentScrollWidth]);

  var __containerHasSizes = useCallback(function __containerHasSizes() {
    return __state_containerClientHeight > 0 && __state_containerClientWidth > 0;
  }, [__state_containerClientHeight, __state_containerClientWidth]);

  var __cssClasses = useCallback(function __cssClasses() {
    var {
      classes,
      direction,
      disabled,
      showScrollbar
    } = props;
    var classesMap = {
      "dx-scrollable": true,
      [SCROLLABLE_SIMULATED_CLASS]: true,
      ["dx-scrollable-".concat(direction)]: true,
      [SCROLLABLE_DISABLED_CLASS]: !!disabled,
      [SCROLLABLE_SCROLLBARS_ALWAYSVISIBLE]: showScrollbar === "always",
      [String(classes)]: !!classes
    };
    return combineClasses(classesMap);
  }, [props.classes, props.direction, props.disabled, props.showScrollbar]);

  var __direction = useMemo(function __direction() {
    return new ScrollDirection(props.direction);
  }, [props.direction]);

  var __isHoverable = useCallback(function __isHoverable() {
    return !props.disabled && props.showScrollbar === "onHover";
  }, [props.disabled, props.showScrollbar]);

  var __restAttributes = useCallback(function __restAttributes() {
    var restProps = _objectWithoutPropertiesLoose(props, _excluded);

    return restProps;
  }, [props]);

  var __content = useCallback(function __content() {
    if (props.needScrollViewContentWrapper) {
      return __scrollViewContentRef.current;
    }

    return __contentRef.current;
  }, [props.needScrollViewContentWrapper]);

  var __container = useCallback(function __container() {
    return __containerRef.current;
  }, []);

  var __clientHeight = useCallback(function __clientHeight() {
    return __containerRef.current.clientHeight;
  }, []);

  var __clientWidth = useCallback(function __clientWidth() {
    return __containerRef.current.clientWidth;
  }, []);

  var __startLoading = useCallback(function __startLoading() {
    if (loadingIndicatorEnabled.current && isElementVisible(__scrollableRef.current)) {
      __state_setIsLoadPanelVisible(__state_isLoadPanelVisible => true);
    }

    __lock();
  }, [loadingIndicatorEnabled, __lock]);

  var __finishLoading = useCallback(function __finishLoading() {
    __state_setIsLoadPanelVisible(__state_isLoadPanelVisible => false);

    __unlock();
  }, [__unlock]);

  var __scrollLocationChange = useCallback(function __scrollLocationChange(eventData) {
    if (!isElementVisible(__scrollableRef.current)) {
      return;
    }

    var {
      fullScrollProp,
      location,
      needFireScroll
    } = eventData;
    __containerRef.current[fullScrollProp] = location;

    if (fullScrollProp === "scrollLeft") {
      __state_setHScrollLocation(__state_hScrollLocation => -location);
    } else {
      __state_setVScrollLocation(__state_vScrollLocation => -location);
    }

    var scrollDelta = Math.abs(savedScrollOffset.current[fullScrollProp] - location);
    savedScrollOffset.current[fullScrollProp] = location;

    if (needFireScroll && scrollDelta >= 1) {
      __onScroll();
    }
  }, [savedScrollOffset, __onScroll]);

  var __hScrollOffsetMax = useCallback(function __hScrollOffsetMax() {
    return -Math.max(__contentWidth() - __state_containerClientWidth, 0);
  }, [__contentWidth, __state_containerClientWidth]);

  var __vScrollOffsetMax = useCallback(function __vScrollOffsetMax() {
    return -Math.max(__contentHeight() - __state_containerClientHeight, 0);
  }, [__contentHeight, __state_containerClientHeight]);

  var __vScrollOffsetMin = useCallback(function __vScrollOffsetMin() {
    return __pulledDown() && __state_topPocketState !== TopPocketState.STATE_RELEASED ? __state_topPocketHeight : 0;
  }, [__pulledDown, __state_topPocketState, __state_topPocketHeight]);

  var __handleMove = useCallback(function __handleMove(e) {
    var _e$preventDefault, _hScrollbarRef$curre3, _vScrollbarRef$curre3;

    if (__isLocked()) {
      e.cancel = true;
      return;
    }

    (_e$preventDefault = e.preventDefault) === null || _e$preventDefault === void 0 ? void 0 : _e$preventDefault.call(e);

    __adjustDistance(e, "delta");

    eventForUserAction.current = e;
    var isDxMouseWheel = isDxMouseWheelEvent(e.originalEvent);
    (_hScrollbarRef$curre3 = __hScrollbarRef.current) === null || _hScrollbarRef$curre3 === void 0 ? void 0 : _hScrollbarRef$curre3.moveHandler(e.delta.x, isDxMouseWheel);
    (_vScrollbarRef$curre3 = __vScrollbarRef.current) === null || _vScrollbarRef$curre3 === void 0 ? void 0 : _vScrollbarRef$curre3.moveHandler(e.delta.y, isDxMouseWheel);
  }, [__isLocked, __adjustDistance]);

  var __handleEnd = useCallback(function __handleEnd(event) {
    var _hScrollbarRef$curre4, _vScrollbarRef$curre4;

    __adjustDistance(event, "velocity");

    eventForUserAction.current = event;
    (_hScrollbarRef$curre4 = __hScrollbarRef.current) === null || _hScrollbarRef$curre4 === void 0 ? void 0 : _hScrollbarRef$curre4.endHandler(event.velocity.x, true);
    (_vScrollbarRef$curre4 = __vScrollbarRef.current) === null || _vScrollbarRef$curre4 === void 0 ? void 0 : _vScrollbarRef$curre4.endHandler(event.velocity.y, true);
  }, [__adjustDistance]);

  var __isCrossThumbScrolling = useCallback(function __isCrossThumbScrolling(event) {
    var {
      target
    } = event.originalEvent;
    var verticalScrolling = false;
    var horizontalScrolling = false;

    if (__direction.isVertical) {
      verticalScrolling = props.scrollByThumb && __vScrollbarRef.current.isThumb(target);
    }

    if (__direction.isHorizontal) {
      horizontalScrolling = props.scrollByThumb && __hScrollbarRef.current.isThumb(target);
    }

    return verticalScrolling || horizontalScrolling;
  }, [__direction, props.scrollByThumb]);

  var __suppressDirections = useCallback(function __suppressDirections(event) {
    if (isDxMouseWheelEvent(event.originalEvent)) {
      __prepareDirections(true);

      return;
    }

    __prepareDirections(false);

    var {
      target
    } = event.originalEvent;

    if (__direction.isVertical) {
      var scrollbar = __vScrollbarRef.current;
      validDirections.current[DIRECTION_VERTICAL] = __validateEvent(__isContent(target), scrollbar.isScrollbar(target), scrollbar.isThumb(target));
    }

    if (__direction.isHorizontal) {
      var _scrollbar = __hScrollbarRef.current;
      validDirections.current[DIRECTION_HORIZONTAL] = __validateEvent(__isContent(target), _scrollbar.isScrollbar(target), _scrollbar.isThumb(target));
    }
  }, [__prepareDirections, __direction, __validateEvent, __isContent]);

  var __validateWheel = useCallback(function __validateWheel(event) {
    var scrollbar = permissibleWheelDirection(props.direction, event.shiftKey) === DIRECTION_HORIZONTAL ? __hScrollbarRef.current : __vScrollbarRef.current;
    var reachedMin = scrollbar.reachedMin();
    var reachedMax = scrollbar.reachedMax();
    var contentGreaterThanContainer = !reachedMin || !reachedMax;
    var locatedNotAtBound = !reachedMin && !reachedMax;
    var scrollFromMin = reachedMin && event.delta > 0;
    var scrollFromMax = reachedMax && event.delta < 0;
    var validated = contentGreaterThanContainer && (locatedNotAtBound || scrollFromMin || scrollFromMax);
    validated = validated || validateWheelTimer.current !== undefined;

    if (validated) {
      __clearWheelValidationTimer();

      validateWheelTimer.current = setTimeout(__clearWheelValidationTimer, VALIDATE_WHEEL_TIMEOUT);
    }

    return validated;
  }, [props.direction, validateWheelTimer, __clearWheelValidationTimer]);

  var __updateElementDimensions = useCallback(function __updateElementDimensions() {
    if (props.forceGeneratePockets) {
      __setTopPocketDimensions(__topPocketRef.current);

      __setBottomPocketDimensions(__bottomPocketRef.current);
    }

    __setContentDimensions(__content());

    __setContainerDimensions(__containerRef.current);
  }, [props.forceGeneratePockets, __setTopPocketDimensions, __setBottomPocketDimensions, __setContentDimensions, __content, __setContainerDimensions]);

  var __scrollHeight = useCallback(function __scrollHeight() {
    return __content().offsetHeight;
  }, [__content]);

  var __scrollWidth = useCallback(function __scrollWidth() {
    return __content().offsetWidth;
  }, [__content]);

  var __syncScrollbarsWithContent = useCallback(function __syncScrollbarsWithContent() {
    var {
      scrollLeft,
      scrollTop
    } = __containerRef.current;

    __scrollLocationChange({
      fullScrollProp: "scrollTop",
      location: -clampIntoRange(-scrollTop, 0, __vScrollOffsetMax()),
      needFireScroll: false
    });

    if (!props.rtlEnabled) {
      __scrollLocationChange({
        fullScrollProp: "scrollLeft",
        location: -clampIntoRange(-scrollLeft, 0, __hScrollOffsetMax()),
        needFireScroll: false
      });
    }
  }, [__scrollLocationChange, __vScrollOffsetMax, props.rtlEnabled, __hScrollOffsetMax]);

  var __onPullDown = useCallback(function __onPullDown() {
    var _props$onPullDown;

    __state_setTopPocketState(__state_topPocketState => TopPocketState.STATE_REFRESHING);

    loadingIndicatorEnabled.current = false;

    __startLoading();

    (_props$onPullDown = props.onPullDown) === null || _props$onPullDown === void 0 ? void 0 : _props$onPullDown.call(props, {});
  }, [__startLoading, props.onPullDown]);

  var __onReachBottom = useCallback(function __onReachBottom() {
    var _props$onReachBottom;

    loadingIndicatorEnabled.current = false;

    __startLoading();

    (_props$onReachBottom = props.onReachBottom) === null || _props$onReachBottom === void 0 ? void 0 : _props$onReachBottom.call(props, {});
  }, [__startLoading, props.onReachBottom]);

  var __handleInit = useCallback(function __handleInit(event) {
    var _hScrollbarRef$curre5, _vScrollbarRef$curre5;

    __suppressDirections(event);

    __restoreEndActionDirections();

    eventForUserAction.current = event;

    var crossThumbScrolling = __isCrossThumbScrolling(event);

    var {
      left,
      top
    } = getElementOffset(__scrollableRef.current);
    (_hScrollbarRef$curre5 = __hScrollbarRef.current) === null || _hScrollbarRef$curre5 === void 0 ? void 0 : _hScrollbarRef$curre5.initHandler(event, crossThumbScrolling, left);
    (_vScrollbarRef$curre5 = __vScrollbarRef.current) === null || _vScrollbarRef$curre5 === void 0 ? void 0 : _vScrollbarRef$curre5.initHandler(event, crossThumbScrolling, top);
  }, [__suppressDirections, __restoreEndActionDirections, __isCrossThumbScrolling]);

  var __contentTranslateY = useCallback(function __contentTranslateY() {
    var location = __state_vScrollLocation;
    var transformValue = location % 1;

    var maxOffset = __vScrollOffsetMax() - __state_bottomPocketHeight - __state_contentPaddingBottom;

    if (maxOffset >= 0) {
      return 0;
    }

    if (!props.bounceEnabled || inRange(__state_vScrollLocation, maxOffset, 0)) {
      return -__state_topPocketHeight;
    }

    if (location > 0) {
      transformValue = location;
    }

    if (location < maxOffset) {
      transformValue = location - maxOffset;
    }

    return transformValue - __state_topPocketHeight;
  }, [__state_vScrollLocation, __vScrollOffsetMax, __state_bottomPocketHeight, __state_contentPaddingBottom, props.bounceEnabled, __state_topPocketHeight]);

  var __contentTranslateX = useCallback(function __contentTranslateX() {
    var location = __state_hScrollLocation;
    var transformValue = location % 1;

    if (!props.bounceEnabled || __hScrollOffsetMax() === 0 || inRange(__state_hScrollLocation, __hScrollOffsetMax(), 0)) {
      return 0;
    }

    if (location > 0) {
      transformValue = location;
    }

    if (location < __hScrollOffsetMax()) {
      transformValue = location - __hScrollOffsetMax();
    }

    return transformValue;
  }, [__state_hScrollLocation, props.bounceEnabled, __hScrollOffsetMax]);

  var __permissibleDirection = useCallback(function __permissibleDirection() {
    var {
      bounceEnabled
    } = props;
    return allowedDirection(props.direction, -__vScrollOffsetMax(), -__hScrollOffsetMax(), bounceEnabled);
  }, [props.bounceEnabled, props.direction, __vScrollOffsetMax, __hScrollOffsetMax]);

  var __refresh = useCallback(function __refresh() {
    var _props$onPullDown2;

    __state_setTopPocketState(__state_topPocketState => TopPocketState.STATE_READY);

    __startLoading();

    (_props$onPullDown2 = props.onPullDown) === null || _props$onPullDown2 === void 0 ? void 0 : _props$onPullDown2.call(props, {});
  }, [__startLoading, props.onPullDown]);

  var __scrollOffset = useCallback(function __scrollOffset() {
    var {
      scrollLeft,
      scrollTop
    } = savedScrollOffset.current;
    return {
      top: __vScrollOffsetMax() === 0 ? 0 : scrollTop,
      left: __hScrollOffsetMax() === 0 ? 0 : scrollLeft
    };
  }, [savedScrollOffset, __vScrollOffsetMax, __hScrollOffsetMax]);

  var __getEventArgs = useCallback(function __getEventArgs() {
    var scrollOffset = __scrollOffset();

    return _extends({
      event: eventForUserAction.current,
      scrollOffset
    }, getBoundaryProps(props.direction, scrollOffset, __containerRef.current, __state_topPocketHeight));
  }, [__scrollOffset, eventForUserAction, props.direction, __state_topPocketHeight]);

  var __tryGetAllowedDirection = useCallback(function __tryGetAllowedDirection(event) {
    return isDxMouseWheelEvent(event) ? permissibleWheelDirection(props.direction, event.shiftKey) : __permissibleDirection();
  }, [props.direction, __permissibleDirection]);

  var __validateMove = useCallback(function __validateMove(event) {
    if (!props.scrollByContent && !isDefined(event.target.closest(".".concat(SCROLLABLE_SCROLLBAR_CLASS)))) {
      return false;
    }

    return isDefined(__permissibleDirection());
  }, [props.scrollByContent, __permissibleDirection]);

  var __contentStyles = useMemo(function __contentStyles() {
    return {
      transform: "translate(".concat(__contentTranslateX(), "px, ").concat(__contentTranslateY(), "px)")
    };
  }, [__contentTranslateX, __contentTranslateY]);

  var __containerStyles = useMemo(function __containerStyles() {
    var direction = __permissibleDirection();

    var vDirectionAllowed = direction === DIRECTION_VERTICAL || direction === DIRECTION_BOTH;
    var hDirectionAllowed = direction === DIRECTION_HORIZONTAL || direction === DIRECTION_BOTH;
    var touchDirection = vDirectionAllowed ? "pan-x" : "";
    touchDirection = hDirectionAllowed ? "pan-y" : touchDirection;
    touchDirection = vDirectionAllowed && hDirectionAllowed ? "none" : touchDirection;
    return {
      touchAction: touchDirection
    };
  }, [__permissibleDirection]);

  var __scrollTop = useCallback(function __scrollTop() {
    return __scrollOffset().top;
  }, [__scrollOffset]);

  var __scrollLeft = useCallback(function __scrollLeft() {
    return __scrollOffset().left;
  }, [__scrollOffset]);

  var __handleScroll = useCallback(function __handleScroll() {
    var _props$onScroll;

    if (!__state_scrolling) {
      __syncScrollbarsWithContent();
    }

    (_props$onScroll = props.onScroll) === null || _props$onScroll === void 0 ? void 0 : _props$onScroll.call(props, __getEventArgs());
  }, [__state_scrolling, __syncScrollbarsWithContent, props.onScroll, __getEventArgs]);

  var __onStart = useCallback(function __onStart() {
    var _props$onStart;

    (_props$onStart = props.onStart) === null || _props$onStart === void 0 ? void 0 : _props$onStart.call(props, __getEventArgs());
  }, [props.onStart, __getEventArgs]);

  var __onEnd = useCallback(function __onEnd(direction) {
    if (__direction.isBoth) {
      endActionDirections.current[direction] = true;
      var {
        horizontal,
        vertical
      } = endActionDirections.current;

      if (horizontal && vertical) {
        var _props$onEnd;

        __restoreEndActionDirections();

        __state_setScrolling(__state_scrolling => false);

        (_props$onEnd = props.onEnd) === null || _props$onEnd === void 0 ? void 0 : _props$onEnd.call(props, __getEventArgs());
      }
    } else {
      var _props$onEnd2;

      __state_setScrolling(__state_scrolling => false);

      (_props$onEnd2 = props.onEnd) === null || _props$onEnd2 === void 0 ? void 0 : _props$onEnd2.call(props, __getEventArgs());
    }
  }, [__direction, endActionDirections, __restoreEndActionDirections, props.onEnd, __getEventArgs]);

  var __onUpdated = useCallback(function __onUpdated() {
    var _props$onUpdated;

    (_props$onUpdated = props.onUpdated) === null || _props$onUpdated === void 0 ? void 0 : _props$onUpdated.call(props, __getEventArgs());
  }, [props.onUpdated, __getEventArgs]);

  var __onBounce = useCallback(function __onBounce() {
    var _props$onBounce;

    (_props$onBounce = props.onBounce) === null || _props$onBounce === void 0 ? void 0 : _props$onBounce.call(props, __getEventArgs());
  }, [props.onBounce, __getEventArgs]);

  var __moveIsAllowed = useCallback(function __moveIsAllowed(event) {
    if (props.disabled || isDxMouseWheelEvent(event) && isCommandKeyPressed({
      ctrlKey: event.ctrlKey,
      metaKey: event.metaKey
    })) {
      return false;
    }

    if (props.bounceEnabled) {
      return true;
    }

    return isDxMouseWheelEvent(event) ? __validateWheel(event) : __validateMove(event);
  }, [props.disabled, props.bounceEnabled, __validateWheel, __validateMove]);

  var __handleStart = useCallback(function __handleStart(event) {
    __state_setScrolling(__state_scrolling => true);

    eventForUserAction.current = event;

    __onStart();
  }, [__onStart]);

  var __updateHandler = useCallback(function __updateHandler() {
    __updateElementDimensions();

    __onUpdated();
  }, [__updateElementDimensions, __onUpdated]);

  var __onRelease = useCallback(function __onRelease() {
    __state_setTopPocketState(__state_topPocketState => TopPocketState.STATE_RELEASED);

    loadingIndicatorEnabled.current = true;

    __finishLoading();

    __updateHandler();
  }, [__finishLoading, __updateHandler]);

  var __onVisibilityChangeHandler = useCallback(function __onVisibilityChangeHandler(visible) {
    var _props$onVisibilityCh;

    if (visible) {
      __updateHandler();

      var {
        scrollLeft,
        scrollTop
      } = savedScrollOffset.current;

      __scrollLocationChange({
        fullScrollProp: "scrollTop",
        location: scrollTop,
        needFireScroll: false
      });

      __scrollLocationChange({
        fullScrollProp: "scrollLeft",
        location: scrollLeft,
        needFireScroll: false
      });
    }

    (_props$onVisibilityCh = props.onVisibilityChange) === null || _props$onVisibilityCh === void 0 ? void 0 : _props$onVisibilityCh.call(props, visible);
  }, [__updateHandler, savedScrollOffset, __scrollLocationChange, props.onVisibilityChange]);

  var __validate = useCallback(function __validate(event) {
    if (__isLocked()) {
      return false;
    }

    __updateHandler();

    return __moveIsAllowed(event);
  }, [__isLocked, __updateHandler, __moveIsAllowed]);

  var __scrollByLocation = useCallback(function __scrollByLocation(location) {
    var _hScrollbarRef$curre6, _vScrollbarRef$curre6;

    __updateHandler();

    __state_setScrolling(__state_scrolling => true);

    __prepareDirections(true);

    __onStart();

    var {
      scrollLeft,
      scrollTop
    } = __containerRef.current;
    var {
      left,
      top
    } = location;
    (_hScrollbarRef$curre6 = __hScrollbarRef.current) === null || _hScrollbarRef$curre6 === void 0 ? void 0 : _hScrollbarRef$curre6.scrollTo(scrollLeft + left);
    (_vScrollbarRef$curre6 = __vScrollbarRef.current) === null || _vScrollbarRef$curre6 === void 0 ? void 0 : _vScrollbarRef$curre6.scrollTo(scrollTop + top);

    __state_setScrolling(__state_scrolling => false);
  }, [__updateHandler, __prepareDirections, __onStart]);

  var __getInitEventData = useCallback(function __getInitEventData() {
    return {
      getDirection: event => __tryGetAllowedDirection(event),
      validate: event => __validate(event),
      isNative: false,
      scrollTarget: __containerRef.current
    };
  }, [__tryGetAllowedDirection, __validate]);

  var __scrollByLine = useCallback(function __scrollByLine(lines) {
    var scrollOffset = Math.abs(SCROLL_LINE_HEIGHT / getDevicePixelRatio() * 100) / 100;

    __scrollByLocation({
      top: lines.top * scrollOffset,
      left: lines.left * scrollOffset
    });
  }, [__scrollByLocation]);

  var __scrollByPage = useCallback(function __scrollByPage(page) {
    var distance = {
      left: 0,
      top: 0
    };
    var {
      clientHeight,
      clientWidth
    } = __containerRef.current;

    if (permissibleWheelDirection(props.direction, false) === DIRECTION_VERTICAL) {
      distance.top = page * clientHeight;
    } else {
      distance.left = page * clientWidth;
    }

    __scrollByLocation(distance);
  }, [props.direction, __scrollByLocation]);

  var __scrollByKey = useCallback(function __scrollByKey(key) {
    var {
      scrollLeft,
      scrollTop
    } = __containerRef.current;
    var vOffsetMin = 0;
    var hOffsetMin = 0;
    var vOffsetMax = -__vScrollOffsetMax() + __state_bottomPocketHeight + __state_contentPaddingBottom;
    var hOffsetMax = -__hScrollOffsetMax();
    var offset = getOffsetDistance(key === KEY_CODES.HOME ? {
      top: vOffsetMin,
      left: props.rtlEnabled ? hOffsetMax : hOffsetMin
    } : {
      top: vOffsetMax,
      left: props.rtlEnabled ? hOffsetMin : hOffsetMax
    }, {
      top: scrollTop,
      left: scrollLeft
    });
    var direction = permissibleWheelDirection(props.direction, false);

    __scrollByLocation(direction === DIRECTION_VERTICAL ? {
      top: offset.top,
      left: 0
    } : {
      top: 0,
      left: offset.left
    });
  }, [__vScrollOffsetMax, __state_bottomPocketHeight, __state_contentPaddingBottom, __hScrollOffsetMax, props.rtlEnabled, props.direction, __scrollByLocation]);

  var __release = useCallback(function __release() {
    var _hScrollbarRef$curre7, _vScrollbarRef$curre7;

    __onRelease();

    (_hScrollbarRef$curre7 = __hScrollbarRef.current) === null || _hScrollbarRef$curre7 === void 0 ? void 0 : _hScrollbarRef$curre7.releaseHandler();
    (_vScrollbarRef$curre7 = __vScrollbarRef.current) === null || _vScrollbarRef$curre7 === void 0 ? void 0 : _vScrollbarRef$curre7.releaseHandler();
  }, [__onRelease]);

  var __handleKeyDown = useCallback(function __handleKeyDown(event) {
    if (__state_scrolling) {
      event.originalEvent.stopPropagation();
      event.originalEvent.preventDefault();
      return;
    }

    var isKeySupported = Object.values(KEY_CODES).includes(normalizeKeyName(event));

    if (isKeySupported) {
      event.originalEvent.stopPropagation();
      event.originalEvent.preventDefault();
    }

    switch (normalizeKeyName(event)) {
      case KEY_CODES.DOWN:
        __scrollByLine({
          top: 1,
          left: 0
        });

        break;

      case KEY_CODES.UP:
        __scrollByLine({
          top: -1,
          left: 0
        });

        break;

      case KEY_CODES.RIGHT:
        __scrollByLine({
          top: 0,
          left: 1
        });

        break;

      case KEY_CODES.LEFT:
        __scrollByLine({
          top: 0,
          left: -1
        });

        break;

      case KEY_CODES.PAGE_DOWN:
        __scrollByPage(1);

        break;

      case KEY_CODES.PAGE_UP:
        __scrollByPage(-1);

        break;

      case KEY_CODES.HOME:
        __scrollByKey(KEY_CODES.HOME);

        break;

      case KEY_CODES.END:
        __scrollByKey(KEY_CODES.END);

        break;

      default:
        break;
    }
  }, [__state_scrolling, __scrollByLine, __scrollByPage, __scrollByKey]);

  useEffect(() => {
    return () => __clearWheelValidationTimer();
  }, []);
  useEffect(() => {
    return subscribeToScrollEvent(__containerRef.current, () => {
      __handleScroll();
    });
  }, [__handleScroll]);
  useEffect(() => {
    return subscribeToDXScrollStartEvent(__wrapperRef.current, event => {
      __handleStart(event);
    });
  }, [__handleStart]);
  useEffect(() => {
    return subscribeToScrollInitEvent(__wrapperRef.current, event => {
      __handleInit(event);
    }, __getInitEventData());
  }, [__handleInit, __getInitEventData]);
  useEffect(() => {
    return subscribeToDXScrollMoveEvent(__wrapperRef.current, event => {
      __handleMove(event);
    });
  }, [__handleMove]);
  useEffect(() => {
    return subscribeToDXScrollEndEvent(__wrapperRef.current, event => {
      __handleEnd(event);
    });
  }, [__handleEnd]);
  useEffect(() => {
    return subscribeToDXScrollStopEvent(__wrapperRef.current, () => {
      __handleStop();
    });
  }, [__handleStop]);
  useEffect(() => {
    return subscribeToDXScrollCancelEvent(__wrapperRef.current, event => {
      __handleCancel(event);
    });
  }, [__handleCancel]);
  useEffect(() => {
    return subscribeToDXPointerDownEvent(__wrapperRef.current, () => {
      __state_setActive(__state_active => true);
    });
  }, []);
  useEffect(() => {
    return subscribeToDXPointerUpEvent(domAdapter.getDocument(), () => {
      __state_setActive(__state_active => false);
    });
  }, []);
  useEffect(() => {
    if (__isHoverable()) {
      return subscribeToMouseEnterEvent(__scrollableRef.current, () => {
        __state_setHovered(__state_hovered => true);
      });
    }

    return undefined;
  }, [__isHoverable]);
  useEffect(() => {
    if (__isHoverable()) {
      return subscribeToMouseLeaveEvent(__scrollableRef.current, () => {
        __state_setHovered(__state_hovered => false);
      });
    }

    return undefined;
  }, [__isHoverable]);
  useEffect(() => {
    if (props.disabled) {
      __lock();
    } else {
      __unlock();
    }
  }, [props.disabled, __lock, __unlock]);
  useEffect(() => {
    if (__direction.isBoth) {
      return;
    }

    var inactiveScrollProp = !__direction.isVertical ? "scrollTop" : "scrollLeft";

    __scrollLocationChange({
      fullScrollProp: inactiveScrollProp,
      location: 0,
      needFireScroll: false
    });
  }, [__direction, __scrollLocationChange]);
  useEffect(() => {
    if (props.forceGeneratePockets) {
      __state_setTopPocketState(__state_topPocketState => __pulledDown() ? TopPocketState.STATE_READY : TopPocketState.STATE_RELEASED);
    }
  }, [props.forceGeneratePockets, __pulledDown]);
  useEffect(() => {
    return subscribeToResize(__topPocketRef === null || __topPocketRef === void 0 ? void 0 : __topPocketRef.current, element => {
      __setTopPocketDimensions(element);
    });
  }, []);
  useEffect(() => {
    return subscribeToResize(__bottomPocketRef === null || __bottomPocketRef === void 0 ? void 0 : __bottomPocketRef.current, element => {
      __setBottomPocketDimensions(element);
    });
  }, []);
  useEffect(() => {
    return subscribeToResize(__containerRef.current, element => {
      __setContainerDimensions(element);
    });
  }, []);
  useEffect(() => {
    return subscribeToResize(__content(), element => {
      __setContentDimensions(element);
    });
  }, []);
  useEffect(() => {
    __updateElementDimensions();
  }, []);
  useImperativeHandle(ref, () => ({
    content: __content,
    container: __container,
    refresh: __refresh,
    release: __release,
    updateHandler: __updateHandler,
    scrollHeight: __scrollHeight,
    scrollWidth: __scrollWidth,
    scrollOffset: __scrollOffset,
    scrollTop: __scrollTop,
    scrollLeft: __scrollLeft,
    clientHeight: __clientHeight,
    clientWidth: __clientWidth,
    validate: __validate,
    moveIsAllowed: __moveIsAllowed,
    scrollByLocation: __scrollByLocation
  }), [__content, __container, __refresh, __release, __updateHandler, __scrollHeight, __scrollWidth, __scrollOffset, __scrollTop, __scrollLeft, __clientHeight, __clientWidth, __validate, __moveIsAllowed, __scrollByLocation]);
  return viewFunction({
    props: _extends({}, props, {
      loadPanelTemplate: getTemplate(props.loadPanelTemplate, props.loadPanelRender, props.loadPanelComponent)
    }),
    active: __state_active,
    hovered: __state_hovered,
    scrolling: __state_scrolling,
    containerClientWidth: __state_containerClientWidth,
    containerClientHeight: __state_containerClientHeight,
    contentScrollWidth: __state_contentScrollWidth,
    contentScrollHeight: __state_contentScrollHeight,
    contentClientWidth: __state_contentClientWidth,
    contentClientHeight: __state_contentClientHeight,
    contentPaddingBottom: __state_contentPaddingBottom,
    topPocketHeight: __state_topPocketHeight,
    bottomPocketHeight: __state_bottomPocketHeight,
    topPocketState: __state_topPocketState,
    isLoadPanelVisible: __state_isLoadPanelVisible,
    vScrollLocation: __state_vScrollLocation,
    hScrollLocation: __state_hScrollLocation,
    wrapperRef: __wrapperRef,
    contentRef: __contentRef,
    scrollViewContentRef: __scrollViewContentRef,
    containerRef: __containerRef,
    scrollableRef: __scrollableRef,
    topPocketRef: __topPocketRef,
    bottomPocketRef: __bottomPocketRef,
    vScrollbarRef: __vScrollbarRef,
    hScrollbarRef: __hScrollbarRef,
    pulledDown: __pulledDown(),
    handleScroll: __handleScroll,
    syncScrollbarsWithContent: __syncScrollbarsWithContent,
    startLoading: __startLoading,
    finishLoading: __finishLoading,
    getEventArgs: __getEventArgs,
    getInitEventData: __getInitEventData,
    onStart: __onStart,
    onEnd: __onEnd,
    restoreEndActionDirections: __restoreEndActionDirections,
    onUpdated: __onUpdated,
    onBounce: __onBounce,
    onPullDown: __onPullDown,
    onRelease: __onRelease,
    onReachBottom: __onReachBottom,
    scrollLocationChange: __scrollLocationChange,
    hScrollOffsetMax: __hScrollOffsetMax(),
    vScrollOffsetMax: __vScrollOffsetMax(),
    vScrollOffsetMin: __vScrollOffsetMin(),
    onScroll: __onScroll,
    handleInit: __handleInit,
    handleStart: __handleStart,
    handleMove: __handleMove,
    handleEnd: __handleEnd,
    handleStop: __handleStop,
    handleCancel: __handleCancel,
    isCrossThumbScrolling: __isCrossThumbScrolling,
    adjustDistance: __adjustDistance,
    suppressDirections: __suppressDirections,
    validateEvent: __validateEvent,
    prepareDirections: __prepareDirections,
    isContent: __isContent,
    tryGetAllowedDirection: __tryGetAllowedDirection,
    isLocked: __isLocked,
    validateWheel: __validateWheel,
    clearWheelValidationTimer: __clearWheelValidationTimer,
    validateMove: __validateMove,
    handleKeyDown: __handleKeyDown,
    scrollByLine: __scrollByLine,
    scrollByPage: __scrollByPage,
    scrollByKey: __scrollByKey,
    lock: __lock,
    unlock: __unlock,
    onVisibilityChangeHandler: __onVisibilityChangeHandler,
    updateElementDimensions: __updateElementDimensions,
    setTopPocketDimensions: __setTopPocketDimensions,
    setBottomPocketDimensions: __setBottomPocketDimensions,
    setContentDimensions: __setContentDimensions,
    setContainerDimensions: __setContainerDimensions,
    contentHeight: __contentHeight(),
    contentWidth: __contentWidth(),
    containerHasSizes: __containerHasSizes(),
    contentStyles: __contentStyles,
    contentTranslateY: __contentTranslateY(),
    contentTranslateX: __contentTranslateX(),
    containerStyles: __containerStyles,
    cssClasses: __cssClasses(),
    direction: __direction,
    permissibleDirection: __permissibleDirection(),
    isHoverable: __isHoverable(),
    restAttributes: __restAttributes()
  });
});
export { ScrollableSimulated };
export default ScrollableSimulated;
ScrollableSimulated.defaultProps = ScrollableSimulatedProps;