import { BaseWidgetProps } from "../common/base_props";
import { ItemClickEvent, SelectionChangedEvent } from 'devextreme/ui/button_group';
import { ButtonClickEvent } from 'devextreme/ui/drop_down_button';
import { EventCallback } from "../common/event_callback";
export declare type CollectionWidgetItemType = {
    text?: string;
    disabled?: boolean;
    html?: string;
    visible?: boolean;
};
export declare const CollectionWidgetItem: CollectionWidgetItemType;
export declare type BaseToolbarItemPropsType = {
    rtlEnabled?: boolean | undefined;
};
export declare const BaseToolbarItemProps: BaseToolbarItemPropsType;
export declare type ToolbarTextBoxPropsType = typeof BaseToolbarItemProps & {
    value: string | null;
    onValueChanged?: EventCallback<any>;
};
export declare const ToolbarTextBoxProps: ToolbarTextBoxPropsType;
export declare type ToolbarCheckBoxPropsType = typeof BaseToolbarItemProps & {
    value: boolean | null;
    onValueChanged?: EventCallback<any>;
};
export declare const ToolbarCheckBoxProps: ToolbarCheckBoxPropsType;
export declare type ToolbarWidgetType = "dxButton" | "dxCheckBox" | "dxTextBox" | "dxButtonGroup" | "dxDropDownButton";
export declare type ToolbarShowTextType = "always" | "inMenu";
export declare type ToolbarLocateInMenuType = "always" | "auto" | "never";
export declare type ToolbarLocationType = "after" | "before" | "center";
export interface CollectionItemType {
    text?: string;
    disabled?: boolean;
    html?: string;
    visible?: boolean;
}
export interface ToolbarButtonGroupItemPropsType extends CollectionItemType {
    hint?: string;
    icon?: string;
    type?: ToolbarButtonType;
    elementAttr?: {
        [key: string]: any;
    };
}
export declare type ToolbarButtonGroupSelectionMode = "multiple" | "single";
export declare type ToolbarButtonGroupPropsType = typeof BaseToolbarItemProps & {
    items?: ToolbarButtonGroupItemPropsType[];
    keyExpr?: string;
    onItemClick?: (e: ItemClickEvent) => void;
    onSelectionChanged?: (e: SelectionChangedEvent) => void;
    selectedItemKeys?: any[];
    selectedItems?: any[];
    selectionMode?: ToolbarButtonGroupSelectionMode;
    stylingMode?: ToolbarButtonStylingMode;
};
export declare const ToolbarButtonGroupProps: ToolbarButtonGroupPropsType;
export declare type ToolbarButtonStylingMode = "text" | "outlined" | "contained";
export declare type ToolbarButtonType = "back" | "danger" | "default" | "normal" | "success";
export declare type ToolbarButtonPropsType = typeof BaseToolbarItemProps & {
    text?: string;
    type?: ToolbarButtonType;
    onClick?: () => void;
    stylingMode?: ToolbarButtonStylingMode;
};
export declare const ToolbarButtonProps: ToolbarButtonPropsType;
export declare type ToolbarDropDownButtonItemPropsType = typeof CollectionWidgetItem & {
    badge?: string;
    icon?: string;
    showChevron?: boolean;
    onClick?: () => void;
};
export declare const ToolbarDropDownButtonItemProps: ToolbarDropDownButtonItemPropsType;
export declare type ToolbarDropDownButtonItemPropsTypeType = {
    badge?: string;
    icon?: string;
    showChevron?: boolean;
    onClick?: () => void;
    text?: string;
    disabled?: boolean;
    html?: string;
    visible?: boolean;
};
export declare const ToolbarDropDownButtonItemPropsType: ToolbarDropDownButtonItemPropsTypeType;
export declare type ToolbarDropDownButtonPropsType = typeof BaseToolbarItemProps & {
    dataSource?: (string | typeof ToolbarDropDownButtonItemPropsType)[];
    items?: (string | typeof ToolbarDropDownButtonItemPropsType)[];
    displayExpr?: string;
    dropDownOptions?: any;
    icon?: string;
    keyExpr?: string;
    noDataText?: string;
    onButtonClick?: ((e: ButtonClickEvent) => void) | string;
    onItemClick?: ((e: ItemClickEvent) => void) | string;
    onSelectionChanged?: ((e: SelectionChangedEvent) => void) | string;
    opened?: boolean;
    selectedItem?: string | number | any;
    selectedItemKey?: string | number;
    showArrowIcon?: boolean;
    splitButton?: boolean;
    stylingMode?: ToolbarButtonStylingMode;
    text?: string;
    useSelectMode?: boolean;
    wrapItemText?: boolean;
};
export declare const ToolbarDropDownButtonProps: ToolbarDropDownButtonPropsType;
export declare type ToolbarItemType = typeof CollectionWidgetItem & {
    cssClass?: string;
    locateInMenu?: ToolbarLocateInMenuType;
    location?: ToolbarLocationType;
    widget?: ToolbarWidgetType;
    options?: typeof ToolbarButtonProps | typeof ToolbarButtonGroupProps | typeof ToolbarDropDownButtonProps | typeof ToolbarTextBoxProps | typeof ToolbarCheckBoxProps;
    showText?: ToolbarShowTextType;
};
export declare const ToolbarItem: ToolbarItemType;
export declare type ToolbarItemTypeType = {
    cssClass?: string;
    locateInMenu?: ToolbarLocateInMenuType;
    location?: ToolbarLocationType;
    widget?: ToolbarWidgetType;
    options?: typeof ToolbarButtonProps | typeof ToolbarButtonGroupProps | typeof ToolbarDropDownButtonProps | typeof ToolbarTextBoxProps | typeof ToolbarCheckBoxProps;
    showText?: ToolbarShowTextType;
    text?: string;
    disabled?: boolean;
    html?: string;
    visible?: boolean;
};
export declare const ToolbarItemType: ToolbarItemTypeType;
export declare type ToolbarPropsType = typeof BaseWidgetProps & {
    items?: (string | typeof ToolbarItemType)[];
};
export declare const ToolbarProps: ToolbarPropsType;
